@@ -25,7 +25,7 @@
 #ifdef HAVE_PTHREAD
 /* this include is needed on AIX to have errno stored in thread local storage */
 #include <pthread.h>
-#endif 
+#endif
 
 #include <errno.h>
 #include <netdb.h>
@@ -38,13 +38,14 @@
 #include <arpa/inet.h>
 #include <fcntl.h>
 
-#include "upsclient.h"
 #include "common.h"
+#include "nut_stdint.h"
 #include "timehead.h"
+#include "upsclient.h"
 
 /* WA for Solaris/i386 bug: non-blocking connect sets errno to ENOENT */
-#if (defined NUT_PLATFORM_SOLARIS && CPU_TYPE == i386)
-	#define SOLARIS_i386_NBCONNECT_ENOENT(status) (ENOENT == (status))
+#if (defined NUT_PLATFORM_SOLARIS)
+	#define SOLARIS_i386_NBCONNECT_ENOENT(status) ( (!strcmp("i386", CPU_TYPE)) ? (ENOENT == (status)) : 0 )
 #else
 	#define SOLARIS_i386_NBCONNECT_ENOENT(status) (0)
 #endif  /* end of Solaris/i386 WA for non-blocking connect */
@@ -75,7 +76,7 @@
 #define shutdown_how 2
 #endif
 
-struct {
+static struct {
 	int	flags;
 	const	char	*str;
 } upscli_errlist[] =
@@ -131,7 +132,7 @@ typedef struct HOST_CERT_s {
 	const char	*certname;
 	int			certverify;
 	int			forcessl;
-	
+
 	struct HOST_CERT_s	*next;
 }	HOST_CERT_t;
 static HOST_CERT_t* upscli_find_host_cert(const char* hostname);
@@ -153,7 +154,7 @@ static char* nsscertpasswd = NULL;
 
 static void ssl_debug(void)
 {
-	int	e;
+	unsigned long	e;
 	char	errmsg[SMALLBUF];
 
 	while ((e = ERR_get_error()) != 0) {
@@ -162,32 +163,36 @@ static void ssl_debug(void)
 	}
 }
 
-static int ssl_error(SSL *ssl, int ret)
+static int ssl_error(SSL *ssl, ssize_t ret)
 {
 	int	e;
 
-	e = SSL_get_error(ssl, ret);
+	if (ret >= INT_MAX) {
+		upslogx(LOG_ERR, "ssl_error() ret=%zd would not fit in an int", ret);
+		return -1;
+	}
+	e = SSL_get_error(ssl, (int)ret);
 
 	switch (e)
 	{
 	case SSL_ERROR_WANT_READ:
-		upslogx(LOG_ERR, "ssl_error() ret=%d SSL_ERROR_WANT_READ", ret);
+		upslogx(LOG_ERR, "ssl_error() ret=%zd SSL_ERROR_WANT_READ", ret);
 		break;
 
 	case SSL_ERROR_WANT_WRITE:
-		upslogx(LOG_ERR, "ssl_error() ret=%d SSL_ERROR_WANT_WRITE", ret);
+		upslogx(LOG_ERR, "ssl_error() ret=%zd SSL_ERROR_WANT_WRITE", ret);
 		break;
 
 	case SSL_ERROR_SYSCALL:
 		if (ret == 0 && ERR_peek_error() == 0) {
 			upslogx(LOG_ERR, "ssl_error() EOF from client");
 		} else {
-			upslogx(LOG_ERR, "ssl_error() ret=%d SSL_ERROR_SYSCALL", ret);
+			upslogx(LOG_ERR, "ssl_error() ret=%zd SSL_ERROR_SYSCALL", ret);
 		}
 		break;
 
 	default:
-		upslogx(LOG_ERR, "ssl_error() ret=%d SSL_ERROR %d", ret, e);
+		upslogx(LOG_ERR, "ssl_error() ret=%zd SSL_ERROR %d", ret, e);
 		ssl_debug();
 	}
 
@@ -196,9 +201,12 @@ static int ssl_error(SSL *ssl, int ret)
 
 #elif defined(WITH_NSS) /* WITH_OPENSSL */
 
-static char *nss_password_callback(PK11SlotInfo *slot, PRBool retry, 
+static char *nss_password_callback(PK11SlotInfo *slot, PRBool retry,
 		void *arg)
 {
+	NUT_UNUSED_VARIABLE(retry);
+	NUT_UNUSED_VARIABLE(arg);
+
 	upslogx(LOG_INFO, "Intend to retrieve password for %s / %s: password %sconfigured",
 		PK11_GetSlotName(slot), PK11_GetTokenName(slot), nsscertpasswd?"":"not ");
 	return nsscertpasswd ? PL_strdup(nsscertpasswd) : NULL;
@@ -233,6 +241,10 @@ static SECStatus AuthCertificateDontVerify(CERTCertDBHandle *arg, PRFileDesc *fd
 	PRBool checksig, PRBool isServer)
 {
 	UPSCONN_t *ups   = (UPSCONN_t *)SSL_RevealPinArg(fd);
+	NUT_UNUSED_VARIABLE(arg);
+	NUT_UNUSED_VARIABLE(checksig);
+	NUT_UNUSED_VARIABLE(isServer);
+
 	upslogx(LOG_INFO, "Do not intend to authenticate server %s",
 		ups?ups->host:"<unnamed>");
 	return SECSuccess;
@@ -241,6 +253,8 @@ static SECStatus AuthCertificateDontVerify(CERTCertDBHandle *arg, PRFileDesc *fd
 static SECStatus BadCertHandler(UPSCONN_t *arg, PRFileDesc *fd)
 {
 	HOST_CERT_t* cert;
+	NUT_UNUSED_VARIABLE(fd);
+
 	upslogx(LOG_WARNING, "Certificate validation failed for %s",
 		(arg&&arg->host)?arg->host:"<unnamed>");
 	/* BadCertHandler is called when the NSS certificate validation is failed.
@@ -288,49 +302,63 @@ static SECStatus GetClientAuthData(UPSCONN_t *arg, PRFileDesc *fd,
 
 static void HandshakeCallback(PRFileDesc *fd, UPSCONN_t *client_data)
 {
+	NUT_UNUSED_VARIABLE(fd);
+
 	upslogx(LOG_INFO, "SSL handshake done successfully with server %s",
 		client_data->host);
 }
 
 #endif /* WITH_OPENSSL | WITH_NSS */
 
-int upscli_init(int certverify, const char *certpath, 
+int upscli_init(int certverify, const char *certpath,
 					const char *certname, const char *certpasswd)
 {
 #ifdef WITH_OPENSSL
-	int ret, ssl_mode = SSL_VERIFY_NONE;
-#if OPENSSL_VERSION_NUMBER >= 0x10000000L
-	const SSL_METHOD	*ssl_method;
-#else
-	SSL_METHOD	*ssl_method;
-#endif
+	long ret;
+	int ssl_mode = SSL_VERIFY_NONE;
+	NUT_UNUSED_VARIABLE(certname);
+	NUT_UNUSED_VARIABLE(certpasswd);
 #elif defined(WITH_NSS) /* WITH_OPENSSL */
 	SECStatus	status;
+#else
+	NUT_UNUSED_VARIABLE(certverify);
+	NUT_UNUSED_VARIABLE(certpath);
+	NUT_UNUSED_VARIABLE(certname);
+	NUT_UNUSED_VARIABLE(certpasswd);
 #endif /* WITH_OPENSSL | WITH_NSS */
 
-
 	if (upscli_initialized == 1) {
 		upslogx(LOG_WARNING, "upscli already initialized");
 		return -1;
 	}
-	
+
 #ifdef WITH_OPENSSL
-	
-	SSL_library_init();
+
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
 	SSL_load_error_strings();
+	SSL_library_init();
 
-	ssl_method = TLSv1_client_method();
+	ssl_ctx = SSL_CTX_new(SSLv23_client_method());
+#else
+	ssl_ctx = SSL_CTX_new(TLS_client_method());
+#endif
 
-	if (!ssl_method) {
-		return 0;
-	}
-
-	ssl_ctx = SSL_CTX_new(ssl_method);
 	if (!ssl_ctx) {
 		upslogx(LOG_ERR, "Can not initialize SSL context");
 		return -1;
 	}
-	
+
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+	/* set minimum protocol TLSv1 */
+	SSL_CTX_set_options(ssl_ctx, SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3);
+#else
+	ret = SSL_CTX_set_min_proto_version(ssl_ctx, TLS1_VERSION);
+	if (ret != 1) {
+		upslogx(LOG_ERR, "Can not set minimum protocol to TLSv1");
+		return -1;
+	}
+#endif
+
 	if (!certpath) {
 		if (certverify == 1) {
 			upslogx(LOG_ERR, "Can not verify certificate if any is specified");
@@ -353,13 +381,13 @@ int upscli_init(int certverify, const char *certpath,
 			return -1;
 		}
 
-		SSL_CTX_set_verify(ssl_ctx, ssl_mode, NULL);		
+		SSL_CTX_set_verify(ssl_ctx, ssl_mode, NULL);
 	}
 #elif defined(WITH_NSS) /* WITH_OPENSSL */
 	PR_Init(PR_USER_THREAD, PR_PRIORITY_NORMAL, 0);
-	
+
 	PK11_SetPasswordFunc(nss_password_callback);
-	
+
 	if (certpath) {
 		upslogx(LOG_INFO, "Init SSL with cerificate database located at %s", certpath);
 		status = NSS_Init(certpath);
@@ -372,16 +400,16 @@ int upscli_init(int certverify, const char *certpath,
 		nss_error("upscli_init / NSS_[NoDB]_Init");
 		return -1;
 	}
-	
+
 	status = NSS_SetDomesticPolicy();
 	if (status != SECSuccess) {
 		upslogx(LOG_ERR, "Can not initialize SSL policy");
 		nss_error("upscli_init / NSS_SetDomesticPolicy");
 		return -1;
 	}
-	
+
 	SSL_ClearSessionCache();
-	
+
 	status = SSL_OptionSetDefault(SSL_ENABLE_SSL3, PR_TRUE);
 	if (status != SECSuccess) {
 		upslogx(LOG_ERR, "Can not enable SSLv3");
@@ -407,8 +435,13 @@ int upscli_init(int certverify, const char *certpath,
 		nsscertpasswd = xstrdup(certpasswd);
 	}
 	verify_certificate = certverify;
+#else
+	/* Note: historically we do not return with error here,
+	 * just fall through to below and treat as initialized.
+	 */
+	upslogx(LOG_ERR, "upscli_init called but SSL wasn't compiled in");
 #endif /* WITH_OPENSSL | WITH_NSS */
-	
+
 	upscli_initialized = 1;
 	return 1;
 }
@@ -423,6 +456,11 @@ void upscli_add_host_cert(const char* hostname, const char* certname, int certve
 	cert->certverify = certverify;
 	cert->forcessl = forcessl;
 	first_host_cert = cert;
+#else
+	NUT_UNUSED_VARIABLE(hostname);
+	NUT_UNUSED_VARIABLE(certname);
+	NUT_UNUSED_VARIABLE(certverify);
+	NUT_UNUSED_VARIABLE(forcessl);
 #endif /* WITH_NSS */
 }
 
@@ -438,11 +476,13 @@ static HOST_CERT_t* upscli_find_host_cert(const char* hostname)
 			cert = cert->next;
 		}
 	}
+#else
+	NUT_UNUSED_VARIABLE(hostname);
 #endif /* WITH_NSS */
 	return NULL;
 }
 
-int upscli_cleanup()
+int upscli_cleanup(void)
 {
 #ifdef WITH_OPENSSL
 	if (ssl_ctx) {
@@ -452,7 +492,7 @@ int upscli_cleanup()
 
 #endif /* WITH_OPENSSL */
 
-#ifdef WITH_NSS	
+#ifdef WITH_NSS
 	/* Called to force cache clearing to prevent NSS shutdown failures.
 	 * http://www.mozilla.org/projects/security/pki/nss/ref/ssl/sslfnc.html#1138601
 	 */
@@ -465,7 +505,7 @@ int upscli_cleanup()
 	 */
 	PL_ArenaFinish();
 #endif /* WITH_NSS */
-	
+
 	upscli_initialized = 0;
 	return 1;
 }
@@ -477,6 +517,16 @@ const char *upscli_strerror(UPSCONN_t *ups)
 	char	sslbuf[UPSCLI_ERRBUF_LEN];
 #endif
 
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
+#pragma GCC diagnostic ignored "-Wformat-security"
+#endif
+
 	if (!ups) {
 		return upscli_errlist[UPSCLI_ERR_INVALIDARG].str;
 	}
@@ -517,12 +567,12 @@ const char *upscli_strerror(UPSCONN_t *ups)
 		if (PR_GetErrorTextLength() < UPSCLI_ERRBUF_LEN) {
 			PR_GetErrorText(ups->errbuf);
 		} else {
-			snprintf(ups->errbuf, UPSCLI_ERRBUF_LEN, 
+			snprintf(ups->errbuf, UPSCLI_ERRBUF_LEN,
 				"SSL error #%ld, message too long to be displayed",
 				(long)PR_GetError());
 		}
 #else
-		snprintf(ups->errbuf, UPSCLI_ERRBUF_LEN, 
+		snprintf(ups->errbuf, UPSCLI_ERRBUF_LEN,
 			"SSL error, but SSL wasn't enabled at compile-time");
 #endif	/* WITH_OPENSSL | WITH_NSS */
 		return ups->errbuf;
@@ -534,6 +584,10 @@ const char *upscli_strerror(UPSCONN_t *ups)
 		return ups->errbuf;
 	}
 
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic pop
+#endif
+
 	/* fallthrough */
 
 	snprintf(ups->errbuf, UPSCLI_ERRBUF_LEN, "Unknown error flag %d",
@@ -545,9 +599,9 @@ const char *upscli_strerror(UPSCONN_t *ups)
 /* Read up to buflen bytes from fd and return the number of bytes
    read. If no data is available within d_sec + d_usec, return 0.
    On error, a value < 0 is returned (errno indicates error). */
-static int upscli_select_read(const int fd, void *buf, const size_t buflen, const long d_sec, const long d_usec)
+static ssize_t upscli_select_read(const int fd, void *buf, const size_t buflen, const time_t d_sec, const suseconds_t d_usec)
 {
-	int		ret;
+	ssize_t		ret;
 	fd_set		fds;
 	struct timeval	tv;
 
@@ -566,17 +620,37 @@ static int upscli_select_read(const int fd, void *buf, const size_t buflen, cons
 	return read(fd, buf, buflen);
 }
 
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP_BESIDEFUNC) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS_BESIDEFUNC) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE_BESIDEFUNC) )
+# pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS_BESIDEFUNC
+# pragma GCC diagnostic ignored "-Wtype-limits"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE_BESIDEFUNC
+# pragma GCC diagnostic ignored "-Wtautological-constant-out-of-range-compare"
+#endif
 /* internal: abstract the SSL calls for the other functions */
-static int net_read(UPSCONN_t *ups, char *buf, size_t buflen)
+static ssize_t net_read(UPSCONN_t *ups, char *buf, size_t buflen, const time_t timeout)
 {
-	int	ret;
+	ssize_t	ret = -1;
 
 #ifdef WITH_SSL
 	if (ups->ssl) {
 #ifdef WITH_OPENSSL
-		ret = SSL_read(ups->ssl, buf, buflen);
+		/* SSL_* routines deal with int type for return and buflen
+		 * We might need to window our I/O if we exceed 2GB (in
+		 * 32-bit builds)... Not likely to exceed in 64-bit builds,
+		 * but smaller systems with 16-bits might be endangered :)
+		 */
+		assert(buflen <= INT_MAX);
+		int iret = SSL_read(ups->ssl, buf, (int)buflen);
+		assert(iret <= SSIZE_MAX);
+		ret = (ssize_t)iret;
 #elif defined(WITH_NSS) /* WITH_OPENSSL */
-		ret = PR_Read(ups->ssl, buf, buflen);
+		/* PR_* routines deal in PRInt32 type
+		 * We might need to window our I/O if we exceed 2GB :) */
+		assert(buflen <= PR_INT32_MAX);
+		ret = PR_Read(ups->ssl, buf, (PRInt32)buflen);
 #endif	/* WITH_OPENSSL | WITH_NSS*/
 
 		if (ret < 1) {
@@ -587,7 +661,7 @@ static int net_read(UPSCONN_t *ups, char *buf, size_t buflen)
 	}
 #endif
 
-	ret = upscli_select_read(ups->fd, buf, buflen, 5, 0);
+	ret = upscli_select_read(ups->fd, buf, buflen, timeout, 0);
 
 	/* error reading data, server disconnected? */
 	if (ret < 0) {
@@ -602,13 +676,16 @@ static int net_read(UPSCONN_t *ups, char *buf, size_t buflen)
 
 	return ret;
 }
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP_BESIDEFUNC) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS_BESIDEFUNC) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE_BESIDEFUNC) )
+# pragma GCC diagnostic pop
+#endif
 
 /* Write up to buflen bytes to fd and return the number of bytes
    written. If no data is available within d_sec + d_usec, return 0.
    On error, a value < 0 is returned (errno indicates error). */
-static int upscli_select_write(const int fd, const void *buf, const size_t buflen, const long d_sec, const long d_usec)
+static ssize_t upscli_select_write(const int fd, const void *buf, const size_t buflen, const time_t d_sec, const suseconds_t d_usec)
 {
-	int		ret;
+	ssize_t		ret;
 	fd_set		fds;
 	struct timeval	tv;
 
@@ -627,17 +704,37 @@ static int upscli_select_write(const int fd, const void *buf, const size_t bufle
 	return write(fd, buf, buflen);
 }
 
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP_BESIDEFUNC) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS_BESIDEFUNC) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE_BESIDEFUNC) )
+# pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS_BESIDEFUNC
+# pragma GCC diagnostic ignored "-Wtype-limits"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE_BESIDEFUNC
+# pragma GCC diagnostic ignored "-Wtautological-constant-out-of-range-compare"
+#endif
 /* internal: abstract the SSL calls for the other functions */
-static int net_write(UPSCONN_t *ups, const char *buf, size_t buflen)
+static ssize_t net_write(UPSCONN_t *ups, const char *buf, size_t buflen, const time_t timeout)
 {
-	int	ret;
+	ssize_t	ret = -1;
 
 #ifdef WITH_SSL
 	if (ups->ssl) {
 #ifdef WITH_OPENSSL
-		ret = SSL_write(ups->ssl, buf, buflen);
+		/* SSL_* routines deal with int type for return and buflen
+		 * We might need to window our I/O if we exceed 2GB (in
+		 * 32-bit builds)... Not likely to exceed in 64-bit builds,
+		 * but smaller systems with 16-bits might be endangered :)
+		 */
+		assert(buflen <= INT_MAX);
+		int iret = SSL_write(ups->ssl, buf, (int)buflen);
+		assert(iret <= SSIZE_MAX);
+		ret = (ssize_t)iret;
 #elif defined(WITH_NSS) /* WITH_OPENSSL */
-		ret = PR_Write(ups->ssl, buf, buflen);
+		/* PR_* routines deal in PRInt32 type
+		 * We might need to window our I/O if we exceed 2GB :) */
+		assert(buflen <= PR_INT32_MAX);
+		ret = PR_Write(ups->ssl, buf, (PRInt32)buflen);
 #endif /* WITH_OPENSSL | WITH_NSS */
 
 		if (ret < 1) {
@@ -648,7 +745,7 @@ static int net_write(UPSCONN_t *ups, const char *buf, size_t buflen)
 	}
 #endif
 
-	ret = upscli_select_write(ups->fd, buf, buflen, 0, 0);
+	ret = upscli_select_write(ups->fd, buf, buflen, timeout, 0);
 
 	/* error writing data, server disconnected? */
 	if (ret < 0) {
@@ -663,6 +760,9 @@ static int net_write(UPSCONN_t *ups, const char *buf, size_t buflen)
 
 	return ret;
 }
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP_BESIDEFUNC) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS_BESIDEFUNC) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE_BESIDEFUNC) )
+# pragma GCC diagnostic pop
+#endif
 
 
 #ifdef WITH_SSL
@@ -670,7 +770,7 @@ static int net_write(UPSCONN_t *ups, const char *buf, size_t buflen)
 /*
  * 1 : OK
  * -1 : ERROR
- * 0 : SSL NOT SUPPORTED 
+ * 0 : SSL NOT SUPPORTED
  */
 static int upscli_sslinit(UPSCONN_t *ups, int verifycert)
 {
@@ -679,7 +779,7 @@ static int upscli_sslinit(UPSCONN_t *ups, int verifycert)
 #elif defined(WITH_NSS) /* WITH_OPENSSL */
 	SECStatus	status;
 	PRFileDesc	*socket;
-	HOST_CERT_t *cert; 
+	HOST_CERT_t *cert;
 #endif /* WITH_OPENSSL | WITH_NSS */
 	char	buf[UPSCLI_NETBUF_LEN];
 
@@ -691,7 +791,7 @@ static int upscli_sslinit(UPSCONN_t *ups, int verifycert)
 			"force initialisation without SSL configuration");
 		upscli_init(0, NULL, NULL, NULL);
 	}
-	
+
 	/* see if upsd even talks SSL/TLS */
 	snprintf(buf, sizeof(buf), "STARTTLS\n");
 
@@ -708,7 +808,7 @@ static int upscli_sslinit(UPSCONN_t *ups, int verifycert)
 	}
 
 	/* upsd is happy, so let's crank up the client */
-	
+
 #ifdef WITH_OPENSSL
 
 	if (!ssl_ctx) {
@@ -737,7 +837,7 @@ static int upscli_sslinit(UPSCONN_t *ups, int verifycert)
 	switch(res)
 	{
 	case 1:
-		upsdebugx(3, "SSL connected");
+		upsdebugx(3, "SSL connected (%s)", SSL_get_version(ups->ssl));
 		break;
 	case 0:
 		upslog_with_errno(1, "SSL_connect do not accept handshake.");
@@ -747,7 +847,7 @@ static int upscli_sslinit(UPSCONN_t *ups, int verifycert)
 		upslog_with_errno(1, "Unknown return value from SSL_connect %d", res);
 		ssl_error(ups->ssl, res);
 		return -1;
-	}	
+	}
 
 	return 1;
 
@@ -764,12 +864,12 @@ static int upscli_sslinit(UPSCONN_t *ups, int verifycert)
 		nss_error("upscli_sslinit / SSL_ImportFD");
 		return -1;
 	}
-	
+
 	if (SSL_SetPKCS11PinArg(ups->ssl, ups) == -1){
 		nss_error("upscli_sslinit / SSL_SetPKCS11PinArg");
 		return -1;
 	}
-	
+
 	if (verifycert) {
 		status = SSL_AuthCertificateHook(ups->ssl,
 			(SSLAuthCertificate)AuthCertificate, CERT_GetDefaultCertDB());
@@ -781,19 +881,19 @@ static int upscli_sslinit(UPSCONN_t *ups, int verifycert)
 		nss_error("upscli_sslinit / SSL_AuthCertificateHook");
 		return -1;
 	}
-	
+
 	status = SSL_BadCertHook(ups->ssl, (SSLBadCertHandler)BadCertHandler, ups);
 	if (status != SECSuccess) {
 		nss_error("upscli_sslinit / SSL_BadCertHook");
 		return -1;
 	}
-	
+
 	status = SSL_GetClientAuthDataHook(ups->ssl, (SSLGetClientAuthData)GetClientAuthData, ups);
 	if (status != SECSuccess) {
 		nss_error("upscli_sslinit / SSL_GetClientAuthDataHook");
 		return -1;
 	}
-	
+
 	status = SSL_HandshakeCallback(ups->ssl, (SSLHandshakeCallback)HandshakeCallback, ups);
 	if (status != SECSuccess) {
 		nss_error("upscli_sslinit / SSL_HandshakeCallback");
@@ -830,9 +930,9 @@ static int upscli_sslinit(UPSCONN_t *ups, int verifycert)
 		/* TODO : Close the connection. */
 		return -1;
 	}
-	
+
 	return 1;
-	
+
 #endif /* WITH_OPENSSL | WITH_NSS */
 }
 
@@ -840,12 +940,15 @@ static int upscli_sslinit(UPSCONN_t *ups, int verifycert)
 
 static int upscli_sslinit(UPSCONN_t *ups, int verifycert)
 {
+	NUT_UNUSED_VARIABLE(ups);
+	NUT_UNUSED_VARIABLE(verifycert);
+
 	return 0;	/* not supported */
 }
 
 #endif /* WITH_SSL */
 
-int upscli_tryconnect(UPSCONN_t *ups, const char *host, int port, int flags,struct timeval * timeout)
+int upscli_tryconnect(UPSCONN_t *ups, const char *host, uint16_t port, int flags, struct timeval * timeout)
 {
 	int				sock_fd;
 	struct addrinfo	hints, *res, *ai;
@@ -871,7 +974,7 @@ int upscli_tryconnect(UPSCONN_t *ups, const char *host, int port, int flags,stru
 		return -1;
 	}
 
-	snprintf(sport, sizeof(sport), "%hu", (unsigned short int)port);
+	snprintf(sport, sizeof(sport), "%ju", (uintmax_t)port);
 
 	memset(&hints, 0, sizeof(hints));
 
@@ -1005,7 +1108,7 @@ int upscli_tryconnect(UPSCONN_t *ups, const char *host, int port, int flags,stru
 	ups->port = port;
 
 	hostcert = upscli_find_host_cert(host);
-	
+
 	if (hostcert != NULL) {
 		/* An host security rule is specified. */
 		certverify	= hostcert->certverify;
@@ -1015,39 +1118,39 @@ int upscli_tryconnect(UPSCONN_t *ups, const char *host, int port, int flags,stru
 		forcessl	= (flags & UPSCLI_CONN_REQSSL) != 0 ? 1 : 0;
 	}
 	tryssl = (flags & UPSCLI_CONN_TRYSSL) != 0 ? 1 : 0;
-	
+
 	if (tryssl || forcessl) {
 		ret = upscli_sslinit(ups, certverify);
 		if (forcessl && ret != 1) {
-			upslogx(LOG_ERR, "Can not connect to %s in SSL, disconnect", host);
+			upslogx(LOG_ERR, "Can not connect to NUT server %s in SSL, disconnect", host);
 			ups->upserror = UPSCLI_ERR_SSLFAIL;
 			upscli_disconnect(ups);
 			return -1;
 		} else if (tryssl && ret == -1) {
-			upslogx(LOG_NOTICE, "Error while connecting to %s, disconnect", host);
+			upslogx(LOG_NOTICE, "Error while connecting to NUT server %s, disconnect", host);
 			upscli_disconnect(ups);
 			return -1;
 		} else if (tryssl && ret == 0) {
 			if (certverify != 0) {
-				upslogx(LOG_NOTICE, "Can not connect to %s in SSL and "
+				upslogx(LOG_NOTICE, "Can not connect to NUT server %s in SSL and "
 				"certificate is needed, disconnect", host);
 				upscli_disconnect(ups);
 				return -1;
 			}
-			upsdebugx(3, "Can not connect to %s in SSL, continue uncrypted", host);
+			upsdebugx(3, "Can not connect to NUT server %s in SSL, continue unencrypted", host);
 		} else {
-			upslogx(LOG_INFO, "Connected to %s in SSL", host);
+			upslogx(LOG_INFO, "Connected to NUT server %s in SSL", host);
 			if (certverify == 0) {
 				/* you REALLY should set CERTVERIFY to 1 if using SSL... */
 				upslogx(LOG_WARNING, "Certificate verification is disabled");
 			}
 		}
 	}
-	
+
 	return 0;
 }
 
-int upscli_connect(UPSCONN_t *ups, const char *host, int port, int flags)
+int upscli_connect(UPSCONN_t *ups, const char *host, uint16_t port, int flags)
 {
 	return upscli_tryconnect(ups,host,port,flags,NULL);
 }
@@ -1058,7 +1161,7 @@ static struct {
 	const	char	*text;
 }	upsd_errlist[] =
 {
-	{ UPSCLI_ERR_VARNOTSUPP,	"VAR-NOT-SUPPORTED"	},	
+	{ UPSCLI_ERR_VARNOTSUPP,	"VAR-NOT-SUPPORTED"	},
 	{ UPSCLI_ERR_UNKNOWNUPS,	"UNKNOWN-UPS"		},
 	{ UPSCLI_ERR_ACCESSDENIED, 	"ACCESS-DENIED"		},
 	{ UPSCLI_ERR_PWDREQUIRED,	"PASSWORD-REQUIRED"	},
@@ -1083,7 +1186,7 @@ static struct {
 	{ UPSCLI_ERR_INVPASSWORD,	"INVALID-PASSWORD"	},
 	{ UPSCLI_ERR_USERREQUIRED,	"USERNAME-REQUIRED"	},
 	{ UPSCLI_ERR_DRVNOTCONN,	"DRIVER-NOT-CONNECTED"	},
-	
+
 	{ 0,			NULL,		}
 };
 
@@ -1120,9 +1223,9 @@ static int upscli_errcheck(UPSCONN_t *ups, char *buf)
 }
 
 static void build_cmd(char *buf, size_t bufsize, const char *cmdname,
-	int numarg, const char **arg)
+	size_t numarg, const char **arg)
 {
-	int	i;
+	size_t	i;
 	size_t	len;
 	char	enc[UPSCLI_NETBUF_LEN];
 	const char	*format;
@@ -1142,8 +1245,20 @@ static void build_cmd(char *buf, size_t bufsize, const char *cmdname,
 		/* snprintfcat would tie us to common */
 
 		len = strlen(buf);
-		snprintf(buf + len, bufsize - len, format, 
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
+#pragma GCC diagnostic ignored "-Wformat-security"
+#endif
+		snprintf(buf + len, bufsize - len, format,
 			pconf_encode(arg[i], enc, sizeof(enc)));
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic pop
+#endif
 	}
 
 	len = strlen(buf);
@@ -1151,9 +1266,9 @@ static void build_cmd(char *buf, size_t bufsize, const char *cmdname,
 }
 
 /* make sure upsd is giving us what we asked for */
-static int verify_resp(int num, const char **q, char **a)
+static int verify_resp(size_t num, const char **q, char **a)
 {
-	int	i;
+	size_t	i;
 
 	for (i = 0; i < num; i++) {
 		if (strcasecmp(q[i], a[i]) != 0) {
@@ -1166,11 +1281,11 @@ static int verify_resp(int num, const char **q, char **a)
 	return 1;	/* OK */
 }
 
-int upscli_get(UPSCONN_t *ups, unsigned int numq, const char **query, 
-		unsigned int *numa, char ***answer)
+int upscli_get(UPSCONN_t *ups, size_t numq, const char **query,
+		size_t *numa, char ***answer)
 {
 	char	cmd[UPSCLI_NETBUF_LEN], tmp[UPSCLI_NETBUF_LEN];
-	
+
 	if (!ups) {
 		return -1;
 	}
@@ -1219,7 +1334,7 @@ int upscli_get(UPSCONN_t *ups, unsigned int numq, const char **query,
 	return 0;
 }
 
-int upscli_list_start(UPSCONN_t *ups, unsigned int numq, const char **query)
+int upscli_list_start(UPSCONN_t *ups, size_t numq, const char **query)
 {
 	char	cmd[UPSCLI_NETBUF_LEN], tmp[UPSCLI_NETBUF_LEN];
 
@@ -1277,8 +1392,8 @@ int upscli_list_start(UPSCONN_t *ups, unsigned int numq, const char **query)
 	return 0;
 }
 
-int upscli_list_next(UPSCONN_t *ups, unsigned int numq, const char **query, 
-		unsigned int *numa, char ***answer)
+int upscli_list_next(UPSCONN_t *ups, size_t numq, const char **query,
+		size_t *numa, char ***answer)
 {
 	char	tmp[UPSCLI_NETBUF_LEN];
 
@@ -1326,9 +1441,9 @@ int upscli_list_next(UPSCONN_t *ups, unsigned int numq, const char **query,
 	return 1;
 }
 
-int upscli_sendline(UPSCONN_t *ups, const char *buf, size_t buflen)
+ssize_t upscli_sendline_timeout(UPSCONN_t *ups, const char *buf, size_t buflen, const time_t timeout)
 {
-	int	ret;
+	ssize_t	ret;
 
 	if (!ups) {
 		return -1;
@@ -1349,7 +1464,7 @@ int upscli_sendline(UPSCONN_t *ups, const char *buf, size_t buflen)
 		return -1;
 	}
 
-	ret = net_write(ups, buf, buflen);
+	ret = net_write(ups, buf, buflen, timeout);
 
 	if (ret < 1) {
 		upscli_disconnect(ups);
@@ -1359,9 +1474,14 @@ int upscli_sendline(UPSCONN_t *ups, const char *buf, size_t buflen)
 	return 0;
 }
 
-int upscli_readline(UPSCONN_t *ups, char *buf, size_t buflen)
+ssize_t upscli_sendline(UPSCONN_t *ups, const char *buf, size_t buflen)
 {
-	int	ret;
+	return upscli_sendline_timeout(ups, buf, buflen, 0);
+}
+
+ssize_t upscli_readline_timeout(UPSCONN_t *ups, char *buf, size_t buflen, const time_t timeout)
+{
+	ssize_t	ret;
 	size_t	recv;
 
 	if (!ups) {
@@ -1387,14 +1507,17 @@ int upscli_readline(UPSCONN_t *ups, char *buf, size_t buflen)
 
 		if (ups->readidx == ups->readlen) {
 
-			ret = net_read(ups, ups->readbuf, sizeof(ups->readbuf));
+			ret = net_read(ups, ups->readbuf, sizeof(ups->readbuf), timeout);
 
 			if (ret < 1) {
 				upscli_disconnect(ups);
 				return -1;
 			}
 
-			ups->readlen = ret;
+			/* Here ret is safe to cast since it is >=1 and certainly
+			 * fits under SIZE_MAX being it signed sibling
+			 */
+			ups->readlen = (size_t)ret;
 			ups->readidx = 0;
 		}
 
@@ -1409,8 +1532,13 @@ int upscli_readline(UPSCONN_t *ups, char *buf, size_t buflen)
 	return 0;
 }
 
+ssize_t upscli_readline(UPSCONN_t *ups, char *buf, size_t buflen)
+{
+	return upscli_readline_timeout(ups, buf, buflen, DEFAULT_NETWORK_TIMEOUT);
+}
+
 /* split upsname[@hostname[:port]] into separate components */
-int upscli_splitname(const char *buf, char **upsname, char **hostname, int *port)
+int upscli_splitname(const char *buf, char **upsname, char **hostname, uint16_t *port)
 {
 	char	*s, tmp[SMALLBUF], *last = NULL;
 
@@ -1446,9 +1574,10 @@ int upscli_splitname(const char *buf, char **upsname, char **hostname, int *port
 }
 
 /* split hostname[:port] into separate components */
-int upscli_splitaddr(const char *buf, char **hostname, int *port)
+int upscli_splitaddr(const char *buf, char **hostname, uint16_t *port)
 {
 	char	*s, tmp[SMALLBUF], *last = NULL;
+	long	l;
 
 	/* paranoia */
 	if ((!buf) || (!hostname) || (!port)) {
@@ -1491,10 +1620,13 @@ int upscli_splitaddr(const char *buf, char **hostname, int *port)
 		}
 	}
 
-	if ((*(++s) == '\0') || ((*port = strtol(s, NULL, 10)) < 1 )) {
+	/* Check that "long" port fits in an "uint16_t" so is in IP range
+	 * (under 65536) */
+	if ((*(++s) == '\0') || ((l = strtol(s, NULL, 10)) < 1 ) || (l > 65535)) {
 		fprintf(stderr, "upscli_splitaddr: no port specified after ':' separator\n");
 		return -1;
 	}
+	*port = (uint16_t)l;
 
 	return 0;
 }
@@ -1518,7 +1650,7 @@ int upscli_disconnect(UPSCONN_t *ups)
 		return 0;
 	}
 
-	net_write(ups, "LOGOUT\n", 7);
+	net_write(ups, "LOGOUT\n", 7, 0);
 
 #ifdef WITH_OPENSSL
 	if (ups->ssl) {
