@@ -26,7 +26,7 @@
  * There is now a context buffer, and you call pconf_init to set it up.
  * All subsequent calls must have it as the first argument.  There are
  * two entry points for parsing lines.  You can have it read a file
- * (pconf_file_begin and pconf_file_next), take lines directly from 
+ * (pconf_file_begin and pconf_file_next), take lines directly from
  * the caller (pconf_line), or go along a character at a time (pconf_char).
  * The parsing is identical no matter how you feed it.
  *
@@ -38,7 +38,7 @@
  *
  * Fatal errors are those that involve memory allocation.  If the user
  * defines an error handler when calling pconf_init, that function will
- * be called with the error message before parseconf exits.  By default 
+ * be called with the error message before parseconf exits.  By default
  * it will just write the message to stderr before exiting.
  *
  * Input vs. Output:
@@ -57,12 +57,12 @@
  * also allows you to join lines, allowing you to have logical lines
  * that span physical lines, just like you can do in some shells.
  *
- * Lines normally end with a newline, but reaching EOF will also force 
+ * Lines normally end with a newline, but reaching EOF will also force
  * parsing on what's been scanned so far.
- * 
+ *
  * Design:
  *
- * Characters are read one at a time to drive the state machine.  
+ * Characters are read one at a time to drive the state machine.
  * As words are completed (by hitting whitespace or ending a "" item),
  * they are committed to the next buffer in the arglist.  realloc is
  * used, so the buffer can grow to handle bigger words.
@@ -76,15 +76,20 @@
  *
  */
 
+#include "common.h"
+
 #include <ctype.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
-#include <string.h>	
+#include <string.h>
 #include <unistd.h>
+#include <fcntl.h>
 
 #include "parseconf.h"
+#include "attribute.h"
+#include "nut_stdint.h"
 
 /* possible states */
 
@@ -97,6 +102,9 @@
 #define STATE_ENDOFLINE		7
 #define STATE_PARSEERR		8
 
+static void pconf_fatal(PCONF_CTX_t *ctx, const char *errtxt)
+	__attribute__((noreturn));
+
 static void pconf_fatal(PCONF_CTX_t *ctx, const char *errtxt)
 {
 	if (ctx->errhandler)
@@ -109,7 +117,7 @@ static void pconf_fatal(PCONF_CTX_t *ctx, const char *errtxt)
 
 static void add_arg_word(PCONF_CTX_t *ctx)
 {
-	int	argpos;
+	size_t	argpos;
 	size_t	wbuflen;
 
 	/* this is where the new value goes */
@@ -122,13 +130,13 @@ static void add_arg_word(PCONF_CTX_t *ctx)
 		ctx->maxargs = ctx->numargs;
 
 		/* resize the lists */
-		ctx->arglist = realloc(ctx->arglist, 
+		ctx->arglist = realloc(ctx->arglist,
 			sizeof(char *) * ctx->numargs);
 
 		if (!ctx->arglist)
 			pconf_fatal(ctx, "realloc arglist failed");
 
-		ctx->argsize = realloc(ctx->argsize, 
+		ctx->argsize = realloc(ctx->argsize,
 			sizeof(size_t) * ctx->numargs);
 
 		if (!ctx->argsize)
@@ -171,7 +179,7 @@ static void addchar(PCONF_CTX_t *ctx)
 
 	wbuflen = strlen(ctx->wordbuf);
 
-	/* CVE-2012-2944: only allow the subset Ascii charset from Space to ~ */
+	/* CVE-2012-2944: only allow the subset of ASCII charset from Space to ~ */
 	if ((ctx->ch < 0x20) || (ctx->ch > 0x7f)) {
 		fprintf(stderr, "addchar: discarding invalid character (0x%02x)!\n",
 				ctx->ch);
@@ -199,7 +207,7 @@ static void addchar(PCONF_CTX_t *ctx)
 		ctx->wordptr = &ctx->wordbuf[wbuflen];
 	}
 
-	*ctx->wordptr++ = ctx->ch;
+	*ctx->wordptr++ = (char)ctx->ch;
 	*ctx->wordptr = '\0';
 }
 
@@ -234,8 +242,8 @@ static int findwordstart(PCONF_CTX_t *ctx)
 		return STATE_FINDEOL;
 
 	/* space = not in a word yet, so loop back */
-	if (isspace(ctx->ch))
-		return STATE_FINDWORDSTART;				
+	if (isspace((size_t)ctx->ch))
+		return STATE_FINDWORDSTART;
 
 	/* \ = literal = accept the next char blindly */
 	if (ctx->ch == '\\')
@@ -255,7 +263,7 @@ static int findwordstart(PCONF_CTX_t *ctx)
 	}
 
 	return STATE_COLLECT;
-}	
+}
 
 /* eat characters until the end of the line is found */
 static int findeol(PCONF_CTX_t *ctx)
@@ -274,7 +282,7 @@ static void pconf_seterr(PCONF_CTX_t *ctx, const char *errmsg)
 	snprintf(ctx->errmsg, PCONF_ERR_LEN, "%s", errmsg);
 
 	ctx->error = 1;
-}	
+}
 
 /* quote characters inside a word bounded by "quotes" */
 static int quotecollect(PCONF_CTX_t *ctx)
@@ -291,7 +299,7 @@ static int quotecollect(PCONF_CTX_t *ctx)
 	/* another " means we're done with this word */
 	if (ctx->ch == '"') {
 		endofword(ctx);
-	
+
 		return STATE_FINDWORDSTART;
 	}
 
@@ -334,7 +342,7 @@ static int collect(PCONF_CTX_t *ctx)
 	}
 
 	/* space means the word is done */
-	if (isspace(ctx->ch)) {
+	if (isspace((size_t)ctx->ch)) {
 		endofword(ctx);
 
 		return STATE_FINDWORDSTART;
@@ -443,6 +451,9 @@ int pconf_file_begin(PCONF_CTX_t *ctx, const char *fn)
 		return 0;
 	}
 
+	/* prevent fd leaking to child processes */
+	fcntl(fileno(ctx->f), F_SETFD, FD_CLOEXEC);
+
 	return 1;	/* OK */
 }
 
@@ -571,7 +582,7 @@ int pconf_line(PCONF_CTX_t *ctx, const char *line)
 	/* deal with any lingering characters */
 
 	/* still building a word? */
-	if (ctx->wordptr != ctx->wordbuf)	
+	if (ctx->wordptr != ctx->wordbuf)
 		endofword(ctx);		/* tie it off */
 
 	return 1;
