@@ -35,9 +35,9 @@
  *    original.  Also, there is now a builtin-test, just compile with:
  *           gcc -DTEST_SNPRINTF -o snprintf snprintf.c -lm
  *    and run snprintf for results.
- * 
+ *
  *  Thomas Roessler <roessler@guug.de> 01/27/98 for mutt 0.89i
- *    The PGP code was using unsigned hexadecimal formats. 
+ *    The PGP code was using unsigned hexadecimal formats.
  *    Unfortunately, unsigned formats simply didn't work.
  *
  *  Michael Elkins <me@cs.hmc.edu> 03/05/98 for mutt 0.90.8
@@ -54,7 +54,7 @@
 #include "config.h"
 
 #include <string.h>
-# include <ctype.h>
+#include <ctype.h>
 #include <sys/types.h>
 
 #if !defined(HAVE_SNPRINTF) || !defined(HAVE_VSNPRINTF)
@@ -102,14 +102,14 @@
 /*int snprintf (char *str, size_t count, const char *fmt, ...);*/
 /*int vsnprintf (char *str, size_t count, const char *fmt, va_list arg);*/
 
-static void dopr (char *buffer, size_t maxlen, const char *format, 
+static void dopr (char *buffer, size_t maxlen, const char *format,
                   va_list args);
 static void fmtstr (char *buffer, size_t *currlen, size_t maxlen,
-		    char *value, int flags, int min, int max);
+                    char *value, int flags, int min, int max);
 static void fmtint (char *buffer, size_t *currlen, size_t maxlen,
-		    long value, int base, int min, int max, int flags);
+                    long value, int base, int min, int max, int flags);
 static void fmtfp (char *buffer, size_t *currlen, size_t maxlen,
-		   LDOUBLE fvalue, int min, int max, int flags);
+                   LDOUBLE fvalue, int min, int max, int flags);
 static void dopr_outch (char *buffer, size_t *currlen, size_t maxlen, char c );
 
 /*
@@ -137,10 +137,26 @@ static void dopr_outch (char *buffer, size_t *currlen, size_t maxlen, char c );
 
 /* Conversion Flags */
 #define DP_C_SHORT   1
+/* Note: Originally DP_C_SHORT converted to "short int" types, but modernish
+ * (C99+ or even earlier) standards require that the minimal type passed
+ * through variadic args '...' is an int, and smaller types are padded up
+ * to it - so value shifts in memory and erroneous access crashes can occur
+ * if smaller data is accessed blindly. Code below has been fixed to not pass
+ * "short int" anymore - it just casts the int to desired smaller type (and
+ * so drops the padding bits). */
 #define DP_C_LONG    2
 #define DP_C_LDOUBLE 3
 #define DP_C_LLONG   4
 
+#ifdef C89PLUS
+#undef C89PLUS
+#endif
+
+#if defined(__STDC__) || defined(__STDC_VERSION__)
+/* C89+ and C90+ code respectively */
+#define C89PLUS 1
+#endif
+
 #define char_to_int(p) ((p)- '0')
 #define MAX(p,q) (((p) >= (q)) ? (p) : (q))
 
@@ -156,7 +172,7 @@ static void dopr (char *buffer, size_t maxlen, const char *format, va_list args)
   int flags;
   int cflags;
   size_t currlen;
-  
+
   state = DP_S_DEFAULT;
   currlen = flags = cflags = min = 0;
   max = -1;
@@ -164,237 +180,259 @@ static void dopr (char *buffer, size_t maxlen, const char *format, va_list args)
 
   while (state != DP_S_DONE)
   {
-    if ((ch == '\0') || (currlen >= maxlen)) 
+    if ((ch == '\0') || (currlen >= maxlen))
       state = DP_S_DONE;
 
-    switch(state) 
+    switch(state)
     {
     case DP_S_DEFAULT:
-      if (ch == '%') 
-	state = DP_S_FLAGS;
-      else 
-	dopr_outch (buffer, &currlen, maxlen, ch);
+      if (ch == '%')
+        state = DP_S_FLAGS;
+      else
+        dopr_outch (buffer, &currlen, maxlen, ch);
       ch = *format++;
       break;
     case DP_S_FLAGS:
-      switch (ch) 
+      switch (ch)
       {
       case '-':
-	flags |= DP_F_MINUS;
+        flags |= DP_F_MINUS;
         ch = *format++;
-	break;
+        break;
       case '+':
-	flags |= DP_F_PLUS;
+        flags |= DP_F_PLUS;
         ch = *format++;
-	break;
+        break;
       case ' ':
-	flags |= DP_F_SPACE;
+        flags |= DP_F_SPACE;
         ch = *format++;
-	break;
+        break;
       case '#':
-	flags |= DP_F_NUM;
+        flags |= DP_F_NUM;
         ch = *format++;
-	break;
+        break;
       case '0':
-	flags |= DP_F_ZERO;
+        flags |= DP_F_ZERO;
         ch = *format++;
-	break;
+        break;
       default:
-	state = DP_S_MIN;
-	break;
+        state = DP_S_MIN;
+        break;
       }
       break;
     case DP_S_MIN:
-      if (isdigit((unsigned char)ch)) 
+      if (isdigit((unsigned char)ch))
       {
-	min = 10*min + char_to_int (ch);
-	ch = *format++;
-      } 
-      else if (ch == '*') 
+        min = 10*min + char_to_int (ch);
+        ch = *format++;
+      }
+      else if (ch == '*')
       {
-	min = va_arg (args, int);
-	ch = *format++;
-	state = DP_S_DOT;
-      } 
-      else 
-	state = DP_S_DOT;
+        min = va_arg (args, int);
+        ch = *format++;
+        state = DP_S_DOT;
+      }
+      else
+        state = DP_S_DOT;
       break;
     case DP_S_DOT:
-      if (ch == '.') 
+      if (ch == '.')
       {
-	state = DP_S_MAX;
-	ch = *format++;
-      } 
-      else 
-	state = DP_S_MOD;
+        state = DP_S_MAX;
+        ch = *format++;
+      }
+      else
+        state = DP_S_MOD;
       break;
     case DP_S_MAX:
-      if (isdigit((unsigned char)ch)) 
+      if (isdigit((unsigned char)ch))
       {
-	if (max < 0)
-	  max = 0;
-	max = 10*max + char_to_int (ch);
-	ch = *format++;
-      } 
-      else if (ch == '*') 
+        if (max < 0)
+          max = 0;
+        max = 10*max + char_to_int (ch);
+        ch = *format++;
+      }
+      else if (ch == '*')
       {
-	max = va_arg (args, int);
-	ch = *format++;
-	state = DP_S_MOD;
-      } 
-      else 
-	state = DP_S_MOD;
+        max = va_arg (args, int);
+        ch = *format++;
+        state = DP_S_MOD;
+      }
+      else
+        state = DP_S_MOD;
       break;
     case DP_S_MOD:
-      switch (ch) 
+      switch (ch)
       {
       case 'h':
-	cflags = DP_C_SHORT;
-	ch = *format++;
-	break;
+        cflags = DP_C_SHORT;
+        ch = *format++;
+        break;
       case 'l':
-	cflags = DP_C_LONG;
-	ch = *format++;
-	if (ch == 'l') {	/* It's a long long */
-	  cflags = DP_C_LLONG;
-	  ch = *format++;
-	}
-	break;
+        cflags = DP_C_LONG;
+        ch = *format++;
+        if (ch == 'l') {        /* It's a long long */
+          cflags = DP_C_LLONG;
+          ch = *format++;
+        }
+        break;
       case 'L':
-	cflags = DP_C_LDOUBLE;
-	ch = *format++;
-	break;
+        cflags = DP_C_LDOUBLE;
+        ch = *format++;
+        break;
       default:
-	break;
+        break;
       }
       state = DP_S_CONV;
       break;
     case DP_S_CONV:
-      switch (ch) 
+      switch (ch)
       {
       case 'd':
       case 'i':
-	if (cflags == DP_C_SHORT) 
-	  value = va_arg (args, short int);
-	else if (cflags == DP_C_LONG)
-	  value = va_arg (args, long int);
-	else if (cflags == DP_C_LLONG)
-	  value = va_arg (args, LLONG);
-	else
-	  value = va_arg (args, int);
-	fmtint (buffer, &currlen, maxlen, value, 10, min, max, flags);
-	break;
+        if (cflags == DP_C_SHORT)
+#ifdef C89PLUS
+          value = (short int)va_arg (args, int);
+#else
+          value = va_arg (args, short int);
+#endif
+        else if (cflags == DP_C_LONG)
+          value = va_arg (args, long int);
+        else if (cflags == DP_C_LLONG)
+          value = va_arg (args, LLONG);
+        else
+          value = va_arg (args, int);
+        fmtint (buffer, &currlen, maxlen, value, 10, min, max, flags);
+        break;
       case 'o':
-	flags |= DP_F_UNSIGNED;
-	if (cflags == DP_C_SHORT)
-	  value = va_arg (args, unsigned short int);
-	else if (cflags == DP_C_LONG)
-	  value = (long)va_arg (args, unsigned long int);
-	else if (cflags == DP_C_LLONG)
-	  value = (long)va_arg (args, unsigned LLONG);
-	else
-	  value = (long)va_arg (args, unsigned int);
-	fmtint (buffer, &currlen, maxlen, value, 8, min, max, flags);
-	break;
+        flags |= DP_F_UNSIGNED;
+        if (cflags == DP_C_SHORT)
+#ifdef C89PLUS
+          value = (unsigned short int)va_arg (args, unsigned int);
+#else
+          value = va_arg (args, unsigned short int);
+#endif
+        else if (cflags == DP_C_LONG)
+          value = (long)va_arg (args, unsigned long int);
+        else if (cflags == DP_C_LLONG)
+          value = (long)va_arg (args, unsigned LLONG);
+        else
+          value = (long)va_arg (args, unsigned int);
+        fmtint (buffer, &currlen, maxlen, value, 8, min, max, flags);
+        break;
       case 'u':
-	flags |= DP_F_UNSIGNED;
-	if (cflags == DP_C_SHORT)
-	  value = va_arg (args, unsigned short int);
-	else if (cflags == DP_C_LONG)
-	  value = (long)va_arg (args, unsigned long int);
-	else if (cflags == DP_C_LLONG)
-	  value = (LLONG)va_arg (args, unsigned LLONG);
-	else
-	  value = (long)va_arg (args, unsigned int);
-	fmtint (buffer, &currlen, maxlen, value, 10, min, max, flags);
-	break;
+        flags |= DP_F_UNSIGNED;
+        if (cflags == DP_C_SHORT)
+#ifdef C89PLUS
+          value = (unsigned short int)va_arg (args, unsigned int);
+#else
+          value = va_arg (args, unsigned short int);
+#endif
+        else if (cflags == DP_C_LONG)
+          value = (long)va_arg (args, unsigned long int);
+        else if (cflags == DP_C_LLONG)
+          value = (LLONG)va_arg (args, unsigned LLONG);
+        else
+          value = (long)va_arg (args, unsigned int);
+        fmtint (buffer, &currlen, maxlen, value, 10, min, max, flags);
+        break;
       case 'X':
-	flags |= DP_F_UP;
+        flags |= DP_F_UP;
+        goto fallthrough_case_x;
       case 'x':
-	flags |= DP_F_UNSIGNED;
-	if (cflags == DP_C_SHORT)
-	  value = va_arg (args, unsigned short int);
-	else if (cflags == DP_C_LONG)
-	  value = (long)va_arg (args, unsigned long int);
-	else if (cflags == DP_C_LLONG)
-	  value = (LLONG)va_arg (args, unsigned LLONG);
-	else
-	  value = (long)va_arg (args, unsigned int);
-	fmtint (buffer, &currlen, maxlen, value, 16, min, max, flags);
-	break;
+      fallthrough_case_x:
+        flags |= DP_F_UNSIGNED;
+        if (cflags == DP_C_SHORT)
+#ifdef C89PLUS
+          value = (unsigned short int)va_arg (args, unsigned int);
+#else
+          value = va_arg (args, unsigned short int);
+#endif
+        else if (cflags == DP_C_LONG)
+          value = (long)va_arg (args, unsigned long int);
+        else if (cflags == DP_C_LLONG)
+          value = (LLONG)va_arg (args, unsigned LLONG);
+        else
+          value = (long)va_arg (args, unsigned int);
+        fmtint (buffer, &currlen, maxlen, value, 16, min, max, flags);
+        break;
       case 'f':
-	if (cflags == DP_C_LDOUBLE)
-	  fvalue = va_arg (args, LDOUBLE);
-	else
-	  fvalue = va_arg (args, double);
-	/* um, floating point? */
-	fmtfp (buffer, &currlen, maxlen, fvalue, min, max, flags);
-	break;
+        if (cflags == DP_C_LDOUBLE)
+          fvalue = va_arg (args, LDOUBLE);
+        else
+          fvalue = va_arg (args, double);
+        /* um, floating point? */
+        fmtfp (buffer, &currlen, maxlen, fvalue, min, max, flags);
+        break;
       case 'E':
-	flags |= DP_F_UP;
+        flags |= DP_F_UP;
+        goto fallthrough_case_e;
       case 'e':
-	if (cflags == DP_C_LDOUBLE)
-	  fvalue = va_arg (args, LDOUBLE);
-	else
-	  fvalue = va_arg (args, double);
-	break;
+      fallthrough_case_e:
+        if (cflags == DP_C_LDOUBLE)
+          fvalue = va_arg (args, LDOUBLE);
+        else
+          fvalue = va_arg (args, double);
+        break;
       case 'G':
-	flags |= DP_F_UP;
+        flags |= DP_F_UP;
+        goto fallthrough_case_g;
       case 'g':
-	if (cflags == DP_C_LDOUBLE)
-	  fvalue = va_arg (args, LDOUBLE);
-	else
-	  fvalue = va_arg (args, double);
-	break;
+      fallthrough_case_g:
+        if (cflags == DP_C_LDOUBLE)
+          fvalue = va_arg (args, LDOUBLE);
+        else
+          fvalue = va_arg (args, double);
+        break;
       case 'c':
-	dopr_outch (buffer, &currlen, maxlen, va_arg (args, int));
-	break;
+        dopr_outch (buffer, &currlen, maxlen, va_arg (args, int));
+        break;
       case 's':
-	strvalue = va_arg (args, char *);
-	if (max < 0) 
-	  max = maxlen; /* ie, no max */
-	fmtstr (buffer, &currlen, maxlen, strvalue, flags, min, max);
-	break;
+        strvalue = va_arg (args, char *);
+        if (max < 0)
+          max = maxlen; /* ie, no max */
+        fmtstr (buffer, &currlen, maxlen, strvalue, flags, min, max);
+        break;
       case 'p':
-	strvalue = va_arg (args, void *);
-	fmtint (buffer, &currlen, maxlen, (long) strvalue, 16, min, max, flags);
-	break;
+        strvalue = va_arg (args, void *);
+        fmtint (buffer, &currlen, maxlen, (long) strvalue, 16, min, max, flags);
+        break;
       case 'n':
-	if (cflags == DP_C_SHORT) 
-	{
-	  short int *num;
-	  num = va_arg (args, short int *);
-	  *num = currlen;
-        } 
-	else if (cflags == DP_C_LONG) 
-	{
-	  long int *num;
-	  num = va_arg (args, long int *);
-	  *num = (long int)currlen;
-        } 
-	else if (cflags == DP_C_LLONG) 
-	{
-	  LLONG *num;
-	  num = va_arg (args, LLONG *);
-	  *num = (LLONG)currlen;
-        } 
-	else 
-	{
-	  int *num;
-	  num = va_arg (args, int *);
-	  *num = currlen;
+        if (cflags == DP_C_SHORT)
+        {
+          short int *num;
+          num = va_arg (args, short int *);
+          *num = currlen;
         }
-	break;
+        else if (cflags == DP_C_LONG)
+        {
+          long int *num;
+          num = va_arg (args, long int *);
+          *num = (long int)currlen;
+        }
+        else if (cflags == DP_C_LLONG)
+        {
+          LLONG *num;
+          num = va_arg (args, LLONG *);
+          *num = (LLONG)currlen;
+        }
+        else
+        {
+          int *num;
+          num = va_arg (args, int *);
+          *num = currlen;
+        }
+        break;
       case '%':
-	dopr_outch (buffer, &currlen, maxlen, ch);
-	break;
+        dopr_outch (buffer, &currlen, maxlen, ch);
+        break;
       case 'w':
-	/* not supported yet, treat as next char */
-	ch = *format++;
-	break;
+        /* not supported yet, treat as next char */
+        ch = *format++;
+        break;
       default:
-	/* Unknown, skip */
-	break;
+        /* Unknown, skip */
+        break;
       }
       ch = *format++;
       state = DP_S_DEFAULT;
@@ -408,18 +446,18 @@ static void dopr (char *buffer, size_t maxlen, const char *format, va_list args)
       break; /* some picky compilers need this */
     }
   }
-  if (currlen < maxlen - 1) 
+  if (currlen < maxlen - 1)
     buffer[currlen] = '\0';
-  else 
+  else
     buffer[maxlen - 1] = '\0';
 }
 
 static void fmtstr (char *buffer, size_t *currlen, size_t maxlen,
-		    char *value, int flags, int min, int max)
+                    char *value, int flags, int min, int max)
 {
   int padlen, strln;     /* amount to pad */
   int cnt = 0;
-  
+
   if (value == 0)
   {
     value = "<NULL>";
@@ -427,23 +465,23 @@ static void fmtstr (char *buffer, size_t *currlen, size_t maxlen,
 
   for (strln = 0; value[strln]; ++strln); /* strlen */
   padlen = min - strln;
-  if (padlen < 0) 
+  if (padlen < 0)
     padlen = 0;
-  if (flags & DP_F_MINUS) 
+  if (flags & DP_F_MINUS)
     padlen = -padlen; /* Left Justify */
 
-  while ((padlen > 0) && (cnt < max)) 
+  while ((padlen > 0) && (cnt < max))
   {
     dopr_outch (buffer, currlen, maxlen, ' ');
     --padlen;
     ++cnt;
   }
-  while (*value && (cnt < max)) 
+  while (*value && (cnt < max))
   {
     dopr_outch (buffer, currlen, maxlen, *value++);
     ++cnt;
   }
-  while ((padlen < 0) && (cnt < max)) 
+  while ((padlen < 0) && (cnt < max))
   {
     dopr_outch (buffer, currlen, maxlen, ' ');
     ++padlen;
@@ -454,7 +492,7 @@ static void fmtstr (char *buffer, size_t *currlen, size_t maxlen,
 /* Have to handle DP_F_NUM (ie 0x and 0 alternates) */
 
 static void fmtint (char *buffer, size_t *currlen, size_t maxlen,
-		    long value, int base, int min, int max, int flags)
+                    long value, int base, int min, int max, int flags)
 {
   int signvalue = 0;
   unsigned long uvalue;
@@ -463,7 +501,7 @@ static void fmtint (char *buffer, size_t *currlen, size_t maxlen,
   int spadlen = 0; /* amount to space pad */
   int zpadlen = 0; /* amount to zero pad */
   int caps = 0;
-  
+
   if (max < 0)
     max = 0;
 
@@ -477,12 +515,12 @@ static void fmtint (char *buffer, size_t *currlen, size_t maxlen,
     }
     else
       if (flags & DP_F_PLUS)  /* Do a sign (+/i) */
-	signvalue = '+';
+        signvalue = '+';
     else
       if (flags & DP_F_SPACE)
-	signvalue = ' ';
+        signvalue = ' ';
   }
-  
+
   if (flags & DP_F_UP) caps = 1; /* Should characters be upper case? */
 
   do {
@@ -503,27 +541,27 @@ static void fmtint (char *buffer, size_t *currlen, size_t maxlen,
     zpadlen = MAX(zpadlen, spadlen);
     spadlen = 0;
   }
-  if (flags & DP_F_MINUS) 
+  if (flags & DP_F_MINUS)
     spadlen = -spadlen; /* Left Justifty */
 
 #ifdef DEBUG_SNPRINTF
   printf("zpad: %d, spad: %d, min: %d, max: %d, place: %d\n",
-	 zpadlen, spadlen, min, max, place);
+         zpadlen, spadlen, min, max, place);
 #endif
 
   /* Spaces */
-  while (spadlen > 0) 
+  while (spadlen > 0)
   {
     dopr_outch (buffer, currlen, maxlen, ' ');
     --spadlen;
   }
 
   /* Sign */
-  if (signvalue) 
+  if (signvalue)
     dopr_outch (buffer, currlen, maxlen, signvalue);
 
   /* Zeros */
-  if (zpadlen > 0) 
+  if (zpadlen > 0)
   {
     while (zpadlen > 0)
     {
@@ -533,9 +571,9 @@ static void fmtint (char *buffer, size_t *currlen, size_t maxlen,
   }
 
   /* Digits */
-  while (place > 0) 
+  while (place > 0)
     dopr_outch (buffer, currlen, maxlen, convert[--place]);
-  
+
   /* Left Justified spaces */
   while (spadlen < 0) {
     dopr_outch (buffer, currlen, maxlen, ' ');
@@ -543,6 +581,7 @@ static void fmtint (char *buffer, size_t *currlen, size_t maxlen,
   }
 }
 
+#ifndef HAVE_ABS_VAL
 static LDOUBLE abs_val (LDOUBLE value)
 {
   LDOUBLE result = value;
@@ -552,7 +591,13 @@ static LDOUBLE abs_val (LDOUBLE value)
 
   return result;
 }
+#endif
 
+#ifndef HAVE_FCVT
+/* The two routines that may get defined below are only used if we also don't
+ * have a fcvt() in the system. Defining and not using the routines may be a
+ * warning (fatal with -Werror), so we hide them here. */
+# ifndef HAVE_POW10
 static LDOUBLE pow10 (int exp)
 {
   LDOUBLE result = 1;
@@ -562,10 +607,12 @@ static LDOUBLE pow10 (int exp)
     result *= 10;
     exp--;
   }
-  
+
   return result;
 }
+# endif
 
+# ifndef HAVE_ROUND
 static long round (LDOUBLE value)
 {
   long intpart;
@@ -577,9 +624,11 @@ static long round (LDOUBLE value)
 
   return intpart;
 }
+# endif
+#endif /* HAVE_FCVT */
 
 static void fmtfp (char *buffer, size_t *currlen, size_t maxlen,
-		   LDOUBLE fvalue, int min, int max, int flags)
+                   LDOUBLE fvalue, int min, int max, int flags)
 {
   int signvalue = 0;
   LDOUBLE ufvalue;
@@ -601,12 +650,14 @@ static void fmtfp (char *buffer, size_t *currlen, size_t maxlen,
   int iplace = 0;
   int fplace = 0;
   int padlen = 0; /* amount to pad */
-  int zpadlen = 0; 
+  int zpadlen = 0;
+#ifndef HAVE_FCVT
   int caps = 0;
   long intpart;
   long fracpart;
-  
-  /* 
+#endif
+
+  /*
    * AIX manpage says the default is 0, but Solaris says the default
    * is 6, and sprintf on AIX defaults to 6
    */
@@ -622,7 +673,7 @@ static void fmtfp (char *buffer, size_t *currlen, size_t maxlen,
       signvalue = '+';
     else
       if (flags & DP_F_SPACE)
-	signvalue = ' ';
+        signvalue = ' ';
 
 #if 0
   if (flags & DP_F_UP) caps = 1; /* Should characters be upper case? */
@@ -631,8 +682,8 @@ static void fmtfp (char *buffer, size_t *currlen, size_t maxlen,
 #ifndef HAVE_FCVT
   intpart = (long)ufvalue;
 
-  /* 
-   * Sorry, we only support 9 digits past the decimal because of our 
+  /*
+   * Sorry, we only support 9 digits past the decimal because of our
    * conversion method
    */
   if (max > 9)
@@ -650,8 +701,8 @@ static void fmtfp (char *buffer, size_t *currlen, size_t maxlen,
   }
 
 #ifdef DEBUG_SNPRINTF
-  printf("fmtfp: %g %d.%d min=%d max=%d\n", 
-	 (double)fvalue, intpart, fracpart, min, max);
+  printf("fmtfp: %g %d.%d min=%d max=%d\n",
+         (double)fvalue, intpart, fracpart, min, max);
 #endif
 
   /* Convert integer part */
@@ -707,38 +758,38 @@ static void fmtfp (char *buffer, size_t *currlen, size_t maxlen,
       fconvert[fplace++] = result[--r_length];
 
     while ((dec_pt < 0) && (fplace < max)) {
-		fconvert[fplace++] = '0';
-		dec_pt++;
-	}
+                fconvert[fplace++] = '0';
+                dec_pt++;
+        }
   } else {
     int c;
 
     iplace=0;
     for(c=dec_pt; c; iconvert[iplace++] = result[--c])
-		;
+                ;
     iconvert[iplace] = '\0';
 
     result += dec_pt;
     fplace = 0;
-    
+
     for(c=(r_length-dec_pt); c; fconvert[fplace++] = result[--c])
-		;
+                ;
   }
 #endif /* fcvt */
-  
+
   /* -1 for decimal point, another -1 if we are printing a sign */
-  padlen = min - iplace - max - 1 - ((signvalue) ? 1 : 0); 
+  padlen = min - iplace - max - 1 - ((signvalue) ? 1 : 0);
   zpadlen = max - fplace;
   if (zpadlen < 0)
     zpadlen = 0;
-  if (padlen < 0) 
+  if (padlen < 0)
     padlen = 0;
-  if (flags & DP_F_MINUS) 
+  if (flags & DP_F_MINUS)
     padlen = -padlen; /* Left Justifty */
 
-  if ((flags & DP_F_ZERO) && (padlen > 0)) 
+  if ((flags & DP_F_ZERO) && (padlen > 0))
   {
-    if (signvalue) 
+    if (signvalue)
     {
       dopr_outch (buffer, currlen, maxlen, signvalue);
       --padlen;
@@ -755,10 +806,10 @@ static void fmtfp (char *buffer, size_t *currlen, size_t maxlen,
     dopr_outch (buffer, currlen, maxlen, ' ');
     --padlen;
   }
-  if (signvalue) 
+  if (signvalue)
     dopr_outch (buffer, currlen, maxlen, signvalue);
 
-  while (iplace > 0) 
+  while (iplace > 0)
     dopr_outch (buffer, currlen, maxlen, iconvert[--iplace]);
 
 
@@ -771,10 +822,10 @@ static void fmtfp (char *buffer, size_t *currlen, size_t maxlen,
    * char to print out.
    */
   if (max > 0) {
-	  dopr_outch (buffer, currlen, maxlen, '.');
+          dopr_outch (buffer, currlen, maxlen, '.');
 
-	  while (fplace > 0) 
-		  dopr_outch (buffer, currlen, maxlen, fconvert[--fplace]);
+          while (fplace > 0)
+                  dopr_outch (buffer, currlen, maxlen, fconvert[--fplace]);
   }
 
   while (zpadlen > 0)
@@ -783,7 +834,7 @@ static void fmtfp (char *buffer, size_t *currlen, size_t maxlen,
     --zpadlen;
   }
 
-  while (padlen < 0) 
+  while (padlen < 0)
   {
     dopr_outch (buffer, currlen, maxlen, ' ');
     ++padlen;
@@ -820,7 +871,7 @@ static void dopr_outch (char *buffer, size_t *currlen, size_t maxlen, char c)
   char *fmt;
 #endif
   VA_LOCAL_DECL;
-    
+
   VA_START (fmt);
   VA_SHIFT (str, char *);
   VA_SHIFT (count, size_t );
@@ -860,7 +911,7 @@ static void dopr_outch (char *buffer, size_t *currlen, size_t maxlen, char c)
     "%.1f",
     NULL
   };
-  double fp_nums[] = { -1.5, 134.21, 91340.2, 341.1234, 0203.9, 0.96, 0.996, 
+  double fp_nums[] = { -1.5, 134.21, 91340.2, 341.1234, 0203.9, 0.96, 0.996,
     0.9996, 1.996, 4.136, 6442452944.1234, 0};
   char *int_fmt[] = {
     "%-1.5d",
@@ -888,9 +939,9 @@ static void dopr_outch (char *buffer, size_t *currlen, size_t maxlen, char c)
       sprintf (buf2, fp_fmt[x], fp_nums[y]);
       if (strcmp (buf1, buf2))
       {
-	printf("snprintf doesn't match Format: %s\n\tsnprintf = %s\n\tsprintf  = %s\n", 
-	    fp_fmt[x], buf1, buf2);
-	fail++;
+        printf("snprintf doesn't match Format: %s\n\tsnprintf = %s\n\tsprintf  = %s\n",
+            fp_fmt[x], buf1, buf2);
+        fail++;
       }
       num++;
     }
@@ -902,9 +953,9 @@ static void dopr_outch (char *buffer, size_t *currlen, size_t maxlen, char c)
       sprintf (buf2, int_fmt[x], int_nums[y]);
       if (strcmp (buf1, buf2))
       {
-	printf("snprintf doesn't match Format: %s\n\tsnprintf = %s\n\tsprintf  = %s\n", 
-	    int_fmt[x], buf1, buf2);
-	fail++;
+        printf("snprintf doesn't match Format: %s\n\tsnprintf = %s\n\tsprintf  = %s\n",
+            int_fmt[x], buf1, buf2);
+        fail++;
       }
       num++;
     }
