@@ -1,7 +1,7 @@
 /*  belkin-hid.c - data to monitor Belkin UPS Systems USB/HID devices with NUT
  *
  *  Copyright (C)
- *	2003 - 2008	Arnaud Quette <arnaud.quette@free.fr>
+ *  2003 - 2008 Arnaud Quette <arnaud.quette@free.fr>
  *  2005        Peter Selinger <selinger@users.sourceforge.net>
  *  2011, 2014  Charles Lepple <clepple+nut@gmail>
  *
@@ -29,7 +29,9 @@
 #include "belkin-hid.h"
 #include "usb-common.h"
 
-#define BELKIN_HID_VERSION      "Belkin/Liebert HID 0.17"
+#include <math.h>     /* for fabs() */
+
+#define BELKIN_HID_VERSION      "Belkin/Liebert HID 0.18"
 
 /* Belkin */
 #define BELKIN_VENDORID	0x050d
@@ -65,6 +67,8 @@ static usb_device_id_t belkin_usb_device_table[] = {
 	/* F6C1100-UNV, F6C1200-UNV */
 	{ USB_DEVICE(BELKIN_VENDORID, 0x1100), NULL },
 
+	/* Liebert GXT4 UPS */
+	{ USB_DEVICE(LIEBERT_VENDORID, 0x0004), NULL },
 	/* Liebert PowerSure PSA UPS */
 	{ USB_DEVICE(LIEBERT_VENDORID, 0x0001), NULL },
 	/* Liebert PowerSure PSI 1440 */
@@ -73,7 +77,7 @@ static usb_device_id_t belkin_usb_device_table[] = {
 	{ USB_DEVICE(LIEBERT_VENDORID, 0x0008), NULL },
 
 	/* Terminating entry */
-	{ -1, -1, NULL }
+	{ 0, 0, NULL }
 };
 
 static const char *liebert_online_fun(double value);
@@ -86,35 +90,35 @@ static const char *liebert_config_voltage_fun(double value);
 static const char *liebert_line_voltage_fun(double value);
 
 static info_lkp_t liebert_online_info[] = {
-	{ 0, NULL, liebert_online_fun }
+	{ 0, NULL, liebert_online_fun, NULL }
 };
 
 static info_lkp_t liebert_discharging_info[] = {
-        { 0, NULL, liebert_discharging_fun }
+        { 0, NULL, liebert_discharging_fun, NULL }
 };
 
 static info_lkp_t liebert_charging_info[] = {
-        { 0, NULL, liebert_charging_fun }
+        { 0, NULL, liebert_charging_fun, NULL }
 };
 
 static info_lkp_t liebert_lowbatt_info[] = {
-        { 0, NULL, liebert_lowbatt_fun }
+        { 0, NULL, liebert_lowbatt_fun, NULL }
 };
 
 static info_lkp_t liebert_replacebatt_info[] = {
-        { 0, NULL, liebert_replacebatt_fun }
+        { 0, NULL, liebert_replacebatt_fun, NULL }
 };
 
 static info_lkp_t liebert_shutdownimm_info[] = {
-        { 0, NULL, liebert_shutdownimm_fun }
+        { 0, NULL, liebert_shutdownimm_fun, NULL }
 };
 
 static info_lkp_t liebert_config_voltage_info[] = {
-	{ 0, NULL, liebert_config_voltage_fun },
+	{ 0, NULL, liebert_config_voltage_fun, NULL },
 };
 
 static info_lkp_t liebert_line_voltage_info[] = {
-	{ 0, NULL, liebert_line_voltage_fun },
+	{ 0, NULL, liebert_line_voltage_fun, NULL },
 };
 
 static double liebert_config_voltage_mult = 1.0;
@@ -161,7 +165,7 @@ static const char *liebert_shutdownimm_fun(double value)
 static const char *liebert_config_voltage_fun(double value)
 {
 	if( value < 1 ) {
-		if( abs(value - 1e-7) < 1e-9 ) {
+		if( fabs(value - 1e-7) < 1e-9 ) {
 			liebert_config_voltage_mult = 1e8;
 			liebert_line_voltage_mult = 1e7; /* stomp this in case input voltage was low */
 			upsdebugx(2, "ConfigVoltage = %g -> assuming correction factor = %g",
@@ -179,7 +183,7 @@ static const char *liebert_config_voltage_fun(double value)
 static const char *liebert_line_voltage_fun(double value)
 {
 	if( value < 1 ) {
-		if( abs(value - 1e-7) < 1e-9 ) {
+		if( fabs(value - 1e-7) < 1e-9 ) {
 			liebert_line_voltage_mult = 1e7;
 			upsdebugx(2, "Input/OutputVoltage = %g -> assuming correction factor = %g",
 				value, liebert_line_voltage_mult);
@@ -207,7 +211,7 @@ static const char *belkin_firmware_conversion_fun(double value)
 }
 
 static info_lkp_t belkin_firmware_conversion[] = {
-	{ 0, NULL, belkin_firmware_conversion_fun }
+	{ 0, NULL, belkin_firmware_conversion_fun, NULL }
 };
 
 static const char *belkin_upstype_conversion_fun(double value)
@@ -230,7 +234,7 @@ static const char *belkin_upstype_conversion_fun(double value)
 }
 
 static info_lkp_t belkin_upstype_conversion[] = {
-	{ 0, NULL, belkin_upstype_conversion_fun }
+	{ 0, NULL, belkin_upstype_conversion_fun, NULL }
 };
 
 static const char *belkin_sensitivity_conversion_fun(double value)
@@ -247,17 +251,17 @@ static const char *belkin_sensitivity_conversion_fun(double value)
 }
 
 static info_lkp_t belkin_sensitivity_conversion[] = {
-	{ 0, NULL, belkin_sensitivity_conversion_fun }
+	{ 0, NULL, belkin_sensitivity_conversion_fun, NULL }
 };
 
 static info_lkp_t belkin_test_info[] = {
-	{ 0, "No test initiated", NULL },
-	{ 1, "Done and passed", NULL },
-	{ 2, "Done and warning", NULL },
-	{ 3, "Done and error", NULL },
-	{ 4, "Aborted", NULL },
-	{ 5, "In progress", NULL },
-	{ 0, NULL, NULL }
+	{ 0, "No test initiated", NULL, NULL },
+	{ 1, "Done and passed", NULL, NULL },
+	{ 2, "Done and warning", NULL, NULL },
+	{ 3, "Done and error", NULL, NULL },
+	{ 4, "Aborted", NULL, NULL },
+	{ 5, "In progress", NULL, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 static const char *belkin_overload_conversion_fun(double value)
@@ -270,7 +274,7 @@ static const char *belkin_overload_conversion_fun(double value)
 }
 
 static info_lkp_t belkin_overload_conversion[] = {
-	{ 0, NULL, belkin_overload_conversion_fun }
+	{ 0, NULL, belkin_overload_conversion_fun, NULL }
 };
 
 static const char *belkin_overheat_conversion_fun(double value)
@@ -283,7 +287,7 @@ static const char *belkin_overheat_conversion_fun(double value)
 }
 
 static info_lkp_t belkin_overheat_conversion[] = {
-	{ 0, NULL, belkin_overheat_conversion_fun }
+	{ 0, NULL, belkin_overheat_conversion_fun, NULL }
 };
 
 static const char *belkin_commfault_conversion_fun(double value)
@@ -296,7 +300,7 @@ static const char *belkin_commfault_conversion_fun(double value)
 }
 
 static info_lkp_t belkin_commfault_conversion[] = {
-	{ 0, NULL, belkin_commfault_conversion_fun }
+	{ 0, NULL, belkin_commfault_conversion_fun, NULL }
 };
 
 static const char *belkin_awaitingpower_conversion_fun(double value)
@@ -309,7 +313,7 @@ static const char *belkin_awaitingpower_conversion_fun(double value)
 }
 
 static info_lkp_t belkin_awaitingpower_conversion[] = {
-	{ 0, NULL, belkin_awaitingpower_conversion_fun }
+	{ 0, NULL, belkin_awaitingpower_conversion_fun, NULL }
 };
 
 static const char *belkin_online_conversion_fun(double value)
@@ -322,7 +326,7 @@ static const char *belkin_online_conversion_fun(double value)
 }
 
 static info_lkp_t belkin_online_conversion[] = {
-	{ 0, NULL, belkin_online_conversion_fun }
+	{ 0, NULL, belkin_online_conversion_fun, NULL }
 };
 
 static const char *belkin_lowbatt_conversion_fun(double value)
@@ -335,7 +339,7 @@ static const char *belkin_lowbatt_conversion_fun(double value)
 }
 
 static info_lkp_t belkin_lowbatt_conversion[] = {
-	{ 0, NULL, belkin_lowbatt_conversion_fun }
+	{ 0, NULL, belkin_lowbatt_conversion_fun, NULL }
 };
 
 static const char *belkin_depleted_conversion_fun(double value)
@@ -348,7 +352,7 @@ static const char *belkin_depleted_conversion_fun(double value)
 }
 
 static info_lkp_t belkin_depleted_conversion[] = {
-	{ 0, NULL, belkin_depleted_conversion_fun }
+	{ 0, NULL, belkin_depleted_conversion_fun, NULL }
 };
 
 static const char *belkin_replacebatt_conversion_fun(double value)
@@ -361,7 +365,7 @@ static const char *belkin_replacebatt_conversion_fun(double value)
 }
 
 static info_lkp_t belkin_replacebatt_conversion[] = {
-	{ 0, NULL, belkin_replacebatt_conversion_fun }
+	{ 0, NULL, belkin_replacebatt_conversion_fun, NULL }
 };
 
 /* --------------------------------------------------------------- */
@@ -637,4 +641,5 @@ subdriver_t belkin_subdriver = {
 	belkin_format_model,
 	belkin_format_mfr,
 	belkin_format_serial,
+	fix_report_desc,
 };
