@@ -22,6 +22,7 @@
 
 #include "main.h"
 #include "serial.h"
+#include "nut_stdint.h"
 
 #define UPSDELAY 50000	/* 50 ms delay required for reliable operation */
 #define SER_WAIT_SEC	2	/* allow 2.0 sec for ser_get calls */
@@ -34,7 +35,7 @@
 #endif
 
 #define DRIVER_NAME     "Best Fortress UPS driver"
-#define DRIVER_VERSION  "0.05"
+#define DRIVER_VERSION  "0.06"
 
 /* driver description structure */
 upsdrv_info_t   upsdrv_info = {
@@ -66,16 +67,16 @@ void upsdrv_initinfo(void)
 	dstate_setinfo("ups.delay.shutdown", "10");	/* write only */
 
 	/* tunable via front panel: (european voltage level)
-	   parameter		factory default  range
-	   INFO_LOWXFER	196 V   p7=nnn   160-210
-	   INFO_HIGHXFER	254 V   p8=nnn   215-274
-	   INFO_LOBATTIME	2 min   p2=n     1-5
+	   parameter        factory default  range
+	   INFO_LOWXFER     196 V   p7=nnn   160-210
+	   INFO_HIGHXFER    254 V   p8=nnn   215-274
+	   INFO_LOBATTIME   2 min   p2=n     1-5
 
 	   comm mode    p6=0 dumb DONT USE (will lose access to parameter setting!)
-	   		p6=1 B1200
-			p6=2 B2400
-			P6=3 B4800
-			p6=4 B9600
+	        p6=1 B1200
+	        p6=2 B2400
+	        P6=3 B4800
+	        p6=4 B9600
 	   maybe cycle through speeds to autodetect?
 
 	   echo off     e0
@@ -116,7 +117,7 @@ static int checksum (char * s)
 	int sum;
 	for (i = 40, sum = 0; s[0] && s[1] && i > 0; i--, s += 2) {
 		sum += (fromhex (s[0]) << 4) + fromhex (s[1]);
-	} 
+	}
 	return sum;
 }
 
@@ -126,7 +127,8 @@ static inline int setinfo_int (const char *key, const char * s, size_t len)
 	char buf[10];
 	int val;
 
-	if (len > sizeof(buf)) len = sizeof(buf)-1;
+	if (len > sizeof(buf))
+		len = sizeof(buf)-1;
 	strncpy (buf, s, len);
 	buf[len] = 0;
 	val = atoi(buf);
@@ -141,7 +143,8 @@ static inline void setinfo_int_minutes (const char *key, const char * s, size_t
 {
 	char buf[10];
 
-	if (len > sizeof(buf)) len = sizeof(buf)-1;
+	if (len > sizeof(buf))
+		len = sizeof(buf)-1;
 	strncpy (buf, s, len);
 	buf[len] = 0;
 	dstate_setinfo (key, "%d", 60*atoi (buf));
@@ -151,47 +154,85 @@ static inline void setinfo_int_minutes (const char *key, const char * s, size_t
 static inline void setinfo_float (const char *key, const char * fmt, const char * s, size_t len, double factor)
 {
 	char buf[10];
-	if (len > sizeof(buf)) len = sizeof(buf)-1;
+	if (len > sizeof(buf))
+		len = sizeof(buf)-1;
 	strncpy (buf, s, len);
 	buf[len] = 0;
-	dstate_setinfo (key, fmt, factor * (double)atoi (buf));
+
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
+#pragma GCC diagnostic ignored "-Wformat-security"
+#endif
+	dstate_setinfo (key, fmt, factor * (double)(atoi (buf)));
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic pop
+#endif
 }
 
 static int upssend(const char *fmt,...) {
-	int	ret;
+	int ret;
 	char buf[1024], *p;
-	va_list	ap;
+	va_list ap;
 	unsigned int	sent = 0;
-	int d_usec = UPSDELAY;
+	useconds_t d_usec = UPSDELAY;
 
 	va_start(ap, fmt);
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
+#pragma GCC diagnostic ignored "-Wformat-security"
+#endif
 	ret = vsnprintf(buf, sizeof(buf), fmt, ap);
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic pop
+#endif
 	va_end(ap);
 
 	if ((ret < 1) || (ret >= (int) sizeof(buf)))
 		upslogx(LOG_WARNING, "ser_send_pace: vsnprintf needed more "
 				"than %d bytes", (int)sizeof(buf));
-	for (p = buf; *p; p++) {
+
+	for (p = buf; *p && sent < INT_MAX - 1; p++) {
 		if (write(upsfd, p, 1) != 1)
 			return -1;
 
-		if (d_usec)
+		/* Note: LGTM.com analysis warns that here
+		 * "Comparison is always true because d_usec >= 2"
+		 * since we initialize with UPSDELAY above.
+		 * Do not remove this check just in case that
+		 * initialization changes, or run-time value
+		 * becomes modified, in later iterations.
+		 */
+		if (d_usec > 0)
 			usleep(d_usec);
 
 		sent++;
+		if (sent >= INT_MAX) {
+			upslogx(LOG_WARNING, "ser_send_pace: sent more than INT_MAX, aborting");
+		}
 	}
 
-	return sent;
+	return (int)sent;
 }
 
-static int upsrecv(char *buf,size_t bufsize,char ec,const char *ic)
+static ssize_t upsrecv(char *buf,size_t bufsize,char ec,const char *ic)
 {
 	return ser_get_line(upsfd, buf, bufsize - 1, ec, ic,
-			SER_WAIT_SEC, SER_WAIT_USEC);
+	                    SER_WAIT_SEC, SER_WAIT_USEC);
 }
 
-static int upsflushin(int f,int verbose,const char *ignset)
+static ssize_t upsflushin(int f, int verbose, const char *ignset)
 {
+	NUT_UNUSED_VARIABLE(f);
 	return ser_flush_in(upsfd, ignset, verbose);
 }
 
@@ -199,12 +240,13 @@ static int upsflushin(int f,int verbose,const char *ignset)
 void upsdrv_updateinfo(void)
 {
 	char temp[256];
-	char *p;
+	char *p = NULL;
 	int loadva;
-	int len, recv;
+	size_t len = 0;
+	ssize_t recv;
 	int retry;
 	char ch;
-	int checksum_ok, is_online=1, is_off, low_batt, trimming, boosting;
+	int checksum_ok = -1, is_online = 1, is_off, low_batt, trimming, boosting;
 
 	upsdebugx(1, "upsdrv_updateinfo");
 
@@ -221,8 +263,8 @@ void upsdrv_updateinfo(void)
 			}
 		} while (temp[2] == 0);
 
-		upsdebugx(1, "upsdrv_updateinfo: received %i bytes (try %i)", recv, retry);
-		upsdebug_hex(5, "buffer", temp, recv);
+		upsdebugx(1, "upsdrv_updateinfo: received %zi bytes (try %i)", recv, retry);
+		upsdebug_hex(5, "buffer", temp, (size_t)recv);
 
 		/* syslog (LOG_DAEMON | LOG_NOTICE,"ups: got %d chars '%s'\n", recv, temp + 2); */
 		/* status example:
@@ -237,7 +279,7 @@ void upsdrv_updateinfo(void)
 		/* last bytes are a checksum:
 		   interpret response as hex string, sum of all bytes must be zero
 		 */
-		checksum_ok = (checksum (temp+2) & 0xff) == 0;
+		checksum_ok = ( (checksum (temp+2) & 0xff) == 0 );
 		/* setinfo (INFO_, ""); */
 
 		/* I can't figure out why this is missing the first two chars.
@@ -255,12 +297,19 @@ void upsdrv_updateinfo(void)
 		sleep(SER_WAIT_SEC);
 	}
 
-	if (!checksum_ok) {
-		upsdebugx(2, "checksum corruption");
-		upsdebug_hex(3, "buffer", temp, len);
+	if (!p || len < 1 || checksum_ok < 0) {
+		upsdebugx(2, "pointer to data not initialized after processing");
 		dstate_datastale();
 		return;
 	}
+
+	if (!checksum_ok) {
+		upsdebugx(2, "checksum corruption");
+		upsdebug_hex(3, "buffer", temp, (size_t)len);
+		dstate_datastale();
+		return;
+	}
+
 	/* upslogx(LOG_INFO, "updateinfo: %s", p); */
 
 	setinfo_int ("input.voltage", p+24,4);
@@ -309,9 +358,11 @@ void upsdrv_updateinfo(void)
 /* all UPS tunable parameters are set with command
    'p%d=%s'
 */
-int setparam (int parameter, int dlen, const char * data)
+static int setparam (int parameter, int dlen, const char * data)
 {
 	char reply[80];
+	/* Note the use of "%*s" - parameter (int)dlen specifies
+	 * the string width reserved for data */
 	upssend ("p%d=%*s\r", parameter, dlen, data);
 	if (upsrecv (reply, sizeof(reply), ENDCHAR, "") < 0) return 0;
 	return strncmp (reply, "OK", 2) == 0;
@@ -338,7 +389,7 @@ static void autorestart (int restart)
 /* set UPS parameters */
 static int upsdrv_setvar (const char *var, const char * data) {
 	int parameter;
-	int len = strlen(data);
+	size_t len = strlen(data);
 	upsdebugx(1, "Setvar: %s %s", var, data);
 	if (strcmp("input.transfer.low", var) == 0) {
 		parameter = 7;
@@ -354,8 +405,9 @@ static int upsdrv_setvar (const char *var, const char * data) {
 		return STAT_SET_UNKNOWN;
 	}
 	ups_setsuper (1);
-	if (setparam (parameter, len, data)) {
-		dstate_setinfo (var, "%*s", len, data);
+	assert (len < INT_MAX);
+	if (setparam (parameter, (int)len, data)) {
+		dstate_setinfo (var, "%*s", (int)len, data);
 	}
 	ups_setsuper (0);
 	return STAT_SET_HANDLED;
@@ -396,7 +448,7 @@ static int instcmd (const char *cmdname, const char *extra)
 		upssend ("OFF%s\r", p);
 		return STAT_INSTCMD_HANDLED;
 	}
-	upslogx(LOG_INFO, "instcmd: unknown command %s", cmdname);
+	upslogx(LOG_INFO, "instcmd: unknown command [%s] [%s]", cmdname, extra);
 	return STAT_INSTCMD_UNKNOWN;
 }
 
@@ -411,7 +463,7 @@ void upsdrv_makevartable(void)
 	addvar (VAR_VALUE, "max_load", "rated VA load VA");
 }
 
-struct {
+static struct {
 	const char * val;
 	speed_t speed;
 } speed_table[] = {
