@@ -1,5 +1,10 @@
 /* bestups.c - model specific routines for Best-UPS Fortress models
 
+   OBSOLETION WARNING: Please to not base new development on this
+   codebase, instead create a new subdriver for nutdrv_qx which
+   generally covers all Megatec/Qx protocol family and aggregates
+   device support from such legacy drivers over time.
+
    Copyright (C) 1999  Russell Kroll <rkroll@exploits.org>
 
    ID config option by Jason White <jdwhite@jdwhite.org>
@@ -23,7 +28,7 @@
 #include "serial.h"
 
 #define DRIVER_NAME	"Best UPS driver"
-#define DRIVER_VERSION	"1.06"
+#define DRIVER_VERSION	"1.07"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -122,13 +127,14 @@ static int instcmd(const char *cmdname, const char *extra)
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	upslogx(LOG_NOTICE, "instcmd: unknown command [%s]", cmdname);
+	upslogx(LOG_NOTICE, "instcmd: unknown command [%s] [%s]", cmdname, extra);
 	return STAT_INSTCMD_UNKNOWN;
 }
 
 static int get_ident(char *buf, size_t bufsize)
 {
-	int	i, ret;
+	int	i;
+	ssize_t	ret;
 	char	*ID;
 
 	ID = getval("ID");	/* user-supplied override from ups.conf */
@@ -142,7 +148,7 @@ static int get_ident(char *buf, size_t bufsize)
 	for (i = 0; i < MAXTRIES; i++) {
 		ser_send_pace(upsfd, UPSDELAY, "\rID\r");
 
-		ret = ser_get_line(upsfd, buf, bufsize, ENDCHAR, "", 
+		ret = ser_get_line(upsfd, buf, bufsize, ENDCHAR, "",
 			SER_WAIT_SEC, SER_WAIT_USEC);
 
 		if (ret > 0)
@@ -210,7 +216,7 @@ static void ups_ident(void)
 	if ((!model) || (!rating)) {
 		fatalx(EXIT_FAILURE, "Didn't get a valid ident string");
 	}
-	
+
 	model_set(model, rating);
 
 	/* Battery voltage multiplier */
@@ -240,12 +246,13 @@ static void ups_ident(void)
 static void ups_sync(void)
 {
 	char	buf[256];
-	int	i, ret;
+	int	i;
+	ssize_t	ret;
 
 	for (i = 0; i < MAXTRIES; i++) {
 		ser_send_pace(upsfd, UPSDELAY, "\rQ1\r");
 
-		ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, "", 
+		ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, "",
 			SER_WAIT_SEC, SER_WAIT_USEC);
 
 		/* return once we get something that looks usable */
@@ -263,7 +270,7 @@ void upsdrv_initinfo(void)
 	ups_sync();
 	ups_ident();
 
-	printf("Detected %s %s on %s\n", dstate_getinfo("ups.mfr"), 
+	printf("Detected %s %s on %s\n", dstate_getinfo("ups.mfr"),
 		dstate_getinfo("ups.model"), device_path);
 
 	/* paranoia - cancel any shutdown that might already be running */
@@ -277,13 +284,14 @@ void upsdrv_initinfo(void)
 
 static int ups_on_line(void)
 {
-	int	i, ret;
+	int	i;
+	ssize_t	ret;
 	char	temp[256], pstat[32];
 
 	for (i = 0; i < MAXTRIES; i++) {
 		ser_send_pace(upsfd, UPSDELAY, "\rQ1\r");
 
-		ret = ser_get_line(upsfd, temp, sizeof(temp), ENDCHAR, "", 
+		ret = ser_get_line(upsfd, temp, sizeof(temp), ENDCHAR, "",
 			SER_WAIT_SEC, SER_WAIT_USEC);
 
 		/* Q1 must return 46 bytes starting with a ( */
@@ -303,7 +311,7 @@ static int ups_on_line(void)
 	upslogx(LOG_ERR, "Status read failed: assuming on battery");
 
 	return 0;	/* on battery */
-}	
+}
 
 void upsdrv_shutdown(void)
 {
@@ -319,10 +327,10 @@ void upsdrv_shutdown(void)
 
 void upsdrv_updateinfo(void)
 {
-	char	involt[16], outvolt[16], loadpct[16], acfreq[16], 
+	char	involt[16], outvolt[16], loadpct[16], acfreq[16],
 		battvolt[16], upstemp[16], pstat[16], buf[256];
 	float	bvoltp;
-	int	ret;
+	ssize_t	ret;
 
 	ret = ser_send_pace(upsfd, UPSDELAY, "\rQ1\r");
 
@@ -335,7 +343,7 @@ void upsdrv_updateinfo(void)
 	/* these things need a long time to respond completely */
 	usleep(200000);
 
-	ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, "", 
+	ret = ser_get_line(upsfd, buf, sizeof(buf), ENDCHAR, "",
 		SER_WAIT_SEC, SER_WAIT_USEC);
 
 	if (ret < 1) {
@@ -345,13 +353,13 @@ void upsdrv_updateinfo(void)
 	}
 
 	if (ret < 46) {
-		ser_comm_fail("Poll failed: short read (got %d bytes)", ret);
+		ser_comm_fail("Poll failed: short read (got %zd bytes)", ret);
 		dstate_datastale();
 		return;
 	}
 
 	if (ret > 46) {
-		ser_comm_fail("Poll failed: response too long (got %d bytes)",
+		ser_comm_fail("Poll failed: response too long (got %zd bytes)",
 			ret);
 		dstate_datastale();
 		return;
@@ -366,7 +374,7 @@ void upsdrv_updateinfo(void)
 
 	ser_comm_good();
 
-	sscanf(buf, "%*c%s %*s %s %s %s %s %s %s", involt, outvolt, 
+	sscanf(buf, "%*c%s %*s %s %s %s %s %s %s", involt, outvolt,
 		loadpct, acfreq, battvolt, upstemp, pstat);
 
 	/* Guesstimation of battery charge left (inaccurate) */
@@ -428,6 +436,15 @@ void upsdrv_makevartable(void)
 
 void upsdrv_initups(void)
 {
+	upsdebugx(0,
+		"Please note that this driver is deprecated and will not receive\n"
+		"new development. If it works for managing your devices - fine,\n"
+		"but if you are running it to try setting up a new device, please\n"
+		"consider the newer nutdrv_qx instead, which should handle all 'Qx'\n"
+		"protocol variants for NUT. (Please also report if your device works\n"
+		"with this driver, but nutdrv_qx would not actually support it with\n"
+		"any subdriver!)\n");
+
 	upsfd = ser_open(device_path);
 	ser_set_speed(upsfd, device_path, B2400);
 }
