@@ -1,6 +1,11 @@
 /*
  * blazer_ser.c: support for Megatec/Q1 serial protocol based UPSes
  *
+ * OBSOLETION WARNING: Please to not base new development on this
+ * codebase, instead create a new subdriver for nutdrv_qx which
+ * generally covers all Megatec/Qx protocol family and aggregates
+ * device support from such legacy drivers over time.
+ *
  * A document describing the protocol implemented by this driver can be
  * found online at "http://www.networkupstools.org/protocols/megatec.html".
  *
@@ -26,7 +31,7 @@
 #include "blazer.h"
 
 #define DRIVER_NAME	"Megatec/Q1 protocol serial driver"
-#define DRIVER_VERSION	"1.57"
+#define DRIVER_VERSION	"1.58"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -44,10 +49,10 @@ upsdrv_info_t upsdrv_info = {
  * Returns < 0 on error, 0 on timeout and the number of bytes read on
  * success.
  */
-int blazer_command(const char *cmd, char *buf, size_t buflen)
+ssize_t blazer_command(const char *cmd, char *buf, size_t buflen)
 {
 #ifndef TESTING
-	int	ret;
+	ssize_t	ret;
 
 	ser_flush_io(upsfd);
 
@@ -90,10 +95,11 @@ int blazer_command(const char *cmd, char *buf, size_t buflen)
 			continue;
 		}
 
-		return snprintf(buf, buflen, "%s", testing[i].answer);
+		/* TODO: Range-check int vs ssize_t values */
+		return (ssize_t)snprintf(buf, buflen, "%s", testing[i].answer);
 	}
 
-	return snprintf(buf, buflen, "%s", testing[i].cmd);
+	return (ssize_t)snprintf(buf, buflen, "%s", testing[i].cmd);
 #endif
 }
 
@@ -124,7 +130,7 @@ void upsdrv_initups(void)
 		{ "reverse",	0, 1 },
 		{ "both",	1, 1 },
 		{ "none",	0, 0 },
-		{ NULL }
+		{ NULL, 0, 0 }
 	};
 
 	int	i;
