@@ -1,11 +1,17 @@
 /*
  * blazer_usb.c: support for Megatec/Q1 USB protocol based UPSes
  *
+ * OBSOLETION WARNING: Please to not base new development on this
+ * codebase, instead create a new subdriver for nutdrv_qx which
+ * generally covers all Megatec/Qx protocol family and aggregates
+ * device support from such legacy drivers over time.
+ *
  * A document describing the protocol implemented by this driver can be
  * found online at "http://www.networkupstools.org/protocols/megatec.html".
  *
  * Copyright (C) 2003-2009  Arjen de Korte <adkorte-guest@alioth.debian.org>
  * Copyright (C) 2011-2012  Arnaud Quette <arnaud.quette@free.fr>
+ * Copyright (C) 2016       Eaton
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,12 +29,12 @@
  */
 
 #include "main.h"
-#include "libusb.h"
+#include "nut_libusb.h"
 #include "usb-common.h"
 #include "blazer.h"
 
 #define DRIVER_NAME	"Megatec/Q1 protocol USB driver"
-#define DRIVER_VERSION	"0.12"
+#define DRIVER_VERSION	"0.14"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -61,15 +67,15 @@ static int cypress_command(const char *cmd, char *buf, size_t buflen)
 	memset(tmp, 0, sizeof(tmp));
 	snprintf(tmp, sizeof(tmp), "%s", cmd);
 
-	for (i = 0; i < strlen(tmp); i += ret) {
+	for (i = 0; i < strlen(tmp); i += (size_t)ret) {
 
 		/* Write data in 8-byte chunks */
 		/* ret = usb->set_report(udev, 0, (unsigned char *)&tmp[i], 8); */
 		ret = usb_control_msg(udev, USB_ENDPOINT_OUT + USB_TYPE_CLASS + USB_RECIP_INTERFACE,
-			0x09, 0x200, 0, &tmp[i], 8, 5000);
+			0x09, 0x200, 0, (usb_ctrl_charbuf)&tmp[i], 8, 5000);
 
 		if (ret <= 0) {
-			upsdebugx(3, "send: %s", ret ? usb_strerror() : "timeout");
+			upsdebugx(3, "send: %s", ret ? nut_usb_strerror(ret) : "timeout");
 			return ret;
 		}
 	}
@@ -78,24 +84,27 @@ static int cypress_command(const char *cmd, char *buf, size_t buflen)
 
 	memset(buf, 0, buflen);
 
-	for (i = 0; (i <= buflen-8) && (strchr(buf, '\r') == NULL); i += ret) {
+	for (i = 0; (i <= buflen-8) && (strchr(buf, '\r') == NULL); i += (size_t)ret) {
 
 		/* Read data in 8-byte chunks */
 		/* ret = usb->get_interrupt(udev, (unsigned char *)&buf[i], 8, 1000); */
-		ret = usb_interrupt_read(udev, 0x81, &buf[i], 8, 1000);
+		ret = usb_interrupt_read(udev,
+			0x81,
+			(usb_ctrl_charbuf)&buf[i], 8, 1000);
 
 		/*
 		 * Any errors here mean that we are unable to read a reply (which
 		 * will happen after successfully writing a command to the UPS)
 		 */
 		if (ret <= 0) {
-			upsdebugx(3, "read: %s", ret ? usb_strerror() : "timeout");
+			upsdebugx(3, "read: %s", ret ? nut_usb_strerror(ret) : "timeout");
 			return ret;
 		}
 	}
 
 	upsdebugx(3, "read: %.*s", (int)strcspn(buf, "\r"), buf);
-	return i;
+	/* TODO: Range-check before cast */
+	return (int)i;
 }
 
 
@@ -109,7 +118,9 @@ static int phoenix_command(const char *cmd, char *buf, size_t buflen)
 
 		/* Read data in 8-byte chunks */
 		/* ret = usb->get_interrupt(udev, (unsigned char *)tmp, 8, 1000); */
-		ret = usb_interrupt_read(udev, 0x81, tmp, 8, 1000);
+		ret = usb_interrupt_read(udev,
+			0x81,
+			(usb_ctrl_charbuf)&tmp, 8, 1000);
 
 		/*
 		 * This USB to serial implementation is crappy. In order to read correct
@@ -118,32 +129,34 @@ static int phoenix_command(const char *cmd, char *buf, size_t buflen)
 		 */
 		switch (ret)
 		{
-		case -EPIPE:		/* Broken pipe */
+		case ERROR_PIPE:    /** Pipe error or Broken pipe */
 			usb_clear_halt(udev, 0x81);
-		case -ETIMEDOUT:	/* Connection timed out */
+			break;
+
+		case ERROR_TIMEOUT: /** Operation or Connection timed out */
 			break;
 		}
 
 		if (ret < 0) {
-			upsdebugx(3, "flush: %s", usb_strerror());
+			upsdebugx(3, "flush: %s", nut_usb_strerror(ret));
 			break;
 		}
 
-		upsdebug_hex(4, "dump", tmp, ret);
+		upsdebug_hex(4, "dump", tmp, (size_t)ret);
 	}
 
 	memset(tmp, 0, sizeof(tmp));
 	snprintf(tmp, sizeof(tmp), "%s", cmd);
 
-	for (i = 0; i < strlen(tmp); i += ret) {
+	for (i = 0; i < strlen(tmp); i += (size_t)ret) {
 
 		/* Write data in 8-byte chunks */
 		/* ret = usb->set_report(udev, 0, (unsigned char *)&tmp[i], 8); */
 		ret = usb_control_msg(udev, USB_ENDPOINT_OUT + USB_TYPE_CLASS + USB_RECIP_INTERFACE,
-			0x09, 0x200, 0, &tmp[i], 8, 1000);
+			0x09, 0x200, 0, (usb_ctrl_charbuf)&tmp[i], 8, 1000);
 
 		if (ret <= 0) {
-			upsdebugx(3, "send: %s", ret ? usb_strerror() : "timeout");
+			upsdebugx(3, "send: %s", ret ? nut_usb_strerror(ret) : "timeout");
 			return ret;
 		}
 	}
@@ -152,24 +165,27 @@ static int phoenix_command(const char *cmd, char *buf, size_t buflen)
 
 	memset(buf, 0, buflen);
 
-	for (i = 0; (i <= buflen-8) && (strchr(buf, '\r') == NULL); i += ret) {
+	for (i = 0; (i <= buflen-8) && (strchr(buf, '\r') == NULL); i += (size_t)ret) {
 
 		/* Read data in 8-byte chunks */
 		/* ret = usb->get_interrupt(udev, (unsigned char *)&buf[i], 8, 1000); */
-		ret = usb_interrupt_read(udev, 0x81, &buf[i], 8, 1000);
+		ret = usb_interrupt_read(udev,
+			0x81,
+			(usb_ctrl_charbuf)&buf[i], 8, 1000);
 
 		/*
 		 * Any errors here mean that we are unable to read a reply (which
 		 * will happen after successfully writing a command to the UPS)
 		 */
 		if (ret <= 0) {
-			upsdebugx(3, "read: %s", ret ? usb_strerror() : "timeout");
+			upsdebugx(3, "read: %s", ret ? nut_usb_strerror(ret) : "timeout");
 			return ret;
 		}
 	}
 
 	upsdebugx(3, "read: %.*s", (int)strcspn(buf, "\r"), buf);
-	return i;
+	/* TODO: Range-check before cast */
+	return (int)i;
 }
 
 
@@ -181,14 +197,14 @@ static int ippon_command(const char *cmd, char *buf, size_t buflen)
 
 	snprintf(tmp, sizeof(tmp), "%s", cmd);
 
-	for (i = 0; i < strlen(tmp); i += ret) {
+	for (i = 0; i < strlen(tmp); i += (size_t)ret) {
 
 		/* Write data in 8-byte chunks */
 		ret = usb_control_msg(udev, USB_ENDPOINT_OUT + USB_TYPE_CLASS + USB_RECIP_INTERFACE,
-			0x09, 0x2, 0, &tmp[i], 8, 1000);
+			0x09, 0x2, 0, (usb_ctrl_charbuf)&tmp[i], 8, 1000);
 
 		if (ret <= 0) {
-			upsdebugx(3, "send: %s", (ret != -ETIMEDOUT) ? usb_strerror() : "Connection timed out");
+			upsdebugx(3, "send: %s", (ret != ERROR_TIMEOUT) ? nut_usb_strerror(ret) : "Connection timed out");
 			return ret;
 		}
 	}
@@ -196,14 +212,16 @@ static int ippon_command(const char *cmd, char *buf, size_t buflen)
 	upsdebugx(3, "send: %.*s", (int)strcspn(tmp, "\r"), tmp);
 
 	/* Read all 64 bytes of the reply in one large chunk */
-	ret = usb_interrupt_read(udev, 0x81, tmp, sizeof(tmp), 1000);
+	ret = usb_interrupt_read(udev,
+		0x81,
+		(usb_ctrl_charbuf)&tmp, sizeof(tmp), 1000);
 
 	/*
 	 * Any errors here mean that we are unable to read a reply (which
 	 * will happen after successfully writing a command to the UPS)
 	 */
 	if (ret <= 0) {
-		upsdebugx(3, "read: %s", (ret != -ETIMEDOUT) ? usb_strerror() : "Connection timed out");
+		upsdebugx(3, "read: %s", (ret != ERROR_TIMEOUT) ? nut_usb_strerror(ret) : "Connection timed out");
 		return ret;
 	}
 
@@ -243,7 +261,7 @@ static int krauler_command(const char *cmd, char *buf, size_t buflen)
 		{ "Q\r",  0x07, '\r' },
 		{ "C\r",  0x0b, '\r' },
 		{ "CT\r", 0x0b, '\r' },
-		{ NULL }
+		{ NULL, 0, '\0' }
 	};
 
 	int	i;
@@ -262,14 +280,16 @@ static int krauler_command(const char *cmd, char *buf, size_t buflen)
 
 			if (langid_fix != -1) {
 				/* Apply langid_fix value */
-				ret = usb_get_string(udev, command[i].index, langid_fix, buf, buflen);
+				ret = usb_get_string(udev, command[i].index, langid_fix,
+					(usb_ctrl_charbuf)buf, buflen);
 			}
 			else {
-				ret = usb_get_string_simple(udev, command[i].index, buf, buflen);
+				ret = usb_get_string_simple(udev, command[i].index,
+					(usb_ctrl_charbuf)buf, buflen);
 			}
 
 			if (ret <= 0) {
-				upsdebugx(3, "read: %s", ret ? usb_strerror() : "timeout");
+				upsdebugx(3, "read: %s", ret ? nut_usb_strerror(ret) : "timeout");
 				return ret;
 			}
 
@@ -279,7 +299,7 @@ static int krauler_command(const char *cmd, char *buf, size_t buflen)
 			if (langid_fix != -1) {
 				/* Limit this check, at least for now */
 				/* Invalid receive size - message corrupted */
-				if (ret != buf[0]) 
+				if (ret != buf[0])
 				{
 					upsdebugx(1, "size mismatch: %d / %d", ret, buf[0]);
 					continue;
@@ -288,7 +308,7 @@ static int krauler_command(const char *cmd, char *buf, size_t buflen)
 				/* Simple unicode -> ASCII inplace conversion
 				 * FIXME: this code is at least shared with mge-shut/libshut
 				 * Create a common function? */
-				unsigned int di, si, size = buf[0];
+				size_t di, si, size = (size_t)buf[0];
 				for (di = 0, si = 2; si < size; si += 2) {
 					if (di >= (buflen - 1))
 						break;
@@ -299,7 +319,9 @@ static int krauler_command(const char *cmd, char *buf, size_t buflen)
 						buf[di++] = buf[si];
 				}
 				buf[di] = 0;
-				ret = di;
+				/* with buf a char* array, practical "size" limit and
+				 * so "di" are small enough to cast to int */
+				ret = (int)di;
 			}
 
 			/* "UPS No Ack" has a special meaning */
@@ -326,6 +348,8 @@ static int krauler_command(const char *cmd, char *buf, size_t buflen)
 
 static void *cypress_subdriver(USBDevice_t *device)
 {
+	NUT_UNUSED_VARIABLE(device);
+
 	subdriver_command = &cypress_command;
 	return NULL;
 }
@@ -333,6 +357,8 @@ static void *cypress_subdriver(USBDevice_t *device)
 
 static void *ippon_subdriver(USBDevice_t *device)
 {
+	NUT_UNUSED_VARIABLE(device);
+
 	subdriver_command = &ippon_command;
 	return NULL;
 }
@@ -340,6 +366,8 @@ static void *ippon_subdriver(USBDevice_t *device)
 
 static void *krauler_subdriver(USBDevice_t *device)
 {
+	NUT_UNUSED_VARIABLE(device);
+
 	subdriver_command = &krauler_command;
 	return NULL;
 }
@@ -347,6 +375,8 @@ static void *krauler_subdriver(USBDevice_t *device)
 
 static void *phoenix_subdriver(USBDevice_t *device)
 {
+	NUT_UNUSED_VARIABLE(device);
+
 	subdriver_command = &phoenix_command;
 	return NULL;
 }
@@ -365,13 +395,16 @@ static usb_device_id_t blazer_usb_id[] = {
 	{ USB_DEVICE(0x06da, 0x0601), &phoenix_subdriver },	/* Online Zinto A */
 	{ USB_DEVICE(0x0f03, 0x0001), &cypress_subdriver },	/* Unitek Alpha 1200Sx */
 	{ USB_DEVICE(0x14f0, 0x00c9), &phoenix_subdriver },	/* GE EP series */
-	/* end of list */
-	{-1, -1, NULL}
+
+	/* Terminating entry */
+	{ 0, 0, NULL }
 };
 
 
 static int device_match_func(USBDevice_t *hd, void *privdata)
 {
+	NUT_UNUSED_VARIABLE(privdata);
+
 	if (subdriver_command) {
 		return 1;
 	}
@@ -403,10 +436,10 @@ static USBDeviceMatcher_t device_matcher = {
  * Returns < 0 on error, 0 on timeout and the number of bytes read on
  * success.
  */
-int blazer_command(const char *cmd, char *buf, size_t buflen)
+ssize_t blazer_command(const char *cmd, char *buf, size_t buflen)
 {
 #ifndef TESTING
-	int	ret;
+	ssize_t	ret;
 
 	if (udev == NULL) {
 		ret = usb->open(&udev, &usbdevice, reopen_matcher, NULL);
@@ -423,36 +456,50 @@ int blazer_command(const char *cmd, char *buf, size_t buflen)
 
 	switch (ret)
 	{
-	case -EBUSY:		/* Device or resource busy */
+	case ERROR_BUSY:		/* Device or resource busy */
 		fatal_with_errno(EXIT_FAILURE, "Got disconnected by another driver");
+#ifndef HAVE___ATTRIBUTE__NORETURN
+		exit(EXIT_FAILURE);	/* Should not get here in practice, but compiler is afraid we can fall through */
+#endif
 
+#if WITH_LIBUSB_0_1 /* limit to libusb 0.1 implementation */
 	case -EPERM:		/* Operation not permitted */
 		fatal_with_errno(EXIT_FAILURE, "Permissions problem");
+# ifndef HAVE___ATTRIBUTE__NORETURN
+		exit(EXIT_FAILURE);	/* Should not get here in practice, but compiler is afraid we can fall through */
+# endif
+#endif /* WITH_LIBUSB_0_1 */
 
-	case -EPIPE:		/* Broken pipe */
+	case ERROR_PIPE:		/* Broken pipe */
 		if (usb_clear_halt(udev, 0x81) == 0) {
 			upsdebugx(1, "Stall condition cleared");
 			break;
 		}
-#ifdef ETIME
+#if (defined ETIME) && ETIME && WITH_LIBUSB_0_1
+		goto fallthrough_case_etime;
 	case -ETIME:		/* Timer expired */
+	fallthrough_case_etime:
 #endif
 		if (usb_reset(udev) == 0) {
 			upsdebugx(1, "Device reset handled");
 		}
-	case -ENODEV:		/* No such device */
-	case -EACCES:		/* Permission denied */
-	case -EIO:		/* I/O error */
+		goto fallthrough_case_reconnect;
+	case ERROR_NO_DEVICE: /* No such device */
+	case ERROR_ACCESS:    /* Permission denied */
+	case ERROR_IO:        /* I/O error */
+#if WITH_LIBUSB_0_1 /* limit to libusb 0.1 implementation */
 	case -ENXIO:		/* No such device or address */
-	case -ENOENT:		/* No such file or directory */
+#endif
+	case ERROR_NOT_FOUND:		/* No such file or directory */
+	fallthrough_case_reconnect:
 		/* Uh oh, got to reconnect! */
 		usb->close(udev);
 		udev = NULL;
 		break;
 
-	case -ETIMEDOUT:	/* Connection timed out */
-	case -EOVERFLOW:	/* Value too large for defined data type */
-#ifdef EPROTO
+	case ERROR_TIMEOUT:  /* Connection timed out */
+	case ERROR_OVERFLOW: /* Value too large for defined data type */
+#if EPROTO && WITH_LIBUSB_0_1
 	case -EPROTO:		/* Protocol error */
 #endif
 	default:
@@ -460,7 +507,7 @@ int blazer_command(const char *cmd, char *buf, size_t buflen)
 	}
 
 	return ret;
-#else
+#else	/* if TESTING: */
 	const struct {
 		const char	*command;
 		const char	*answer;
@@ -481,16 +528,41 @@ int blazer_command(const char *cmd, char *buf, size_t buflen)
 			continue;
 		}
 
-		return snprintf(buf, buflen, "%s", testing[i].answer);
+		/* TODO: Range-check int vs ssize_t values */
+		return (ssize_t)snprintf(buf, buflen, "%s", testing[i].answer);
 	}
 
-	return snprintf(buf, buflen, "%s", testing[i].command);
-#endif
+	return (ssize_t)snprintf(buf, buflen, "%s", testing[i].command);
+#endif	/* TESTING */
 }
 
+#ifndef TESTING
+static const struct subdriver_t {
+	const char	*name;
+	int		(*command)(const char *cmd, char *buf, size_t buflen);
+} subdriver[] = {
+	{ "cypress", &cypress_command },
+	{ "phoenix", &phoenix_command },
+	{ "ippon", &ippon_command },
+	{ "krauler", &krauler_command },
+	{ NULL, NULL }
+};
+#endif	/* TESTING */
 
 void upsdrv_help(void)
 {
+#ifndef TESTING
+	printf("\nAcceptable values for 'subdriver' via -x or ups.conf in this driver: ");
+	size_t i;
+
+	for (i = 0; subdriver[i].name != NULL; i++) {
+		if (i>0)
+			printf(", ");
+		printf("%s", subdriver[i].name);
+	}
+	printf("\n\n");
+#endif	/* TESTING */
+
 	printf("Read The Fine Manual ('man 8 blazer_usb')\n");
 }
 
@@ -509,37 +581,30 @@ void upsdrv_makevartable(void)
 void upsdrv_initups(void)
 {
 #ifndef TESTING
-	const struct {
-		const char	*name;
-		int		(*command)(const char *cmd, char *buf, size_t buflen);
-	} subdriver[] = {
-		{ "cypress", &cypress_command },
-		{ "phoenix", &phoenix_command },
-		{ "ippon", &ippon_command },
-		{ "krauler", &krauler_command },
-		{ NULL }
-	};
-
 	int	ret, langid;
 	char	tbuf[255]; /* Some devices choke on size > 255 */
-	char	*regex_array[6];
-
+	char	*regex_array[7];
 	char	*subdrv = getval("subdriver");
 
+	warn_if_bad_usb_port_filename(device_path);
+
 	regex_array[0] = getval("vendorid");
 	regex_array[1] = getval("productid");
 	regex_array[2] = getval("vendor");
 	regex_array[3] = getval("product");
 	regex_array[4] = getval("serial");
 	regex_array[5] = getval("bus");
+	regex_array[6] = getval("device");
 
 	/* check for language ID workaround (#1) */
 	if (getval("langid_fix")) {
 		/* skip "0x" prefix and set back to hexadecimal */
-		if (sscanf(getval("langid_fix") + 2, "%x", &langid_fix) != 1) {
+		unsigned int u_langid_fix;
+		if ( (sscanf(getval("langid_fix") + 2, "%x", &u_langid_fix) != 1) || (u_langid_fix > INT_MAX) ) {
 			upslogx(LOG_NOTICE, "Error enabling language ID workaround");
 		}
 		else {
+			langid_fix = (int)u_langid_fix;
 			upsdebugx(2, "language ID workaround enabled (using '0x%x')", langid_fix);
 		}
 	}
@@ -618,13 +683,13 @@ void upsdrv_initups(void)
 		 *   in the descriptor. See USB 2.0 specification, section 9.6.7, for
 		 *   more information on this.
 		 * This should allow automatic application of the workaround */
-		ret = usb_get_string(udev, 0, 0, tbuf, sizeof(tbuf));
+		ret = usb_get_string(udev, 0, 0, (usb_ctrl_charbuf)tbuf, sizeof(tbuf));
 		if (ret >= 4) {
-			langid = tbuf[2] | (tbuf[3] << 8);
+			langid = (unsigned char)tbuf[2] | ((unsigned char)tbuf[3] << 8);
 			upsdebugx(1, "First supported language ID: 0x%x (please report to the NUT maintainer!)", langid);
 		}
 	}
-#endif
+#endif	/* TESTING */
 	blazer_initups();
 }
 
@@ -645,5 +710,6 @@ void upsdrv_cleanup(void)
 	free(usbdevice.Product);
 	free(usbdevice.Serial);
 	free(usbdevice.Bus);
-#endif
+	free(usbdevice.Device);
+#endif	/* TESTING */
 }
