@@ -2,7 +2,7 @@
 
    Copyright (C)
 	2003	Russell Kroll <rkroll@exploits.org>
-	2012	Arnaud Quette <arnaud.quette@free.fr>
+	2012-2017	Arnaud Quette <arnaud.quette@free.fr>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #ifndef DSTATE_H_SEEN
 #define DSTATE_H_SEEN 1
 
+#include "timehead.h"
 #include "state.h"
 #include "attribute.h"
 
@@ -31,6 +32,10 @@
 #define DS_LISTEN_BACKLOG 16
 #define DS_MAX_READ 256		/* don't read forever from upsd */
 
+#ifndef MAX_STRING_SIZE
+#define MAX_STRING_SIZE	128
+#endif
+
 /* track client connections */
 typedef struct conn_s {
 	int     fd;
@@ -45,7 +50,7 @@ typedef struct conn_s {
 	 * Defaults to nonblocking, for backward compatibility */
 	extern	int	do_synchronous;
 
-void dstate_init(const char *prog, const char *devname);
+char * dstate_init(const char *prog, const char *devname);
 int dstate_poll_fds(struct timeval timeout, int extrafd);
 int dstate_setinfo(const char *var, const char *fmt, ...)
 	__attribute__ ((__format__ (__printf__, 2, 3)));
@@ -53,7 +58,9 @@ int dstate_addenum(const char *var, const char *fmt, ...)
 	__attribute__ ((__format__ (__printf__, 2, 3)));
 int dstate_addrange(const char *var, const int min, const int max);
 void dstate_setflags(const char *var, int flags);
-void dstate_setaux(const char *var, int aux);
+void dstate_addflags(const char *var, const int addflags);
+void dstate_delflags(const char *var, const int delflags);
+void dstate_setaux(const char *var, long aux);
 const char *dstate_getinfo(const char *var);
 void dstate_addcmd(const char *cmdname);
 int dstate_delinfo(const char *var);
@@ -82,5 +89,16 @@ void status_commit(void);
 void alarm_init(void);
 void alarm_set(const char *buf);
 void alarm_commit(void);
+void device_alarm_init(void);
+void device_alarm_commit(const int device_number);
+
+int dstate_detect_phasecount(
+        const char *xput_prefix,
+        const int may_change_dstate,
+        int *inited_phaseinfo,
+        int *num_phases,
+        const int may_reevaluate);
+
+void dstate_dump(void);
 
 #endif	/* DSTATE_H_SEEN */
