@@ -29,18 +29,22 @@
  *   * poll the "port" file for change
  */
 
+#include "config.h" /* must be the first header */
+
 #include <netdb.h>
 #include <netinet/in.h>
 #include <sys/socket.h>
+#include <sys/stat.h>
 #include <string.h>
 
 #include "main.h"
 #include "parseconf.h"
+#include "nut_stdint.h"
 #include "upsclient.h"
 #include "dummy-ups.h"
 
 #define DRIVER_NAME	"Device simulation and repeater driver"
-#define DRIVER_VERSION	"0.14"
+#define DRIVER_VERSION	"0.15"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info =
@@ -52,16 +56,40 @@ upsdrv_info_t upsdrv_info =
 	{ NULL }
 };
 
-#define MODE_NONE	0
-#define MODE_DUMMY		1 /* use the embedded defintion or a definition file */
-#define MODE_REPEATER	2 /* use libupsclient to repeat an UPS */
-#define MODE_META		3 /* consolidate data from several UPSs (TBS) */
+enum drivermode {
+	MODE_NONE = 0,
 
-int mode=MODE_NONE;
+	/* use the embedded defintion or a definition file, parsed in a
+	 * loop again and again (often with TIMER lines to delay changes)
+	 * Default mode for files with *.seq naming pattern
+	 * (legacy-compatibility note: and other patterns except *.dev)
+	 */
+	MODE_DUMMY_LOOP,
+
+	/* use the embedded defintion or a definition file, parsed once
+	 *
+	 * This allows to spin up a dummy device with initial readings
+	 * and retain in memory whatever SET VAR was sent by clients later.
+	 * This is also less stressful on system resources to run the dummy.
+	 *
+	 * Default mode for files with *.dev naming pattern
+	 */
+	MODE_DUMMY_ONCE,
+
+	/* use libupsclient to repeat another UPS */
+	MODE_REPEATER,
+
+	/* consolidate data from several UPSs (TBS) */
+	MODE_META
+};
+typedef enum drivermode drivermode_t;
+
+static drivermode_t mode = MODE_NONE;
 
 /* parseconf context, for dummy mode using a file */
-PCONF_CTX_t	*ctx=NULL;
-time_t		next_update = -1;
+static PCONF_CTX_t	*ctx = NULL;
+static time_t		next_update = -1;
+static struct stat	datafile_stat;
 
 #define MAX_STRING_SIZE	128
 
@@ -77,7 +105,7 @@ static int upsclient_update_vars(void);
 /* connection information */
 static char		*client_upsname = NULL, *hostname = NULL;
 static UPSCONN_t	*ups = NULL;
-static int	port;
+static uint16_t	port;
 
 /* Driver functions */
 
@@ -87,7 +115,8 @@ void upsdrv_initinfo(void)
 
 	switch (mode)
 	{
-		case MODE_DUMMY:
+		case MODE_DUMMY_ONCE:
+		case MODE_DUMMY_LOOP:
 			/* Initialise basic essential variables */
 			for ( item = nut_data ; item->info_type != NULL ; item++ )
 			{
@@ -111,6 +140,7 @@ void upsdrv_initinfo(void)
 
 			dstate_dataok();
 			break;
+
 		case MODE_META:
 		case MODE_REPEATER:
 			/* Obtain the target name */
@@ -139,10 +169,35 @@ void upsdrv_initinfo(void)
 			}
 			/* FIXME: commands and settable variable! */
 			break;
-		default:
+
 		case MODE_NONE:
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_COVERED_SWITCH_DEFAULT) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_UNREACHABLE_CODE) )
+# pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_COVERED_SWITCH_DEFAULT
+# pragma GCC diagnostic ignored "-Wcovered-switch-default"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_UNREACHABLE_CODE
+# pragma GCC diagnostic ignored "-Wunreachable-code"
+#endif
+/* Older CLANG (e.g. clang-3.4) seems to not support the GCC pragmas above */
+#ifdef __clang__
+# pragma clang diagnostic push
+# pragma clang diagnostic ignored "-Wunreachable-code"
+# pragma clang diagnostic ignored "-Wcovered-switch-default"
+#endif
+		/* All enum cases defined as of the time of coding
+		 * have been covered above. Handle later definitions,
+		 * memory corruptions and buggy inputs below...
+		 */
+		default:
 			fatalx(EXIT_FAILURE, "no suitable definition found!");
-			break;
+#ifdef __clang__
+# pragma clang diagnostic pop
+#endif
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_COVERED_SWITCH_DEFAULT) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_UNREACHABLE_CODE) )
+# pragma GCC diagnostic pop
+#endif
 	}
 	upsh.instcmd = instcmd;
 
@@ -157,11 +212,48 @@ void upsdrv_updateinfo(void)
 
 	switch (mode)
 	{
-		case MODE_DUMMY:
+		case MODE_DUMMY_LOOP:
 			/* Now get user's defined variables */
 			if (parse_data_file(upsfd) >= 0)
 				dstate_dataok();
 			break;
+
+		case MODE_DUMMY_ONCE:
+			/* less stress on the sys */
+			if (ctx == NULL && next_update == -1) {
+				struct stat	fs;
+				char fn[SMALLBUF];
+
+				if (device_path[0] == '/')
+					snprintf(fn, sizeof(fn), "%s", device_path);
+				else
+					snprintf(fn, sizeof(fn), "%s/%s", confpath(), device_path);
+
+				if (0 != fstat (upsfd, &fs) && 0 != stat (fn, &fs)) {
+					upsdebugx(2, "Can't open %s currently", fn);
+					/* retry ASAP until we get a file */
+					memset(&datafile_stat, 0, sizeof(struct stat));
+					next_update = 1;
+				} else {
+					if (datafile_stat.st_mtim.tv_sec != fs.st_mtim.tv_sec) {
+						upsdebugx(2, "upsdrv_updateinfo: input file was already read once to the end, but changed later - re-reading");
+						/* updated file => retry ASAP */
+						next_update = 1;
+						datafile_stat = fs;
+					}
+				}
+			}
+
+			if (ctx == NULL && next_update == -1) {
+				upsdebugx(2, "upsdrv_updateinfo: NO-OP: input file was already read once to the end");
+				dstate_dataok();
+			} else {
+				/* initial parsing interrupted by e.g. TIMER line */
+				if (parse_data_file(upsfd) >= 0)
+					dstate_dataok();
+			}
+			break;
+
 		case MODE_META:
 		case MODE_REPEATER:
 			if (upsclient_update_vars() > 0)
@@ -182,19 +274,48 @@ void upsdrv_updateinfo(void)
 				}
 			}
 			break;
+
 		case MODE_NONE:
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_COVERED_SWITCH_DEFAULT) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_UNREACHABLE_CODE) )
+# pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_COVERED_SWITCH_DEFAULT
+# pragma GCC diagnostic ignored "-Wcovered-switch-default"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_UNREACHABLE_CODE
+# pragma GCC diagnostic ignored "-Wunreachable-code"
+#endif
+/* Older CLANG (e.g. clang-3.4) seems to not support the GCC pragmas above */
+#ifdef __clang__
+# pragma clang diagnostic push
+# pragma clang diagnostic ignored "-Wunreachable-code"
+# pragma clang diagnostic ignored "-Wcovered-switch-default"
+#endif
+		/* All enum cases defined as of the time of coding
+		 * have been covered above. Handle later definitions,
+		 * memory corruptions and buggy inputs below...
+		 */
 		default:
 			break;
+#ifdef __clang__
+# pragma clang diagnostic pop
+#endif
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_COVERED_SWITCH_DEFAULT) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_UNREACHABLE_CODE) )
+# pragma GCC diagnostic pop
+#endif
 	}
 }
 
+void upsdrv_shutdown(void)
+	__attribute__((noreturn));
+
 void upsdrv_shutdown(void)
 {
 	fatalx(EXIT_FAILURE, "shutdown not supported");
 }
 
 static int instcmd(const char *cmdname, const char *extra)
-{	
+{
 /*
 	if (!strcasecmp(cmdname, "test.battery.stop")) {
 		ser_send_buf(upsfd, ...);
@@ -206,7 +327,7 @@ static int instcmd(const char *cmdname, const char *extra)
 	 * if (mode == MODE_META) => ?
 	 */
 
-	upslogx(LOG_NOTICE, "instcmd: unknown command [%s]", cmdname);
+	upslogx(LOG_NOTICE, "instcmd: unknown command [%s] [%s]", cmdname, extra);
 	return STAT_INSTCMD_UNKNOWN;
 }
 
@@ -216,13 +337,30 @@ void upsdrv_help(void)
 
 void upsdrv_makevartable(void)
 {
+	addvar(VAR_VALUE,	"mode",	"Specify mode instead of guessing it from port value (dummy = dummy-loop, dummy-once, repeater)"); /* meta */
 }
 
 void upsdrv_initups(void)
 {
+	const char *val;
+
+	val = dstate_getinfo("driver.parameter.mode");
+	if (val) {
+		if (!strcmp(val, "dummy-loop")
+		&&  !strcmp(val, "dummy-once")
+		&&  !strcmp(val, "dummy")
+		&&  !strcmp(val, "repeater")
+		/* &&  !strcmp(val, "meta") */
+		) {
+			fatalx(EXIT_FAILURE, "Unsupported mode was specified: %s", val);
+		}
+	}
+
 	/* check the running mode... */
-	if (strchr(device_path, '@'))
-	{
+	if ( (!val && strchr(device_path, '@'))
+	||   (val && !strcmp(val, "repeater"))
+	/*||   (val && !strcmp(val, "meta")) */
+	) {
 		upsdebugx(1, "Repeater mode");
 		mode = MODE_REPEATER;
 		dstate_setinfo("driver.parameter.mode", "repeater");
@@ -230,9 +368,95 @@ void upsdrv_initups(void)
 	}
 	else
 	{
-		upsdebugx(1, "Dummy (simulation) mode");
-		mode = MODE_DUMMY;
-		dstate_setinfo("driver.parameter.mode", "dummy");
+		char fn[SMALLBUF];
+		mode = MODE_NONE;
+
+		if (val) {
+			if (!strcmp(val, "dummy-loop")) {
+				upsdebugx(2, "Dummy (simulation) mode looping infinitely was explicitly requested");
+				mode = MODE_DUMMY_LOOP;
+			} else
+			if (!strcmp(val, "dummy-once")) {
+				upsdebugx(2, "Dummy (simulation) mode with data read once was explicitly requested");
+				mode = MODE_DUMMY_ONCE;
+			} else
+			if (!strcmp(val, "dummy")) {
+				upsdebugx(2, "Dummy (simulation) mode default (looping infinitely) was explicitly requested");
+				mode = MODE_DUMMY_LOOP;
+			}
+		}
+
+		if (mode == MODE_NONE) {
+			if (str_ends_with(device_path, ".seq")) {
+				upsdebugx(2, "Dummy (simulation) mode with a sequence file name pattern (looping infinitely)");
+				mode = MODE_DUMMY_LOOP;
+			} else if (str_ends_with(device_path, ".dev")) {
+				upsdebugx(2, "Dummy (simulation) mode with a device data dump file name pattern (read once)");
+				mode = MODE_DUMMY_ONCE;
+			}
+		}
+
+		/* Report decisions similar to those above,
+		 * just a bit shorter and at another level */
+		switch (mode) {
+			case MODE_DUMMY_ONCE:
+				upsdebugx(1, "Dummy (simulation) mode using data read once");
+				dstate_setinfo("driver.parameter.mode", "dummy-once");
+				break;
+
+			case MODE_DUMMY_LOOP:
+				upsdebugx(1, "Dummy (simulation) mode looping infinitely");
+				dstate_setinfo("driver.parameter.mode", "dummy-loop");
+				break;
+
+			case MODE_NONE:
+			case MODE_REPEATER:
+			case MODE_META:
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_COVERED_SWITCH_DEFAULT) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_UNREACHABLE_CODE) )
+# pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_COVERED_SWITCH_DEFAULT
+# pragma GCC diagnostic ignored "-Wcovered-switch-default"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_UNREACHABLE_CODE
+# pragma GCC diagnostic ignored "-Wunreachable-code"
+#endif
+/* Older CLANG (e.g. clang-3.4) seems to not support the GCC pragmas above */
+#ifdef __clang__
+# pragma clang diagnostic push
+# pragma clang diagnostic ignored "-Wunreachable-code"
+# pragma clang diagnostic ignored "-Wcovered-switch-default"
+#endif
+		/* All enum cases defined as of the time of coding
+		 * have been covered above. Handle later definitions,
+		 * memory corruptions and buggy inputs below...
+		 */
+			default:
+				/* This was the only mode until MODE_DUMMY_LOOP
+				 * got split from MODE_DUMMY_ONCE in NUT v2.8.0
+				 * so we keep the previously known mode string
+				 * and it remains default when we are not sure
+				 */
+				upsdebugx(1, "Dummy (simulation) mode default (looping infinitely)");
+				mode = MODE_DUMMY_LOOP;
+				dstate_setinfo("driver.parameter.mode", "dummy");
+				break;
+#ifdef __clang__
+# pragma clang diagnostic pop
+#endif
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_COVERED_SWITCH_DEFAULT) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_UNREACHABLE_CODE) )
+# pragma GCC diagnostic pop
+#endif
+		}
+
+		if (device_path[0] == '/')
+			snprintf(fn, sizeof(fn), "%s", device_path);
+		else
+			snprintf(fn, sizeof(fn), "%s/%s", confpath(), device_path);
+
+		if (0 != fstat (upsfd, &datafile_stat) && 0 != stat (device_path, &datafile_stat)) {
+			upsdebugx(2, "Can't open %s currently", device_path);
+		}
 	}
 }
 
@@ -326,7 +550,7 @@ static int setvar(const char *varname, const char *val)
 static int upsclient_update_vars(void)
 {
 	int		ret;
-	unsigned int	numq, numa;
+	size_t	numq, numa;
 	const char	*query[4];
 	char		**answer;
 
@@ -347,7 +571,7 @@ static int upsclient_update_vars(void)
 		/* VAR <upsname> <varname> <val> */
 		if (numa < 4)
 		{
-			upsdebugx(1, "Error: insufficient data (got %d args, need at least 4)", numa);
+			upsdebugx(1, "Error: insufficient data (got %zu args, need at least 4)", numa);
 		}
 
 		upsdebugx(5, "Received: %s %s %s %s",
@@ -399,6 +623,7 @@ static int is_valid_data(const char* varname)
 static int is_valid_value(const char* varname, const char *value)
 {
 	dummy_info_t *item;
+	NUT_UNUSED_VARIABLE(value);
 
 	if ( (item = find_info(varname)) != NULL)
 	{
@@ -423,13 +648,14 @@ static void upsconf_err(const char *errmsg)
 
 /* for dummy mode
  * parse the definition file and process its content
- */ 
-static int parse_data_file(int upsfd)
+ */
+static int parse_data_file(int arg_upsfd)
 {
 	char	fn[SMALLBUF];
 	char	*ptr, var_value[MAX_STRING_SIZE];
-	int		value_args = 0, counter;
+	size_t	value_args = 0, counter;
 	time_t	now;
+	NUT_UNUSED_VARIABLE(arg_upsfd);
 
 	time(&now);
 
@@ -531,7 +757,7 @@ static int parse_data_file(int upsfd)
 					ctx->arglist[0], var_value, ctx->errmsg);
 			}
 			else
-			{ 
+			{
 				upsdebugx(3, "parse_data_file: added \"%s\" with value \"%s\"",
 					ctx->arglist[0], var_value);
 			}
