@@ -22,10 +22,11 @@
  *
  * -------------------------------------------------------------------------- */
 
+#include "config.h" /* must be first */
+
 #include <string.h>
 #include <stdlib.h>
 
-#include "config.h"
 #include "hidparser.h"
 #include "nut_stdint.h"  /* for int8_t, int16_t, int32_t */
 #include "common.h"      /* for fatalx() */
@@ -35,13 +36,16 @@ static const uint8_t ItemSize[4] = { 0, 1, 2, 4 };
 /*
  * HIDParser struct
  * -------------------------------------------------------------------------- */
+/* FIXME? Should this structure remain with reasonable fixed int types,
+ * or changed to align with libusb API version and usb_ctrl_* typedefs?
+ */
 typedef struct {
 	const unsigned char	*ReportDesc;		/* Report Descriptor		*/
-	int			ReportDescSize;		/* Size of Report Descriptor	*/
+	size_t			ReportDescSize;		/* Size of Report Descriptor	*/
 
 	uint16_t	Pos;				/* Store current pos in descriptor	*/
 	uint8_t		Item;				/* Store current Item		*/
-	long		Value;				/* Store current Value		*/
+	uint32_t	Value;				/* Store current Value		*/
 
 	HIDData_t	Data;				/* Store current environment	*/
 
@@ -56,7 +60,7 @@ typedef struct {
 
 /* return 1 + the position of the leftmost "1" bit of an int, or 0 if
    none. */
-static inline unsigned int hibit(unsigned int x)
+static inline unsigned int hibit(unsigned long x)
 {
 	unsigned int	res = 0;
 
@@ -91,7 +95,7 @@ static void ResetLocalState(HIDParser_t* pParser)
 /*
  * GetReportOffset
  *
- * Return pointer on current offset value for Report designed by 
+ * Return pointer on current offset value for Report designed by
  * ReportID/ReportType
  * -------------------------------------------------------------------------- */
 static uint8_t *GetReportOffset(HIDParser_t* pParser, const uint8_t ReportID, const uint8_t ReportType)
@@ -121,10 +125,10 @@ static uint8_t *GetReportOffset(HIDParser_t* pParser, const uint8_t ReportID, co
 }
 
 /*
- * FormatValue(long Value, uint8_t Size)
+ * FormatValue(uint32_t Value, uint8_t Size)
  * Format Value to fit with long format with respect of negative values
  * -------------------------------------------------------------------------- */
-static long FormatValue(long Value, uint8_t Size)
+static long FormatValue(uint32_t Value, uint8_t Size)
 {
 	switch(Size)
 	{
@@ -135,7 +139,7 @@ static long FormatValue(long Value, uint8_t Size)
 	case 4:
 		return (long)(int32_t)Value;
 	default:
-		return Value;
+		return (long)Value;
 	}
 }
 
@@ -143,34 +147,23 @@ static long FormatValue(long Value, uint8_t Size)
  * HIDParse(HIDParser_t* pParser, HIDData_t *pData)
  *
  * Analyse Report descriptor stored in HIDParser struct and store local and
- * global context. 
+ * global context.
  * Return in pData the last object found.
  * Return -1 when there is no other Item to parse, 1 if a new object was found
  * or 0 if a continuation of a previous object was found.
  * -------------------------------------------------------------------------- */
 static int HIDParse(HIDParser_t *pParser, HIDData_t *pData)
 {
-	int	Found = -1;
+	int	Found = -1, i;
 
 	while ((Found < 0) && (pParser->Pos < pParser->ReportDescSize)) {
 		/* Get new pParser->Item if current pParser->Count is empty */
 		if (pParser->Count == 0) {
 			pParser->Item = pParser->ReportDesc[pParser->Pos++];
 			pParser->Value = 0;
-#if WORDS_BIGENDIAN
-			{
-				int	i;
-				unsigned long	valTmp = 0;
-
-				for (i = 0; i < ItemSize[pParser->Item & SIZE_MASK]; i++) {
-					memcpy(&valTmp, &pParser->ReportDesc[(pParser->Pos)+i], 1);
-					pParser->Value += valTmp >> ((3-i)*8);
-					valTmp = 0;
-				}
+			for (i = 0; i < ItemSize[pParser->Item & SIZE_MASK]; i++) {
+				pParser->Value += (uint32_t)(pParser->ReportDesc[(pParser->Pos)+i]) << (8*i);
 			}
-#else
-			memcpy(&pParser->Value, &pParser->ReportDesc[pParser->Pos], ItemSize[pParser->Item & SIZE_MASK]);
-#endif
 			/* Pos on next item */
 			pParser->Pos += ItemSize[pParser->Item & SIZE_MASK];
 		}
@@ -187,7 +180,7 @@ static int HIDParse(HIDParser_t *pParser, HIDData_t *pData)
 			if ((pParser->Item & SIZE_MASK) > 2) {
 				pParser->UsageTab[pParser->UsageSize] = pParser->Value;
 			} else {
-				pParser->UsageTab[pParser->UsageSize] = (pParser->UPage << 16) | (pParser->Value & 0xFFFF);
+				pParser->UsageTab[pParser->UsageSize] = ((HIDNode_t)(pParser->UPage) << 16) | (pParser->Value & 0xFFFF);
 			}
 
 			/* Increment Usage stack size */
@@ -201,10 +194,10 @@ static int HIDParse(HIDParser_t *pParser, HIDData_t *pData)
 
 			/* Unstack UPage/Usage from UsageTab (never remove the last) */
 			if (pParser->UsageSize > 0) {
-				int	i;
-			
-				for (i = 0; i < pParser->UsageSize; i++) {
-					pParser->UsageTab[i] = pParser->UsageTab[i+1];
+				int	j;
+
+				for (j = 0; j < pParser->UsageSize; j++) {
+					pParser->UsageTab[j] = pParser->UsageTab[j+1];
 				}
 
 				/* Remove Usage */
@@ -222,15 +215,15 @@ static int HIDParse(HIDParser_t *pParser, HIDData_t *pData)
 
 		case ITEM_END_COLLECTION :
 			pParser->Data.Path.Size--;
-			
+
 			/* Remove Index if any */
 			if((pParser->Data.Path.Node[pParser->Data.Path.Size] & 0xffff0000) == 0x00ff0000) {
 				pParser->Data.Path.Size--;
 			}
-			
+
 			ResetLocalState(pParser);
 			break;
-			
+
 		case ITEM_FEATURE:
 		case ITEM_INPUT:
 		case ITEM_OUTPUT:
@@ -241,7 +234,7 @@ static int HIDParse(HIDParser_t *pParser, HIDData_t *pData)
 				/* It is a continuation of a previous object */
 				Found = 0;
 			}
-			
+
 			/* Get new pParser->Count from global value */
 			if(pParser->Count == 0) {
 				pParser->Count = pParser->ReportCount;
@@ -250,87 +243,109 @@ static int HIDParse(HIDParser_t *pParser, HIDData_t *pData)
 			/* Get UPage/Usage from UsageTab and store them in pParser->Data.Path */
 			pParser->Data.Path.Node[pParser->Data.Path.Size] = pParser->UsageTab[0];
 			pParser->Data.Path.Size++;
-			
+
 			/* Unstack UPage/Usage from UsageTab (never remove the last) */
 			if(pParser->UsageSize > 0) {
-				int i;
-				
-				for (i = 0; i < pParser->UsageSize; i++) {
-					pParser->UsageTab[i] = pParser->UsageTab[i+1];
+				int j;
+
+				for (j = 0; j < pParser->UsageSize; j++) {
+					pParser->UsageTab[j] = pParser->UsageTab[j+1];
 				}
+
 				/* Remove Usage */
 				pParser->UsageSize--;
 			}
-			
+
 			/* Copy data type */
 			pParser->Data.Type = (uint8_t)(pParser->Item & ITEM_MASK);
-			
+
 			/* Copy data attribute */
 			pParser->Data.Attribute = (uint8_t)pParser->Value;
-			
+
 			/* Store offset */
 			pParser->Data.Offset = *GetReportOffset(pParser, pParser->Data.ReportID, (uint8_t)(pParser->Item & ITEM_MASK));
-			
+
 			/* Get Object in pData */
 			/* -------------------------------------------------------------------------- */
 			memcpy(pData, &pParser->Data, sizeof(HIDData_t));
 			/* -------------------------------------------------------------------------- */
-			
+
 			/* Increment Report Offset */
 			*GetReportOffset(pParser, pParser->Data.ReportID, (uint8_t)(pParser->Item & ITEM_MASK)) += pParser->Data.Size;
-			
+
 			/* Remove path last node */
 			pParser->Data.Path.Size--;
-			
+
 			/* Decrement count */
 			pParser->Count--;
-			
+
 			if (pParser->Count == 0) {
 				ResetLocalState(pParser);
 			}
 			break;
-			
+
 		case ITEM_REP_ID :
 			pParser->Data.ReportID = (uint8_t)pParser->Value;
 			break;
-			
+
 		case ITEM_REP_SIZE :
 			pParser->Data.Size = (uint8_t)pParser->Value;
 			break;
-			
+
 		case ITEM_REP_COUNT :
 			pParser->ReportCount = (uint8_t)pParser->Value;
 			break;
-			
+
 		case ITEM_UNIT_EXP :
 			pParser->Data.UnitExp = (int8_t)pParser->Value;
 			if (pParser->Data.UnitExp > 7) {
 				pParser->Data.UnitExp |= 0xF0;
 			}
 			break;
-			
+
 		case ITEM_UNIT :
-			pParser->Data.Unit = pParser->Value;
+			/* TOTHINK: Are there cases where Unit is not-signed,
+			 * but a Value too big becomes signed after casting --
+			 * and unintentionally so? */
+			pParser->Data.Unit = (long)pParser->Value;
 			break;
-			
+
 		case ITEM_LOG_MIN :
 			pParser->Data.LogMin = FormatValue(pParser->Value, ItemSize[pParser->Item & SIZE_MASK]);
 			break;
-			
+
 		case ITEM_LOG_MAX :
 			pParser->Data.LogMax = FormatValue(pParser->Value, ItemSize[pParser->Item & SIZE_MASK]);
+			/* HACK: If treating the value as signed (FormatValue(...)) results in a LogMax that is
+			 * less than the LogMin value then it is likely that the LogMax value has been
+			 * incorrectly encoded by the UPS firmware (field too short and overflowed into sign
+			 * bit).  In that case, reinterpret it as an unsigned number and log the problem.
+			 * This hack is not correct in the sense that it only looks at the LogMin value for
+			 * this item, whereas the HID spec says that Logical values persist in global state.
+			 */
+			if (pParser->Data.LogMax < pParser->Data.LogMin) {
+				upslogx(LOG_WARNING,
+					"%s: LogMax is less than LogMin. "
+					"Vendor HID report descriptor may be incorrect; "
+					"interpreting LogMax %ld as %u in ReportID: 0x%02x",
+					__func__,
+					pParser->Data.LogMax,
+					pParser->Value,
+					pParser->Data.ReportID);
+				pParser->Data.LogMax = (long) pParser->Value;
+			}
 			break;
-			
+
 		case ITEM_PHY_MIN :
 			pParser->Data.PhyMin=FormatValue(pParser->Value, ItemSize[pParser->Item & SIZE_MASK]);
 			pParser->Data.have_PhyMin = 1;
 			break;
-			
+
 		case ITEM_PHY_MAX :
 			pParser->Data.PhyMax=FormatValue(pParser->Value, ItemSize[pParser->Item & SIZE_MASK]);
 			pParser->Data.have_PhyMax = 1;
 			break;
-			
+
 		case ITEM_LONG :
 			/* can't handle long items, but should at least skip them */
 			pParser->Pos += (uint8_t)(pParser->Value & 0xff);
@@ -344,8 +359,13 @@ static int HIDParse(HIDParser_t *pParser, HIDData_t *pData)
 		upslogx(LOG_ERR, "%s: Report descriptor too big", __func__);
 	if(pParser->UsageSize >= USAGE_TAB_SIZE)
 		upslogx(LOG_ERR, "%s: HID Usage too high", __func__);
+
+	/* FIXME: comparison is always false due to limited range of data type [-Werror=type-limits]
+	 * with ReportID beint uint8_t and MAX_REPORT being 500 currently */
+	/*
 	if(pParser->Data.ReportID >= MAX_REPORT)
 		upslogx(LOG_ERR, "%s: Too many HID reports", __func__);
+	*/
 
 	return Found;
 }
@@ -355,14 +375,14 @@ static int HIDParse(HIDParser_t *pParser, HIDData_t *pData)
  * Get pData characteristics from pData->Path
  * Return TRUE if object was found
  * -------------------------------------------------------------------------- */
-int FindObject(HIDDesc_t *pDesc, HIDData_t *pData)
+int FindObject(HIDDesc_t *pDesc_arg, HIDData_t *pData)
 {
-	HIDData_t	*pFoundData = FindObject_with_Path(pDesc, &pData->Path, pData->Type);
-	
+	HIDData_t	*pFoundData = FindObject_with_Path(pDesc_arg, &pData->Path, pData->Type);
+
 	if (!pFoundData) {
 		return 0;
 	}
-	
+
 	memcpy(pData, pFoundData, sizeof(*pData));
 	return 1;
 }
@@ -371,24 +391,24 @@ int FindObject(HIDDesc_t *pDesc, HIDData_t *pData)
  * FindObject_with_Path
  * Get pData item with given Path and Type. Return NULL if not found.
  * -------------------------------------------------------------------------- */
-HIDData_t *FindObject_with_Path(HIDDesc_t *pDesc, HIDPath_t *Path, uint8_t Type)
+HIDData_t *FindObject_with_Path(HIDDesc_t *pDesc_arg, HIDPath_t *Path, uint8_t Type)
 {
-	int	i;
+	size_t	i;
+
+	for (i = 0; i < pDesc_arg->nitems; i++) {
+		HIDData_t *pData = &pDesc_arg->item[i];
 
-	for (i = 0; i < pDesc->nitems; i++) {
-		HIDData_t *pData = &pDesc->item[i];
-		
 		if (pData->Type != Type) {
 			continue;
 		}
-		
+
 		if (memcmp(pData->Path.Node, Path->Node, (Path->Size) * sizeof(HIDNode_t))) {
 			continue;
 		}
-		
+
 		return pData;
 	}
-	
+
 	return NULL;
 }
 
@@ -397,50 +417,85 @@ HIDData_t *FindObject_with_Path(HIDDesc_t *pDesc, HIDPath_t *Path, uint8_t Type)
  * Get pData item with given ReportID, Offset, and Type. Return NULL
  * if not found.
  * -------------------------------------------------------------------------- */
-HIDData_t *FindObject_with_ID(HIDDesc_t *pDesc, uint8_t ReportID, uint8_t Offset, uint8_t Type)
+HIDData_t *FindObject_with_ID(HIDDesc_t *pDesc_arg, uint8_t ReportID, uint8_t Offset, uint8_t Type)
 {
-	int	i;
+	size_t	i;
+
+	for (i = 0; i < pDesc_arg->nitems; i++) {
+		HIDData_t *pData = &pDesc_arg->item[i];
 
-	for (i = 0; i < pDesc->nitems; i++) {
-		HIDData_t *pData = &pDesc->item[i];
-		
 		if (pData->ReportID != ReportID) {
 			continue;
 		}
-		
+
 		if (pData->Type != Type) {
 			continue;
 		}
- 
+
 		if (pData->Offset != Offset) {
 			continue;
 		}
-		
+
 		return pData;
 	}
-	
+
+	return NULL;
+}
+
+/*
+ * FindObject_with_ID_Node
+ * Get pData item with given ReportID and Node. Return NULL if not found.
+ * -------------------------------------------------------------------------- */
+HIDData_t *FindObject_with_ID_Node(HIDDesc_t *pDesc_arg, uint8_t ReportID, HIDNode_t Node)
+{
+	size_t	i;
+
+	for (i = 0; i < pDesc_arg->nitems; i++) {
+		HIDData_t *pData = &pDesc_arg->item[i];
+
+		if (pData->ReportID != ReportID) {
+			continue;
+		}
+
+		HIDPath_t * pPath = &pData->Path;
+		uint8_t size = pPath->Size;
+		if (size == 0 || pPath->Node[size-1] != Node) {
+			continue;
+		}
+
+		return pData;
+	}
+
 	return NULL;
 }
 
 /*
  * GetValue
  * Extract data from a report stored in Buf.
- * Use Value, Offset, Size and LogMax of pData.
- * Return response in Value.
+ * Use Offset, Size, LogMin, and LogMax of pData.
+ * Return response in *pValue.
  * -------------------------------------------------------------------------- */
 void GetValue(const unsigned char *Buf, HIDData_t *pData, long *pValue)
 {
+	/* Note:  https://github.com/networkupstools/nut/issues/1023
+	   This conversion code can easily be sensitive to 32- vs 64- bit
+	   compilation environments.  Consider the possibility of overflow
+	   in 32-bit representations when computing with extreme values,
+	   for example LogMax-LogMin+1.
+	   Test carefully in both environments if changing any declarations.
+	*/
+
 	int	Weight, Bit;
-	long	value = 0, rawvalue;
-	long	range, mask, signbit, b, m;
+	unsigned long mask, signbit, magMax, magMin;
+	long	value = 0;
 
 	Bit = pData->Offset + 8;	/* First byte of report is report ID */
-	
+
 	for (Weight = 0; Weight < pData->Size; Weight++, Bit++) {
 		int	State = Buf[Bit >> 3] & (1 << (Bit & 7));
-		
+
 		if(State) {
-			value += (1 << Weight);
+			value += (1L << Weight);
 		}
 	}
 
@@ -466,49 +521,25 @@ void GetValue(const unsigned char *Buf, HIDData_t *pData, long *pValue)
 	"throwing away higher-order bits" exacly means, so we try to do
 	something sensible. -PS */
 
-	rawvalue = value; /* remember this for later */
+	/* determine representation without sign bit */
+	magMax = pData->LogMax >= 0 ? (unsigned long)(pData->LogMax) : (unsigned long)(-(pData->LogMax + 1));
+	magMin = pData->LogMin >= 0 ? (unsigned long)(pData->LogMin) : (unsigned long)(-(pData->LogMin + 1));
 
-	/* figure out how many bits are significant */
-	range = pData->LogMax - pData->LogMin + 1;
-	if (range <= 0) {
-		/* makes no sense, give up */
-		*pValue = value;
-		return;
-	}
-	b = hibit(range-1);
+	/* calculate where the sign bit will be if needed */
+	signbit = 1L << hibit(magMax > magMin ? magMax : magMin);
 
-	/* throw away insignificant bits; the result is >= 0 */
-	mask = (1 << b) - 1;
-	signbit = 1 << (b - 1);
-	value = value & mask;
+	/* but only include sign bit in mask if negative numbers are involved */
+	mask = (signbit - 1) | ((pData->LogMin < 0) ? signbit : 0);
+
+	/* throw away excess high order bits (which may contain garbage) */
+	value = (long)((unsigned long)(value) & mask);
 
 	/* sign-extend it, if appropriate */
-	if (pData->LogMin < 0 && (value & signbit) != 0) {
+	if (pData->LogMin < 0 && ((unsigned long)(value) & signbit) != 0) {
 		value |= ~mask;
 	}
 
-	/* if the resulting value is in the desired range, stop */
-	if (value >= pData->LogMin && value <= pData->LogMax) {
-		*pValue = value;
-		return;
-	}
-
-	/* else, try to reach interval by adjusting high-order bits */
-	m = (value - pData->LogMin) & mask;
-	value = pData->LogMin + m;
-	if (value <= pData->LogMax) {
-		*pValue = value;
-		return;
-	}
-
-	/* if everything else failed, sign-extend the original raw value,
-	and simply round it to the closest point in the interval. */
-	value = rawvalue;
-	mask = (1 << pData->Size) - 1;
-	signbit = 1 << (pData->Size - 1);
-	if (pData->LogMin < 0 && (value & signbit) != 0) {
-		value |= ~mask;
-	}
+	/* clamp returned value to range [LogMin..LogMax] */
 	if (value < pData->LogMin) {
 		value = pData->LogMin;
 	} else if (value > pData->LogMax) {
@@ -527,11 +558,11 @@ void GetValue(const unsigned char *Buf, HIDData_t *pData, long *pValue)
 void SetValue(const HIDData_t *pData, unsigned char *Buf, long Value)
 {
 	int	Weight, Bit;
-	
+
 	Bit = pData->Offset + 8;	/* First byte of report is report ID */
 
 	for (Weight = 0; Weight < pData->Size; Weight++, Bit++) {
-		int	State = Value & (1 << Weight);
+		long	State = Value & (1L << Weight);
 
 		if (State) {
 			Buf[Bit >> 3] |= (1 << (Bit & 7));
@@ -547,78 +578,109 @@ void SetValue(const HIDData_t *pData, unsigned char *Buf, long Value)
    Output: parsed data structure. Returns allocated HIDDesc structure
    on success, NULL on failure with errno set. Note: the value
    returned by this function must be freed with Free_ReportDesc(). */
-HIDDesc_t *Parse_ReportDesc(const unsigned char *ReportDesc, const int n)
+HIDDesc_t *Parse_ReportDesc(const usb_ctrl_charbuf ReportDesc, const usb_ctrl_charbufsize n)
 {
-	int		ret;
-	HIDDesc_t	*pDesc;
+	int		ret = 0;
+	HIDDesc_t	*pDesc_var;
 	HIDParser_t	*parser;
 
-	pDesc = calloc(1, sizeof(*pDesc));
-	if (!pDesc) {
+	pDesc_var = calloc(1, sizeof(*pDesc_var));
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_UNSIGNED_ZERO_COMPARE) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_UNREACHABLE_CODE) )
+# pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS
+# pragma GCC diagnostic ignored "-Wtype-limits"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE
+# pragma GCC diagnostic ignored "-Wtautological-constant-out-of-range-compare"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_UNSIGNED_ZERO_COMPARE
+# pragma GCC diagnostic ignored "-Wtautological-unsigned-zero-compare"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_UNREACHABLE_CODE
+#pragma GCC diagnostic ignored "-Wunreachable-code"
+#endif
+/* Older CLANG (e.g. clang-3.4) seems to not support the GCC pragmas above */
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wunreachable-code"
+#pragma clang diagnostic ignored "-Wtautological-compare"
+#pragma clang diagnostic ignored "-Wtautological-constant-out-of-range-compare"
+#endif
+	if (!pDesc_var
+	|| n < 0 || (uintmax_t)n > SIZE_MAX
+	) {
 		return NULL;
 	}
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_UNSIGNED_ZERO_COMPARE) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_UNREACHABLE_CODE) )
+# pragma GCC diagnostic pop
+#endif
 
-	pDesc->item = calloc(MAX_REPORT, sizeof(*pDesc->item));
-	if (!pDesc->item) {
-		Free_ReportDesc(pDesc);
+	pDesc_var->item = calloc(MAX_REPORT, sizeof(*pDesc_var->item));
+	if (!pDesc_var->item) {
+		Free_ReportDesc(pDesc_var);
 		return NULL;
 	}
 
 	parser = calloc(1, sizeof(*parser));
 	if (!parser) {
-		Free_ReportDesc(pDesc);
+		Free_ReportDesc(pDesc_var);
 		return NULL;
 	}
 
-	parser->ReportDesc = ReportDesc;
-	parser->ReportDescSize = n;
+	parser->ReportDesc = (const unsigned char *)ReportDesc;
+	parser->ReportDescSize = (const size_t)n;
 
-	for (pDesc->nitems = 0; pDesc->nitems < MAX_REPORT; pDesc->nitems += ret) {
-		int	id, max;
+	for (pDesc_var->nitems = 0; pDesc_var->nitems < MAX_REPORT; pDesc_var->nitems += (size_t)ret) {
+		uint8_t	id;
+		size_t	max;
 
-		ret = HIDParse(parser, &pDesc->item[pDesc->nitems]);
+		ret = HIDParse(parser, &pDesc_var->item[pDesc_var->nitems]);
 		if (ret < 0) {
 			break;
 		}
 
-		id = pDesc->item[pDesc->nitems].ReportID;
+		id = pDesc_var->item[pDesc_var->nitems].ReportID;
 
 		/* calculate bit range of this item within report */
-		max = pDesc->item[pDesc->nitems].Offset + pDesc->item[pDesc->nitems].Size;
+		max = pDesc_var->item[pDesc_var->nitems].Offset + pDesc_var->item[pDesc_var->nitems].Size;
 
 		/* convert to bytes */
 		max = (max + 7) >> 3;
 
 		/* update report length */
-		if (max > pDesc->replen[id]) {
-			pDesc->replen[id] = max;
+		if (max > pDesc_var->replen[id]) {
+			pDesc_var->replen[id] = max;
 		}
 	}
 
 	/* Sanity check: are there remaining HID objects that can't
 	 * be processed? */
-	if ((pDesc->nitems == MAX_REPORT) && (parser->Pos < parser->ReportDescSize))
+	if ((pDesc_var->nitems == MAX_REPORT) && (parser->Pos < parser->ReportDescSize))
 		upslogx(LOG_ERR, "ERROR in %s: Too many HID objects", __func__);
 
 	free(parser);
 
-	if (pDesc->nitems == 0) {
-		Free_ReportDesc(pDesc);
+	if (pDesc_var->nitems == 0) {
+		Free_ReportDesc(pDesc_var);
 		return NULL;
 	}
 
-	pDesc->item = realloc(pDesc->item, pDesc->nitems * sizeof(*pDesc->item));
+	pDesc_var->item = realloc(pDesc_var->item, pDesc_var->nitems * sizeof(*pDesc_var->item));
 
-	return pDesc;
+	return pDesc_var;
 }
 
 /* free a parsed report descriptor, as allocated by Parse_ReportDesc() */
-void Free_ReportDesc(HIDDesc_t *pDesc)
+void Free_ReportDesc(HIDDesc_t *pDesc_arg)
 {
-	if (!pDesc) {
+	if (!pDesc_arg) {
 		return;
 	}
 
-	free(pDesc->item);
-	free(pDesc);
+	free(pDesc_arg->item);
+	free(pDesc_arg);
 }
