@@ -2,6 +2,7 @@
  *
  *  Copyright (C)
  *  2009	Richard Gregory <r.gregory liverpool ac uk>
+ *  2017	Nash Kaminski <nashkaminski at kaminski dot io>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,53 +28,53 @@
 #define IsBitSet(val, bit) ((val) & (1 << (bit)))
 
 #define DRIVER_NAME	"Liebert ESP-II serial UPS driver"
-#define DRIVER_VERSION	"0.03"
+#define DRIVER_VERSION	"0.05"
 #define UPS_SHUTDOWN_DELAY 12 /* it means UPS will be shutdown 120 sec */
 #define SHUTDOWN_CMD_LEN  8
 
 /* values for sending to UPS */
 enum mult_enum {
-        M_10,
-        M_0_1,
-        M_VOLTAGE_I,
-        M_VOLTAGE_O,
-        M_VOLTAGE_B,
-        M_CURRENT_I,
-        M_CURRENT_O,
-        M_CURRENT_B,
-        M_LOAD_VA,
-        M_LOAD_WATT,
-        M_FREQUENCY,
-        M_VOLT_DC,
-        M_TEMPERATURE,
-        M_CURRENT_DC ,
-        M_BAT_RUNTIME,
-        M_NOMPOWER,
-        M_POWER,
-        M_REALPOWER,
-        M_LOADPERC      
+	M_10,
+	M_0_1,
+	M_VOLTAGE_I,
+	M_VOLTAGE_O,
+	M_VOLTAGE_B,
+	M_CURRENT_I,
+	M_CURRENT_O,
+	M_CURRENT_B,
+	M_LOAD_VA,
+	M_LOAD_WATT,
+	M_FREQUENCY,
+	M_VOLT_DC,
+	M_TEMPERATURE,
+	M_CURRENT_DC ,
+	M_BAT_RUNTIME,
+	M_NOMPOWER,
+	M_POWER,
+	M_REALPOWER,
+	M_LOADPERC
 };
 
 static float multi[19]={
-  10.0, 
-  0.1, 
-  0.1, /* volt */
-  0.1,
-  0.1,
-  0.1, /* curr */
-  0.1,
-  0.1,
-  100.0, /* va */
-  100.0, /* W */
-  0.01,  /* FREQ */
-  0.1,   /* V DC*/
-  0.1,   /* TEMP*/
-  0.01,  /* CUR DC*/
-  60.0,  /* BAT RUNTIME*/
-  100.0, /* NOMPOWER*/
-  100.0, /* POWER*/
-  100.0, /* REAL POWER*/
-  1.0    /* LOADPERC*/
+	10.0,
+	0.1,
+	0.1, /* volt */
+	0.1,
+	0.1,
+	0.1, /* curr */
+	0.1,
+	0.1,
+	100.0, /* va */
+	100.0, /* W */
+	0.01,  /* FREQ */
+	0.1,   /* V DC*/
+	0.1,   /* TEMP*/
+	0.01,  /* CUR DC*/
+	60.0,  /* BAT RUNTIME*/
+	100.0, /* NOMPOWER*/
+	100.0, /* POWER*/
+	100.0, /* REAL POWER*/
+	1.0    /* LOADPERC*/
 };
 
 static int instcmd(const char *cmdname, const char *extra);
@@ -84,7 +85,8 @@ upsdrv_info_t upsdrv_info = {
 	DRIVER_NAME,
 	DRIVER_VERSION,
 	"Richard Gregory <r.gregory liv ac uk>\n" \
-	"Robert Jobbagy <jobbagy.robert at gmail dot com",
+	"Robert Jobbagy <jobbagy.robert at gmail dot com\n" \
+	"Nash Kaminski <nashkaminski at kaminski dot io>",
 	DRV_EXPERIMENTAL,
 	{ NULL }
 };
@@ -98,19 +100,27 @@ static const unsigned char
 	cmd_bitfield7[]		= { 1,148,2,1,7,159 },	/* AMBIENT_OVERTEMP (2) */
 	cmd_battestres[]	= { 1,148,2,1,12,164 },	/* BATTERY_TEST_RESULT */
 	cmd_selftestres[]	= { 1,148,2,1,13,165 },	/* SELF_TEST_RESULT */
-	cmd_upstype[] 		= { 1,136,2,1,1,141},   /* type bits + number of phases in bit groups*/
-	cmd_scaling1[] 		= { 1,131,2,1,2,137},   /* part of multiplier information*/
+	cmd_upstype[]  		= { 1,136,2,1,1,141}, 	/* type bits + number of phases in bit groups*/
+	cmd_scaling1[] 		= { 1,131,2,1,2,137}, 	/* part of multiplier information*/
 
-	/* Shutdown commands by Robert Jobbagy */	
+	/* Shutdown commands by Robert Jobbagy */
 	cmd_setOutOffMode[]	= { 1,156,4,1,6,0,1,169}, /* UPS OutOffMode command */
 	cmd_setOutOffDelay[] = {1,156,4,1,5,0,UPS_SHUTDOWN_DELAY,167+UPS_SHUTDOWN_DELAY}, /* UPS Shutdown with delay */
-	cmd_sysLoadKey[]    = {1,156,2,1,7,167}, /* UPS SysLoadKey */
-	cmd_shutdown[]		= {1,156,4,1,136,76,76,194}; /* UPS shutdown */
+	cmd_sysLoadKey[]	= {1,156,2,1,7,167}, /* UPS SysLoadKey */
+	cmd_shutdown[]  	= {1,156,4,1,136,76,76,194}; /* UPS shutdown */
 
-static int num_inphases = 1, num_outphases = 1;
+/* Quiesce the compiler warnings about the fields below */
+static void NUT_UNUSED_FUNCTION_dummy_bitfields(void)
+{
+	NUT_UNUSED_VARIABLE(cmd_battestres);
+	NUT_UNUSED_VARIABLE(cmd_selftestres);
+	NUT_UNUSED_VARIABLE(cmd_bitfield7);
+}
+
+static unsigned int num_inphases = 1, num_outphases = 1;
 
 static char cksum(const char *buf, const size_t len)
- {
+{
 	char	sum = 0;
 	size_t	i;
 
@@ -121,34 +131,34 @@ static char cksum(const char *buf, const size_t len)
 	return sum;
 }
 
-static int do_command(const unsigned char *command, char *reply, int cmd_len)
+static ssize_t do_command(const unsigned char *command, char *reply, size_t cmd_len)
 {
-	int	ret;
+	ssize_t	ret;
 
 	ret = ser_send_buf(upsfd, command, cmd_len);
 	if (ret < 0) {
 		upsdebug_with_errno(2, "send");
 		return -1;
-	} else if (ret < cmd_len) {
-		upsdebug_hex(2, "send: truncated", command, ret);
+	} else if ((size_t)ret < cmd_len) {
+		upsdebug_hex(2, "send: truncated", command, (size_t)ret);
 		return -1;
 	}
 
-	upsdebug_hex(2, "send", command, ret);
+	upsdebug_hex(2, "send", command, (size_t)ret);
 
 	ret = ser_get_buf_len(upsfd, reply, 8, 1, 0); /* it needs that this driver works with USB to Serial cable */
 	if (ret < 0) {
 		upsdebug_with_errno(2, "read");
 		return -1;
 	} else if (ret < 6) {
-		upsdebug_hex(2, "read: truncated", reply, ret);
+		upsdebug_hex(2, "read: truncated", reply, (size_t)ret);
 		return -1;
 	} else if (reply[7] != cksum(reply, 7)) {
-		upsdebug_hex(2, "read: checksum error", reply, ret);
+		upsdebug_hex(2, "read: checksum error", reply, (size_t)ret);
 		return -1;
 	}
 
-	upsdebug_hex(2, "read", reply, ret);
+	upsdebug_hex(2, "read", reply, (size_t)ret);
 	return ret;
 }
 
@@ -158,15 +168,16 @@ void upsdrv_initinfo(void)
 		const char	*var;
 		unsigned char	len;
 	} vartab[] = {
-		{ "ups.model",15 },
-		{ "ups.firmware",8 },
-		{ "ups.serial",10 },
-		{ "ups.mfr.date",4 },
-		{ NULL }
+		{ "ups.model", 15 },
+		{ "ups.firmware", 8 },
+		{ "ups.serial", 10 },
+		{ "ups.mfr.date", 4 },
+		{ NULL, 0 }
 	};
 
 	char	buf[LARGEBUF];
-	int	i,bitn,vari,ret=0,offset=4,readok=0;
+	int	i, bitn, vari, offset=4, readok=0;
+	ssize_t	ret=0;
 	char	command[6], reply[8];
 	unsigned int	value;
 
@@ -177,20 +188,20 @@ void upsdrv_initinfo(void)
 
 		for (i = 0; i < vartab[vari].len; i++) {
 			snprintf(command, sizeof(command), "\x01\x88\x02\x01%c", i+offset);
-		command[5] = cksum(command, 5);
+			command[5] = cksum(command, 5);
 
-		ret = do_command((unsigned char *)command, reply, 6);
+			ret = do_command((unsigned char *)command, reply, 6);
 			if (ret < 8) {
-				upsdebug_hex(2, "send: truncated", command, ret);
+				upsdebug_hex(2, "send: truncated", command, (size_t)ret);
 				break;
 			}
 
 			buf[i<<1] = reply[6];
 			buf[(i<<1)+1] = reply[5];
-	}
+		}
 
-	buf[i<<1] = 0;
-		upsdebugx(1, "return: %d (8=success)", ret);
+		buf[i<<1] = 0;
+		upsdebugx(1, "return: %zd (8=success)", ret);
 
 		if (ret == 8) { /* last command successful */
 			dstate_setinfo(vartab[vari].var,"%s",buf);
@@ -206,7 +217,7 @@ void upsdrv_initinfo(void)
 	memcpy(command,cmd_upstype,6);
 	ret = do_command((unsigned char *)command, reply, 6);
 	if (ret < 8) {
-		upsdebug_hex(2, "send: phase detection: truncated", command, ret);
+		upsdebug_hex(2, "send: phase detection: truncated", command, (size_t)ret);
 	}
 	else {
 		/* input: from bit 0 to bit 1 (2 bits) */
@@ -237,18 +248,19 @@ void upsdrv_initinfo(void)
 	}
 
 	/* determine scaling */
-	/* full scaling output not defined yet, but we can differentiate sets of 
+	/* full scaling output not defined yet, but we can differentiate sets of
 	 * multipliers based on a sample scaling reading */
 	memcpy(command,cmd_scaling1,6);
 	ret = do_command((unsigned char *)command, reply, 6);
 	if (ret < 8) {
-		upsdebug_hex(2, "send: scaling detection: truncated", command, ret);
+		upsdebug_hex(2, "send: scaling detection: truncated", command, (size_t)ret);
 	}
 	else { /* add here multipliers that differentiate between models */
 		switch (reply[6]) {
 		case 1: /* GXT-2 */
 			multi[M_FREQUENCY]=0.1;
-			multi[M_VOLT_DC]=1.0;
+			/* Confirmed correct on a Liebert GXT2-6000RT208 */
+			multi[M_VOLT_DC]=0.1;
 			multi[M_POWER]=1.0;
 			multi[M_NOMPOWER]=1.0;
 			break;
@@ -310,16 +322,27 @@ void upsdrv_updateinfo(void)
 		{ { 0 }, NULL, NULL, 0 }
 	};
 
+	static cmd_s vartab2o[] = { /*split-phase out, only V line-line is reported*/
+		{ { 1,144,2,1,24,172 },	"output.L1.power.percent", "%.0f", M_LOADPERC },
+		{ { 1,145,2,1,24,173 },	"output.L2.power.percent", "%.0f", M_LOADPERC },
+		{ { 1,144,2,1,22,170 },	"output.L1.power", "%.0f", M_POWER },
+		{ { 1,145,2,1,22,171 },	"output.L2.power", "%.0f", M_POWER },
+		{ { 1,144,2,1,21,169 },	"output.L1.realpower", "%.0f", M_POWER },
+		{ { 1,145,2,1,21,170 },	"output.L2.realpower", "%.0f", M_POWER },
+		{ { 1,144,2,1,3,151 },	"output.voltage", "%.1f", M_VOLTAGE_O },
+		{ { 0 }, NULL, NULL, 0 }
+	};
+
 	static cmd_s vartab3o[] = { /*3-phase out */
-		{ { 1,144,2,1,24,172 },	"ups.L1.load", "%.0f", M_LOADPERC },
-		{ { 1,145,2,1,24,173 },	"ups.L2.load", "%.0f", M_LOADPERC },
-		{ { 1,146,2,1,24,174 },	"ups.L3.load", "%.0f", M_LOADPERC },
-		{ { 1,144,2,1,22,170 },	"ups.L1.power", "%.0f", M_POWER },
-		{ { 1,145,2,1,22,171 },	"ups.L2.power", "%.0f", M_POWER },
-		{ { 1,146,2,1,22,172 },	"ups.L3.power", "%.0f", M_POWER },
-		{ { 1,144,2,1,21,169 },	"ups.L1.realpower", "%.0f", M_POWER },
-		{ { 1,145,2,1,21,170 },	"ups.L2.realpower", "%.0f", M_POWER },
-		{ { 1,146,2,1,21,171 },	"ups.L3.realpower", "%.0f", M_POWER },
+		{ { 1,144,2,1,24,172 },	"output.L1.power.percent", "%.0f", M_LOADPERC },
+		{ { 1,145,2,1,24,173 },	"output.L2.power.percent", "%.0f", M_LOADPERC },
+		{ { 1,146,2,1,24,174 },	"output.L3.power.percent", "%.0f", M_LOADPERC },
+		{ { 1,144,2,1,22,170 },	"output.L1.power", "%.0f", M_POWER },
+		{ { 1,145,2,1,22,171 },	"output.L2.power", "%.0f", M_POWER },
+		{ { 1,146,2,1,22,172 },	"output.L3.power", "%.0f", M_POWER },
+		{ { 1,144,2,1,21,169 },	"output.L1.realpower", "%.0f", M_POWER },
+		{ { 1,145,2,1,21,170 },	"output.L2.realpower", "%.0f", M_POWER },
+		{ { 1,146,2,1,21,171 },	"output.L3.realpower", "%.0f", M_POWER },
 		{ { 1,144,2,1,3,151 },	"output.L1-N.voltage", "%.1f", M_VOLTAGE_O },
 		{ { 1,145,2,1,3,152 },	"output.L2-N.voltage", "%.1f", M_VOLTAGE_O },
 		{ { 1,146,2,1,3,153 },	"output.L3-N.voltage", "%.1f", M_VOLTAGE_O },
@@ -329,6 +352,17 @@ void upsdrv_updateinfo(void)
 		{ { 0 }, NULL, NULL, 0 }
 	};
 
+	static cmd_s vartab2i[] = { /*split-phase in, only reports V L-L */
+		{ { 1,144,2,1,1,149 },	"input.voltage", "%.1f", M_VOLTAGE_I },
+
+		{ { 1,144,2,1,5,153 },	"input.bypass.voltage", "%.1f", M_VOLTAGE_B },
+
+		{ { 1,144,2,1,6,154 },	"input.bypass.current", "%.1f", M_CURRENT_B },
+
+		{ { 1,144,2,1,2,150 },	"input.current", "%.1f", M_CURRENT_I },
+		{ { 0 }, NULL, NULL, 0 }
+	};
+
 	static cmd_s vartab3i[] = { /*3-phase in */
 		{ { 1,144,2,1,1,149 },	"input.L1-N.voltage", "%.1f", M_VOLTAGE_I },
 		{ { 1,145,2,1,1,150 },	"input.L2-N.voltage", "%.1f", M_VOLTAGE_I },
@@ -347,62 +381,105 @@ void upsdrv_updateinfo(void)
 		{ { 1,146,2,1,2,152 },	"input.L3.current", "%.1f", M_CURRENT_I },
 		{ { 0 }, NULL, NULL, 0 }
 	};
-	
+
 	static cmd_s * cmdin_p;
 	static cmd_s * cmdout_p;
 
 	const char	*val;
 	char	reply[8];
-	int	ret, i;
+	ssize_t	ret;
+	int	i;
 
 	for (i = 0; vartab[i].var; i++) {
-		int16_t	val;
+		int16_t	intval;
 		ret = do_command(vartab[i].cmd, reply, 6);
 		if (ret < 8) {
 			continue;
 		}
-		val = (unsigned char)reply[5];
-		val <<= 8;
-		val += (unsigned char)reply[6];
-		dstate_setinfo(vartab[i].var, vartab[i].fmt, val * multi[vartab[i].multindex]);
+		intval = (unsigned char)reply[5];
+		intval <<= 8;
+		intval += (unsigned char)reply[6];
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
+#pragma GCC diagnostic ignored "-Wformat-security"
+#endif
+		dstate_setinfo(vartab[i].var, vartab[i].fmt, multi[vartab[i].multindex] * intval);
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic pop
+#endif
 	}
 
-	if (num_inphases>1){
+	if (num_inphases==3){
 		cmdin_p=vartab3i;
 	}
+	else if(num_inphases==2){
+		cmdin_p=vartab2i;
+	}
 	else {
 		cmdin_p=vartab1i;
 	}
 
-	if (num_outphases>1){
+	if (num_outphases==3){
 		cmdout_p=vartab3o;
 	}
+	else if(num_outphases==2){
+		cmdout_p=vartab2o;
+	}
 	else {
 		cmdout_p=vartab1o;
 	}
 
 	for (i = 0; cmdin_p[i].var; i++) {
-		int16_t	val;
+		int16_t	intval;
 		ret = do_command(cmdin_p[i].cmd, reply, 6);
 		if (ret < 8) {
 			continue;
 		}
-		val = (unsigned char)reply[5];
-		val <<= 8;
-		val += (unsigned char)reply[6];
-		dstate_setinfo(cmdin_p[i].var, cmdin_p[i].fmt, val * multi[cmdin_p[i].multindex]);
+		intval = (unsigned char)reply[5];
+		intval <<= 8;
+		intval += (unsigned char)reply[6];
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
+#pragma GCC diagnostic ignored "-Wformat-security"
+#endif
+		dstate_setinfo(cmdin_p[i].var, cmdin_p[i].fmt, multi[cmdin_p[i].multindex] * intval);
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic pop
+#endif
 	}
 
 	for (i = 0; cmdout_p[i].var; i++) {
-		int16_t	val;
+		int16_t	intval;
 		ret = do_command(cmdout_p[i].cmd, reply, 6);
 		if (ret < 8) {
 			continue;
 		}
-		val = (unsigned char)reply[5];
-		val <<= 8;
-		val += (unsigned char)reply[6];
-		dstate_setinfo(cmdout_p[i].var, cmdout_p[i].fmt, val * multi[cmdout_p[i].multindex]);
+		intval = (unsigned char)reply[5];
+		intval <<= 8;
+		intval += (unsigned char)reply[6];
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
+#pragma GCC diagnostic ignored "-Wformat-security"
+#endif
+		dstate_setinfo(cmdout_p[i].var, cmdout_p[i].fmt, multi[cmdout_p[i].multindex] * intval);
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic pop
+#endif
 	}
 
 	status_init();
@@ -476,12 +553,12 @@ void upsdrv_updateinfo(void)
 void upsdrv_shutdown(void)
 {
 	char reply[8];
-	
+
 	if(!(do_command(cmd_setOutOffMode, reply, 8) != -1) &&
-	(do_command(cmd_setOutOffDelay, reply, 8) != -1) &&
-	(do_command(cmd_sysLoadKey, reply, 6) != -1) &&
-	(do_command(cmd_shutdown, reply, 8) != -1))
-	upslogx(LOG_ERR, "Failed to shutdown UPS");
+	    (do_command(cmd_setOutOffDelay, reply, 8) != -1) &&
+	    (do_command(cmd_sysLoadKey, reply, 6) != -1) &&
+	    (do_command(cmd_shutdown, reply, 8) != -1))
+			upslogx(LOG_ERR, "Failed to shutdown UPS");
 }
 
 static int instcmd(const char *cmdname, const char *extra)
@@ -491,8 +568,8 @@ static int instcmd(const char *cmdname, const char *extra)
 		ser_send_buf(upsfd, ...);
 		return STAT_INSTCMD_HANDLED;
 	}
- */
-	upslogx(LOG_NOTICE, "instcmd: unknown command [%s]", cmdname);
+*/
+	upslogx(LOG_NOTICE, "instcmd: unknown command [%s] [%s]", cmdname, extra);
 	return STAT_INSTCMD_UNKNOWN;
 }
 
@@ -504,8 +581,8 @@ static int setvar(const char *varname, const char *val)
 	ser_send_buf(upsfd, ...);
 		return STAT_SET_HANDLED;
 	}
- */
-	upslogx(LOG_NOTICE, "setvar: unknown variable [%s]", varname);
+*/
+	upslogx(LOG_NOTICE, "setvar: unknown variable [%s] [%s]", varname, val);
 	return STAT_SET_UNKNOWN;
 }
 
@@ -524,6 +601,9 @@ void upsdrv_initups(void)
 	const char *val = getval("baudrate");
 	speed_t baudrate = B2400;
 
+	/* No-op, just made to quiesce the compiler warnings */
+	NUT_UNUSED_FUNCTION_dummy_bitfields();
+
 	if (val) {
 		switch (atoi(val))
 		{
