@@ -18,15 +18,17 @@
 */
 
 /* Uncomment if you want to read additional Meta System UPS data */
-/* 
-#define EXTRADATA 
+/*
+#define EXTRADATA
 */
 
 #include "main.h"
 #include "serial.h"
+#include "nut_float.h"
+#include "nut_stdint.h"
 
 #define DRIVER_NAME	"Metasystem UPS driver"
-#define DRIVER_VERSION	"0.07"
+#define DRIVER_VERSION	"0.08"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -38,8 +40,8 @@ upsdrv_info_t upsdrv_info = {
 };
 
 /* Autorestart flag */
-int autorestart = 0;
-int nominal_power = 0;
+static int autorestart = 0;
+static int nominal_power = 0;
 
 /* ups commands */
 #define UPS_INFO 			0x00
@@ -68,14 +70,14 @@ static int instcmd(const char *cmdname, const char *extra);
 	DATA_LENGTH is number of data bytes + the checksum byte
 	DATA ......
 	CHECKSUM is the sum modulus 256 of all DATA bytes + DATA_LENGTH
-	
+
 	The answer from the UPS have the same packet format and the first
 	data byte is equal to the command that the ups is answering to
 */
-int get_word(unsigned char *buffer) {		/* return an integer reading a word in the supplied buffer */
+static uint16_t get_word(unsigned char *buffer) {		/* return an integer reading a word in the supplied buffer */
 	unsigned char a, b;
-	int result;
-	
+	uint16_t result;
+
 	a = buffer[0];
 	b = buffer[1];
 	result = b*256 + a;
@@ -83,9 +85,9 @@ int get_word(unsigned char *buffer) {		/* return an integer reading a word in th
 }
 
 
-long int get_long(unsigned char *buffer) {	/* return a long integer reading 4 bytes in the supplied buffer */
+static uint32_t get_long(unsigned char *buffer) {	/* return a long integer reading 4 bytes in the supplied buffer */
 	unsigned char a, b, c, d;
-	long int result;
+	uint32_t result;
 	a=buffer[0];
 	b=buffer[1];
 	c=buffer[2];
@@ -94,7 +96,19 @@ long int get_long(unsigned char *buffer) {	/* return a long integer reading 4 by
 	return result;
 }
 
-void send_zeros(void) {				/* send 100 times the value 0x00.....it seems to be used for resetting */
+static float get_word_float(unsigned char *buffer) {
+	/* return a float converted after reading a word in the supplied buffer */
+	/* NOTE: This started as a wrapper for legacy logic that directly assigned
+	 *   float_num = get_word(...)
+	 * in code below. No idea if the protocol really sends 16-bit floats.
+	 * FWIW, bcmxcp.c copied get_word() and get_long() from metasys.c but did
+	 * implement a get_float() for IEEE-754 32-bit values.
+	 */
+	return (float)(int16_t)get_word(buffer);
+}
+
+
+static void send_zeros(void) {				/* send 100 times the value 0x00.....it seems to be used for resetting */
 	unsigned char buf[100];				/* the ups serial port */
 
 	memset(buf, '\0', sizeof(buf));
@@ -104,19 +118,21 @@ void send_zeros(void) {				/* send 100 times the value 0x00.....it seems to be u
 
 
 /* was used just for the debug process */
-void dump_buffer(unsigned char *buffer, int buf_len) {
+static void dump_buffer(unsigned char *buffer, int buf_len) {
 	int i;
-	for (i = 0; i < buf_len; i++) { 
-		printf("byte %d: %x\n", i, buffer[i]);	
+	for (i = 0; i < buf_len; i++) {
+		printf("byte %d: %x\n", i, buffer[i]);
 	}
 	return;
 }
 
 /* send a read command to the UPS, it retries 5 times before give up
    it's a 4 byte request (STX, LENGTH, COMMAND and CHECKSUM) */
-void send_read_command(char command) {
-	int retry, sent;
+static void send_read_command(unsigned char command) {
+	int retry;
+	ssize_t sent;
 	unsigned char buf[4];
+
 	retry = 0;
 	sent = 0;
 	while ((sent != 4) && (retry < 5)) {
@@ -130,11 +146,13 @@ void send_read_command(char command) {
 	}
 }
 
-/* send a write command to the UPS, the write command and the value to be written are passed 
-   with a char* buffer 
+/* send a write command to the UPS, the write command and the value to be written are passed
+   with a char* buffer
    it retries 5 times before give up */
-void send_write_command(unsigned char *command, int command_length) {
-	int i, retry, sent, checksum;
+static void send_write_command(unsigned char *command, size_t command_length) {
+	int retry, checksum;
+	ssize_t sent;
+	size_t i;
 	unsigned char raw_buf[255];
 
 	/* prepares the raw data */
@@ -150,57 +168,60 @@ void send_write_command(unsigned char *command, int command_length) {
 	raw_buf[command_length] = (unsigned char)checksum;
 	command_length +=1;
 
+	assert (command_length < INT_MAX);
 	retry = 0;
 	sent = 0;
-	while ((sent != (command_length)) && (retry < 5)) {
+	while ((sent != (int)(command_length)) && (retry < 5)) {
 		if (retry == 4) send_zeros();	/* last retry is preceded by a serial reset... */
 		sent = ser_send_buf(upsfd, raw_buf, (command_length));
-		if (sent != (command_length)) printf("Error sending command %d\n", raw_buf[2]);
+		if (sent < 0) ser_comm_fail("Error sending command %d\n", raw_buf[2]);
+		if (sent != (int)(command_length)) printf("Error sending command %d\n", raw_buf[2]);
 		retry += 1;
 	}
 }
 
-
 /* get the answer of a command from the ups */
-int get_answer(unsigned char *data) {
+static int get_answer(unsigned char *data) {
 	unsigned char my_buf[255];	/* packet has a maximum length of 256 bytes */
-	int packet_length, checksum, i, res;
+	unsigned char packet_length, checksum, i;
+	ssize_t res;
+
 	/* Read STX byte */
 	res = ser_get_char(upsfd, my_buf, 1, 0);
 	if (res < 1) {
-		ser_comm_fail("Receive error (STX): %d!!!\n", res);
-		return -1;	
+		ser_comm_fail("Receive error (STX): %zd!!!\n", res);
+		return -1;
 	}
 	if (my_buf[0] != 0x02) {
 		ser_comm_fail("Receive error (STX): packet not on start!!\n");
-		return -1;	
+		return -1;
 	}
 	/* Read data length byte */
 	res = ser_get_char(upsfd, my_buf, 1, 0);
 	if (res < 1) {
-		ser_comm_fail("Receive error (length): %d!!!\n", res);
-		return -1;	
+		ser_comm_fail("Receive error (length): %zd!!!\n", res);
+		return -1;
 	}
 	packet_length = my_buf[0];
 	if (packet_length < 2) {
 		ser_comm_fail("Receive error (length): packet length %d!!!\n", packet_length);
-		return -1;	
+		return -1;
 	}
 	/* Try to read all the remainig bytes (packet_length) */
 	res = ser_get_buf_len(upsfd, my_buf, packet_length, 1, 0);
 	if (res != packet_length) {
-		ser_comm_fail("Receive error (data): got %d bytes instead of %d!!!\n", res, packet_length);
-		return -1;	
+		ser_comm_fail("Receive error (data): got %zd bytes instead of %d!!!\n", res, packet_length);
+		return -1;
 	}
 
-	/* now we have the whole answer from the ups, we can checksum it 
+	/* now we have the whole answer from the ups, we can checksum it
 	   checksum byte is equal to the sum modulus 256 of all the data bytes + packet_length
 	   (no STX no checksum byte itself) */
 	checksum = packet_length;
-	for (i = 0; i < (packet_length - 1); i++) checksum += my_buf[i];  
+	for (i = 0; i < (packet_length - 1); i++) checksum += my_buf[i];
 	checksum = checksum % 256;
 	if (my_buf[packet_length-1] != checksum) {
-		ser_comm_fail("checksum error! got %x instad of %x, received %d bytes \n", my_buf[packet_length - 1], checksum, packet_length);
+		ser_comm_fail("checksum error! got %x instead of %x, received %d bytes \n", my_buf[packet_length - 1], checksum, packet_length);
 		dump_buffer(my_buf, packet_length);
 		return -1;
 	}
@@ -212,7 +233,7 @@ int get_answer(unsigned char *data) {
 /* send a read command and try get the answer, if something fails, it retries (5 times max)
    if it is on the 4th or 5th retry, it will flush the serial before sending commands
    it returns the length of the received answer or -1 in case of failure */
-int command_read_sequence(unsigned char command, unsigned char *data) {
+static int command_read_sequence(unsigned char command, unsigned char *data) {
 	int bytes_read = 0;
 	int retry = 0;
 
@@ -234,10 +255,10 @@ int command_read_sequence(unsigned char command, unsigned char *data) {
 /* send a write command and try get the answer, if something fails, it retries (5 times max)
    if it is on the 4th or 5th retry, it will flush the serial before sending commands
    it returns the length of the received answer or -1 in case of failure */
-int command_write_sequence(unsigned char *command, int command_length, unsigned char *answer) {
+static int command_write_sequence(unsigned char *command, size_t command_length, unsigned char *answer) {
 	int bytes_read = 0;
 	int retry = 0;
-	
+
 	while ((bytes_read < 1) && (retry < 5)) {
 		send_write_command(command, command_length);
 		bytes_read = get_answer(answer);
@@ -253,6 +274,15 @@ int command_write_sequence(unsigned char *command, int command_length, unsigned
 	return bytes_read;
 }
 
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP_BESIDEFUNC) && (!defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP_INSIDEFUNC) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS_BESIDEFUNC) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE_BESIDEFUNC) )
+# pragma GCC diagnostic push
+#endif
+#if (!defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP_INSIDEFUNC) && (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS_BESIDEFUNC)
+# pragma GCC diagnostic ignored "-Wtype-limits"
+#endif
+#if (!defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP_INSIDEFUNC) && (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE_BESIDEFUNC)
+# pragma GCC diagnostic ignored "-Wtautological-constant-out-of-range-compare"
+#endif
 void upsdrv_initinfo(void)
 {
 	unsigned char my_answer[255];
@@ -269,7 +299,7 @@ void upsdrv_initinfo(void)
 	 dstate_setinfo("output.current", "%d", -1);
 	dstate_setflags("output.current", ST_FLAG_RW);
 #ifdef EXTRADATA
-	 dstate_setinfo("output.current.peak", "%2.2f", -1);	
+	 dstate_setinfo("output.current.peak", "%2.2f", -1);
 	dstate_setflags("output.current.peak", ST_FLAG_RW);
 	 dstate_setinfo("input.power", "%d", -1);
 	dstate_setflags("input.power", ST_FLAG_RW);
@@ -279,7 +309,7 @@ void upsdrv_initinfo(void)
 #ifdef EXTRADATA
 	 dstate_setinfo("input.current", "%2.2f", -1);
 	dstate_setflags("input.current", ST_FLAG_RW);
-	 dstate_setinfo("input.current.peak", "%2.2f", -1);	
+	 dstate_setinfo("input.current.peak", "%2.2f", -1);
 	dstate_setflags("input.current.peak", ST_FLAG_RW);
 #endif
 	 dstate_setinfo("battery.voltage", "%d", -1);
@@ -315,15 +345,15 @@ void upsdrv_initinfo(void)
 	 dstate_setinfo("ups.test.result", "not yet done...");
 	dstate_setflags("ups.test.result", ST_FLAG_STRING | ST_FLAG_RW);
 	  dstate_setaux("ups.test.result", 20);
-	
+
 	/* UPS INFO READ */
 	res = command_read_sequence(UPS_INFO, my_answer);
 	if (res < 0) fatal_with_errno(EXIT_FAILURE, "Could not communicate with the ups");
-	/* the manufacturer is hard coded into the driver, the model type is in the second 
+	/* the manufacturer is hard coded into the driver, the model type is in the second
 		byte of the answer, the third byte identifies the model version */
 	dstate_setinfo("ups.mfr", "Meta System");
 	i = my_answer[1] * 10 + my_answer[2];
-	switch (i) {	
+	switch (i) {
 		case 11:
 			dstate_setinfo("ups.model", "%s", "HF Line (1 board)");
 			nominal_power = 630;
@@ -343,31 +373,31 @@ void upsdrv_initinfo(void)
 		case 21:
 			dstate_setinfo("ups.model", "%s", "ECO Network 750/1000");
 			nominal_power = 500;
-			break;	
+			break;
 		case 22:
 			dstate_setinfo("ups.model", "%s", "ECO Network 1050/1500");
 			nominal_power = 700;
-			break;	
+			break;
 		case 23:
 			dstate_setinfo("ups.model", "%s", "ECO Network 1500/2000");
 			nominal_power = 1000;
-			break;	
+			break;
 		case 24:
 			dstate_setinfo("ups.model", "%s", "ECO Network 1800/2500");
 			nominal_power = 1200;
-			break;	
+			break;
 		case 25:
 			dstate_setinfo("ups.model", "%s", "ECO Network 2100/3000");
 			nominal_power = 1400;
-			break;	
+			break;
 		case 31:
 			dstate_setinfo("ups.model", "%s", "ECO 308");
 			nominal_power = 500;
-			break;	
+			break;
 		case 32:
 			dstate_setinfo("ups.model", "%s", "ECO 311");
 			nominal_power = 700;
-			break;	
+			break;
 		case 44:
 			dstate_setinfo("ups.model", "%s", "HF Line (4 boards)/2");
 			nominal_power = 2520;
@@ -535,20 +565,42 @@ void upsdrv_initinfo(void)
 
 		default:
 			fatal_with_errno(EXIT_FAILURE, "Unknown UPS");
-			break;
-	} 
-		
-	/* Get the serial number */
-	memcpy(serial, my_answer + 7, res - 7);
+	}
+
+	/* Get the serial number; res >=0 per check above */
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE) )
+/* Note for gating macros above: unsuffixed HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP
+ * means support of contexts both inside and outside function body, so the push
+ * above and pop below (outside this finction) are not used.
+ */
+# pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS
+/* Note that the individual warning pragmas for use inside function bodies
+ * are named without a _INSIDEFUNC suffix, for simplicity and legacy reasons
+ */
+# pragma GCC diagnostic ignored "-Wtype-limits"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE
+# pragma GCC diagnostic ignored "-Wtautological-constant-out-of-range-compare"
+#endif
+	if (res < 7 || (unsigned long long int)res >= SIZE_MAX)
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE) )
+# pragma GCC diagnostic pop
+#endif
+		fatal_with_errno(EXIT_FAILURE, "Could not communicate with the ups");
+	memcpy(serial, my_answer + 7, (size_t)(res - 7));
 	/* serial number start from the 8th byte */
-	serial[12]=0;		/* terminate string */
+	serial[12]='\0';		/* terminate string */
 	dstate_setinfo("ups.serial", "%s", serial);
-	
+
 	/* get the ups firmware. The major number is in the 5th byte, the minor is in the 6th */
 	dstate_setinfo("ups.firmware", "%u.%u", my_answer[5], my_answer[6]);
 
-	printf("Detected %s [%s] v.%s on %s\n", dstate_getinfo("ups.model"), dstate_getinfo("ups.serial"), dstate_getinfo("ups.firmware"), device_path);
-	
+	printf("Detected %s [%s] v.%s on %s\n",
+		dstate_getinfo("ups.model"), dstate_getinfo("ups.serial"),
+		dstate_getinfo("ups.firmware"), device_path);
+
 	/* Add instant commands */
 	dstate_addcmd("shutdown.return");
 	dstate_addcmd("shutdown.stayoff");
@@ -563,17 +615,21 @@ void upsdrv_initinfo(void)
 	upsh.instcmd = instcmd;
 	return;
 }
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP_BESIDEFUNC) && (!defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP_INSIDEFUNC) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS_BESIDEFUNC) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE_BESIDEFUNC) )
+# pragma GCC diagnostic pop
+#endif
 
 void upsdrv_updateinfo(void)
 {
-	int res, int_num;
+	int res;
+	uint16_t int_num;
 #ifdef EXTRADATA
 	int day, hour, minute;
 #endif
 	float float_num;
-	long int long_num;
+	uint32_t long_num;
 	unsigned char my_answer[255];
-	
+
 	/* GET Output data */
 	res = command_read_sequence(UPS_OUTPUT_DATA, my_answer);
 	if (res < 0) {
@@ -587,36 +643,36 @@ void upsdrv_updateinfo(void)
 			dstate_setinfo("ups.load", "%2.1f", float_num);
 		} else {
 			dstate_setinfo("ups.load", "%s", "not available");
-		}	
+		}
 #ifdef EXTRADATA
-		dstate_setinfo("output.power", "%d", int_num);
+		dstate_setinfo("output.power", "%u", int_num);
 #endif
 		/* voltage */
 		int_num = get_word(&my_answer[3]);
-		if (int_num > 0) dstate_setinfo("output.voltage", "%d", int_num);
-		if (int_num == -1) dstate_setinfo("output.voltage", "%s", "overrange");
-		if (int_num == -2) dstate_setinfo("output.voltage", "%s", "not available");
+		if ((int16_t)int_num > 0) dstate_setinfo("output.voltage", "%u", int_num);
+		if ((int16_t)int_num == -1) dstate_setinfo("output.voltage", "%s", "overrange");
+		if ((int16_t)int_num == -2) dstate_setinfo("output.voltage", "%s", "not available");
 		/* current */
-		float_num = get_word(&my_answer[5]);
-		if (float_num == -1) dstate_setinfo("output.current", "%s", "overrange");
-		if (float_num == -2) dstate_setinfo("output.current", "%s", "not available");
+		float_num = get_word_float(&my_answer[5]);
+		if (f_equal(float_num, -1.0)) dstate_setinfo("output.current", "%s", "overrange");
+		if (f_equal(float_num, -2.0)) dstate_setinfo("output.current", "%s", "not available");
 		if (float_num > 0) {
 			float_num = (float)(float_num/10);
 			dstate_setinfo("output.current", "%2.2f", float_num);
 		}
 #ifdef EXTRADATA
 		/* peak current */
-		float_num = get_word(&my_answer[7]);
-		if (float_num == -1) dstate_setinfo("output.current.peak", "%s", "overrange");
-		if (float_num == -2) dstate_setinfo("output.current.peak", "%s", "not available");
+		float_num = get_word_float(&my_answer[7]);
+		if (f_equal(float_num, -1.0)) dstate_setinfo("output.current.peak", "%s", "overrange");
+		if (f_equal(float_num, -2.0)) dstate_setinfo("output.current.peak", "%s", "not available");
 		if (float_num > 0) {
 			float_num = (float)(float_num/10);
 			dstate_setinfo("output.current.peak", "%2.2f", float_num);
 		}
-		
+
 #endif
 	}
-		
+
 	/* GET Input data */
 	res = command_read_sequence(UPS_INPUT_DATA, my_answer);
 	if (res < 0){
@@ -626,18 +682,18 @@ void upsdrv_updateinfo(void)
 #ifdef EXTRADATA
 		/* Active power */
 		int_num = get_word(&my_answer[1]);
-		if (int_num > 0) dstate_setinfo("input.power", "%d", int_num);
-		if (int_num == -1) dstate_setinfo("input.power", "%s", "overrange");
-		if (int_num == -2) dstate_setinfo("input.power", "%s", "not available");
+		if ((int16_t)int_num > 0) dstate_setinfo("input.power", "%u", int_num);
+		if ((int16_t)int_num == -1) dstate_setinfo("input.power", "%s", "overrange");
+		if ((int16_t)int_num == -2) dstate_setinfo("input.power", "%s", "not available");
 #endif
 		/* voltage */
 		int_num = get_word(&my_answer[3]);
-		if (int_num > 0) dstate_setinfo("input.voltage", "%d", int_num);
-		if (int_num == -1) dstate_setinfo("input.voltage", "%s", "overrange");
-		if (int_num == -2) dstate_setinfo("input.voltage", "%s", "not available");
+		if ((int16_t)int_num > 0) dstate_setinfo("input.voltage", "%u", int_num);
+		if ((int16_t)int_num == -1) dstate_setinfo("input.voltage", "%s", "overrange");
+		if ((int16_t)int_num == -2) dstate_setinfo("input.voltage", "%s", "not available");
 #ifdef EXTRADATA
 		/* current */
-		float_num = get_word(&my_answer[5]);
+		float_num = get_word_float(&my_answer[5]);
 		if (float_num == -1) dstate_setinfo("input.current", "%s", "overrange");
 		if (float_num == -2) dstate_setinfo("input.current", "%s", "not available");
 		if (float_num > 0) {
@@ -645,7 +701,7 @@ void upsdrv_updateinfo(void)
 			dstate_setinfo("input.current", "%2.2f", float_num);
 		}
 		/* peak current */
-		float_num = get_word(&my_answer[7]);
+		float_num = get_word_float(&my_answer[7]);
 		if (float_num == -1) dstate_setinfo("input.current.peak", "%s", "overrange");
 		if (float_num == -2) dstate_setinfo("input.current.peak", "%s", "not available");
 		if (float_num > 0) {
@@ -654,8 +710,8 @@ void upsdrv_updateinfo(void)
 		}
 #endif
 	}
-	
-	
+
+
 	/* GET Battery data */
 	res = command_read_sequence(UPS_BATTERY_DATA, my_answer);
 	if (res < 0) {
@@ -663,21 +719,21 @@ void upsdrv_updateinfo(void)
 		dstate_datastale();
 	} else {
 		/* Actual value */
-		float_num = get_word(&my_answer[1]);
+		float_num = get_word_float(&my_answer[1]);
 		float_num = (float)(float_num/10);
 		dstate_setinfo("battery.voltage", "%2.2f", float_num);
 #ifdef EXTRADATA
 		/* reserve threshold */
-		float_num = get_word(&my_answer[3]);
+		float_num = get_word_float(&my_answer[3]);
 		float_num = (float)(float_num/10);
 		dstate_setinfo("battery.voltage.low", "%2.2f", float_num);
 		/* exhaust threshold */
-		float_num = get_word(&my_answer[5]);
+		float_num = get_word_float(&my_answer[5]);
 		float_num = (float)(float_num/10);
 		dstate_setinfo("battery.voltage.exhaust", "%2.2f", float_num);
 #endif
 	}
-	
+
 #ifdef EXTRADATA
 	/* GET history data */
 	res = command_read_sequence(UPS_HISTORY_DATA, my_answer);
@@ -693,8 +749,8 @@ void upsdrv_updateinfo(void)
 		long_num -= (long)(hour*3600);
 		minute = (int)(long_num / 60);
 		long_num -= (minute*60);
-		dstate_setinfo("ups.total.runtime", "%d days %dh %dm %lds", day, hour, minute, long_num);
-		
+		dstate_setinfo("ups.total.runtime", "%d days %dh %dm %lus", day, hour, minute, long_num);
+
 		/* ups inverter runtime */
 		long_num = get_long(&my_answer[5]);
 		day = (int)(long_num / 86400);
@@ -703,22 +759,22 @@ void upsdrv_updateinfo(void)
 		long_num -= (long)(hour*3600);
 		minute = (int)(long_num / 60);
 		long_num -= (minute*60);
-		dstate_setinfo("ups.inverter.runtime", "%d days %dh %dm %lds", day, hour, minute, long_num);
+		dstate_setinfo("ups.inverter.runtime", "%d days %dh %dm %lus", day, hour, minute, long_num);
 		/* ups inverter interventions */
-		dstate_setinfo("ups.inverter.interventions", "%d", get_word(&my_answer[9]));
+		dstate_setinfo("ups.inverter.interventions", "%u", get_word(&my_answer[9]));
 		/* battery full discharges */
-		dstate_setinfo("battery.full.discharges", "%d", get_word(&my_answer[11]));
+		dstate_setinfo("battery.full.discharges", "%u", get_word(&my_answer[11]));
 		/* ups bypass / stabilizer interventions */
 		int_num = get_word(&my_answer[13]);
-		if (int_num == -2) dstate_setinfo("ups.bypass.interventions", "%s", "not avaliable");
-		if (int_num >= 0) dstate_setinfo("ups.bypass.interventions", "%d", int_num);
+		if ((int16_t)int_num == -2) dstate_setinfo("ups.bypass.interventions", "%s", "not avaliable");
+		if ((int16_t)int_num >= 0) dstate_setinfo("ups.bypass.interventions", "%u", int_num);
 		/* ups overheatings */
 		int_num = get_word(&my_answer[15]);
-		if (int_num == -2) dstate_setinfo("ups.overheatings", "%s", "not avalilable");
-		if (int_num >= 0) dstate_setinfo("ups.overheatings", "%d", int_num);
+		if ((int16_t)int_num == -2) dstate_setinfo("ups.overheatings", "%s", "not avalilable");
+		if ((int16_t)int_num >= 0) dstate_setinfo("ups.overheatings", "%u", int_num);
 	}
 #endif
-	
+
 	/* GET times on battery */
 	res = command_read_sequence(UPS_GET_TIMES_ON_BATTERY, my_answer);
 	if (res < 0) {
@@ -727,8 +783,8 @@ void upsdrv_updateinfo(void)
 	} else {
 		autorestart = my_answer[5];
 	}
-	
-	
+
+
 	/* GET schedule */
 	res = command_read_sequence(UPS_GET_SCHEDULING, my_answer);
 	if (res < 0) {
@@ -737,21 +793,21 @@ void upsdrv_updateinfo(void)
 	} else {
 		/* time remaining to shutdown */
 		long_num = get_long(&my_answer[1]);
-		if (long_num == -1) {
-			dstate_setinfo("ups.delay.shutdown", "%d", 120);	
+		if ((int32_t)long_num == -1) {
+			dstate_setinfo("ups.delay.shutdown", "%d", 120);
 		} else {
-			dstate_setinfo("ups.delay.shutdown", "%ld", long_num);
+			dstate_setinfo("ups.delay.shutdown", "%lu", (unsigned long)long_num);
 		}
 		/* time remaining to restart  */
 		long_num = get_long(&my_answer[5]);
-		if (long_num == -1) {
-			dstate_setinfo("ups.delay.start", "%d", 0);	
+		if ((int32_t)long_num == -1) {
+			dstate_setinfo("ups.delay.start", "%d", 0);
 		} else {
-			dstate_setinfo("ups.delay.start", "%ld", long_num);
-		}	
+			dstate_setinfo("ups.delay.start", "%lu", (unsigned long)long_num);
+		}
 	}
-	
-	
+
+
 	/* GET ups status */
 	res = command_read_sequence(UPS_STATUS, my_answer);
 	if (res < 0) {
@@ -764,7 +820,7 @@ void upsdrv_updateinfo(void)
 			dstate_setinfo("ups.temperature", "%d", my_answer[3]);
 		} else {
 			dstate_setinfo("ups.temperature", "%s", "not available");
-		}	
+		}
 		/* Status */
 		status_init();
 		switch (my_answer[1]) {	/* byte 1 = STATUS */
@@ -784,7 +840,7 @@ void upsdrv_updateinfo(void)
 			default:
 				printf("status unknown \n");
 				break;
-		} 
+		}
 		switch (my_answer[2]) {		/* byte 2 = FAULTS */
 			case 0x00:				/* all right */
 				break;
@@ -815,22 +871,22 @@ void upsdrv_shutdown(void)
 	unsigned char command[10], answer[10];
 
 	/* Ensure that the ups is configured for automatically
-	   restart after a complete battery discharge 
+	   restart after a complete battery discharge
 	   and when the power comes back after a shutdown */
 	if (! autorestart) {
 		command[0]=UPS_SET_TIMES_ON_BATTERY;
-		command[1]=0x00;					/* max time on  */ 
+		command[1]=0x00;					/* max time on  */
 		command[2]=0x00;					/* battery */
-		
+
 		command[3]=0x00;					/* max time after */
 		command[4]=0x00;					/* battery reserve */
-		
+
 		command[5]=0x01;					/* autorestart after battery depleted enabled */
 		command_write_sequence(command, 6, answer);
 	}
 
 	/* shedule a shutdown in 120 seconds */
-	command[0]=UPS_SET_SCHEDULING;		
+	command[0]=UPS_SET_SCHEDULING;
 	command[1]=0x96;					/* remaining  */
 	command[2]=0x00;					/* time		 */
 	command[3]=0x00;					/* to */
@@ -876,7 +932,7 @@ static int instcmd(const char *cmdname, const char *extra)
 		/* Same stuff as upsdrv_shutdown() */
 		if (! autorestart) {
 			command[0]=UPS_SET_TIMES_ON_BATTERY;
-			command[1]=0x00;					/* max time on  */ 
+			command[1]=0x00;					/* max time on  */
 			command[2]=0x00;					/* battery */
 			command[3]=0x00;					/* max time after */
 			command[4]=0x00;					/* battery reserve */
@@ -884,7 +940,7 @@ static int instcmd(const char *cmdname, const char *extra)
 			command_write_sequence(command, 6, answer);
 		}
 		/* shedule a shutdown in 30 seconds */
-		command[0]=UPS_SET_SCHEDULING;		
+		command[0]=UPS_SET_SCHEDULING;
 		command[1]=0x1e;					/* remaining  */
 		command[2]=0x00;					/* time		 */
 		command[3]=0x00;					/* to */
@@ -900,12 +956,12 @@ static int instcmd(const char *cmdname, const char *extra)
 
 	if (!strcasecmp(cmdname, "shutdown.stayoff")) {
 		/* shedule a shutdown in 30 seconds with no restart (-1) */
-		command[0]=UPS_SET_SCHEDULING;		
+		command[0]=UPS_SET_SCHEDULING;
 		command[1]=0x1e;					/* remaining  */
 		command[2]=0x00;					/* time		 */
 		command[3]=0x00;					/* to */
 		command[4]=0x00;					/* shutdown 150 secs */
-				
+
 		command[5]=0xff;					/* programmed */
 		command[6]=0xff;					/* time		 */
 		command[7]=0xff;					/* to */
@@ -916,12 +972,12 @@ static int instcmd(const char *cmdname, const char *extra)
 
 	if (!strcasecmp(cmdname, "shutdown.stop")) {
 		/* set shutdown and restart time to -1 (no shutdown, no restart) */
-		command[0]=UPS_SET_SCHEDULING;		
+		command[0]=UPS_SET_SCHEDULING;
 		command[1]=0xff;					/* remaining  */
 		command[2]=0xff;					/* time		 */
 		command[3]=0xff;					/* to */
 		command[4]=0xff;					/* shutdown -1 (no shutdown) */
-				
+
 		command[5]=0xff;					/* programmed */
 		command[6]=0xff;					/* time		 */
 		command[7]=0xff;					/* to */
@@ -932,8 +988,8 @@ static int instcmd(const char *cmdname, const char *extra)
 
 	if (!strcasecmp(cmdname, "test.failure.start")) {
 		/* force ups on battery power */
-		command[0]=UPS_SET_BATTERY_TEST;	
-		command[1]=0x01;					
+		command[0]=UPS_SET_BATTERY_TEST;
+		command[1]=0x01;
 		/* 0 = perform battery test
 		   1 = force UPS on battery power
 		   2 = restore standard mode (mains power) */
@@ -944,7 +1000,7 @@ static int instcmd(const char *cmdname, const char *extra)
 	if (!strcasecmp(cmdname, "test.failure.stop")) {
 		/* restore standard mode (mains power) */
 		command[0]=UPS_SET_BATTERY_TEST;
-		command[1]=0x02;					
+		command[1]=0x02;
 		/* 0 = perform battery test
 		   1 = force UPS on battery power
 		   2 = restore standard mode (mains power) */
@@ -954,8 +1010,8 @@ static int instcmd(const char *cmdname, const char *extra)
 
 	if (!strcasecmp(cmdname, "test.battery.start")) {
 		/* launch battery test */
-		command[0]=UPS_SET_BATTERY_TEST;		
-		command[1]=0x00;					
+		command[0]=UPS_SET_BATTERY_TEST;
+		command[1]=0x00;
 		/* 0 = perform battery test
 		   1 = force UPS on battery power
 		   2 = restore standard mode (mains power) */
@@ -966,22 +1022,22 @@ static int instcmd(const char *cmdname, const char *extra)
 			case 0x00:				/* all right */
 				dstate_setinfo("ups.test.result", "OK");
 				break;
-			case 0x01:				
+			case 0x01:
 				dstate_setinfo("ups.test.result", "Battery charge: 20%%");
 				break;
-			case 0x02:				
+			case 0x02:
 				dstate_setinfo("ups.test.result", "Battery charge: 40%%");
 				break;
-			case 0x03:				
+			case 0x03:
 				dstate_setinfo("ups.test.result", "Battery charge: 60%%");
 				break;
-			case 0x04:				
+			case 0x04:
 				dstate_setinfo("ups.test.result", "Battery charge: 80%%");
 				break;
-			case 0x05:				
+			case 0x05:
 				dstate_setinfo("ups.test.result", "Battery charge: 100%%");
 				break;
-			case 0xfe:				
+			case 0xfe:
 				dstate_setinfo("ups.test.result", "Bad battery pack: replace");
 				break;
 			default:
@@ -996,8 +1052,8 @@ static int instcmd(const char *cmdname, const char *extra)
 
 	if (!strcasecmp(cmdname, "beeper.enable")) {
 		/* set buzzer to not muted */
-		command[0]=UPS_SET_BUZZER_MUTE;		
-		command[1]=0x00;					
+		command[0]=UPS_SET_BUZZER_MUTE;
+		command[1]=0x00;
 		/* 0 = not muted
 		   1 = muted
 		   2 = read current status */
@@ -1007,8 +1063,8 @@ static int instcmd(const char *cmdname, const char *extra)
 
 	if (!strcasecmp(cmdname, "beeper.mute")) {
 		/* set buzzer to muted */
-		command[0]=UPS_SET_BUZZER_MUTE;		
-		command[1]=0x01;					
+		command[0]=UPS_SET_BUZZER_MUTE;
+		command[1]=0x01;
 		/* 0 = not muted
 		   1 = muted
 		   2 = read current status */
@@ -1016,7 +1072,7 @@ static int instcmd(const char *cmdname, const char *extra)
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	upslogx(LOG_NOTICE, "instcmd: unknown command [%s]", cmdname);
+	upslogx(LOG_NOTICE, "instcmd: unknown command [%s] [%s]", cmdname, extra);
 	return STAT_INSTCMD_UNKNOWN;
 }
 
@@ -1038,12 +1094,12 @@ void upsdrv_makevartable(void)
 
 void upsdrv_initups(void)
 {
-	upsfd = ser_open(device_path); 
-	ser_set_speed(upsfd, device_path, B2400); 
+	upsfd = ser_open(device_path);
+	ser_set_speed(upsfd, device_path, B2400);
 	send_zeros();
 }
 
 void upsdrv_cleanup(void)
 {
-	ser_close(upsfd, device_path); 
+	ser_close(upsfd, device_path);
 }
