@@ -2,7 +2,7 @@
  *
  *  Copyright (C)
  *        2003 - 2015 Arnaud Quette <arnaud.quette@free.fr>
- *        2015 Arnaud Quette <ArnaudQuette@Eaton.com>
+ *        2015 - 2016 Eaton / Arnaud Quette <ArnaudQuette@Eaton.com>
  *
  *  Sponsored by MGE UPS SYSTEMS <http://www.mgeups.com>
  *
@@ -36,8 +36,9 @@
 #include "main.h"		/* for getval() */
 #include "usbhid-ups.h"
 #include "mge-hid.h"
+#include "nut_float.h"
 
-#define MGE_HID_VERSION		"MGE HID 1.39"
+#define MGE_HID_VERSION		"MGE HID 1.46"
 
 /* (prev. MGE Office Protection Systems, prev. MGE UPS SYSTEMS) */
 /* Eaton */
@@ -55,6 +56,14 @@
 /* AEG */
 #define AEG_VENDORID 0x2b2d
 
+/* Note that normally this VID is handled by Liebert/Phoenixtec HID mapping,
+ * here it is just for for AEG PROTECT NAS devices: */
+/* Phoenixtec Power Co., Ltd */
+#define PHOENIXTEC 0x06da
+
+/* IBM */
+#define IBM_VENDORID 0x04b3
+
 #ifndef SHUT_MODE
 #include "usb-common.h"
 
@@ -80,9 +89,13 @@ static usb_device_id_t mge_usb_device_table[] = {
 
 	/* PROTECT B / NAS */
 	{ USB_DEVICE(AEG_VENDORID, 0xffff), NULL },
+	{ USB_DEVICE(PHOENIXTEC, 0xffff), NULL },
+
+	/* 6000 VA LCD 4U Rack UPS; 5396-1Kx */
+	{ USB_DEVICE(IBM_VENDORID, 0x0001), NULL },
 
 	/* Terminating entry */
-	{ -1, -1, NULL }
+	{ 0, 0, NULL }
 };
 #endif
 
@@ -110,7 +123,7 @@ typedef enum {
 } models_type_t;
 
 /* Default to line-interactive or online (ie, not offline).
- * This is then overriden for offline, through mge_model_names */
+ * This is then overridden for offline, through mge_model_names */
 static models_type_t	mge_type = MGE_DEFAULT;
 
 /* Countries definition, for region specific settings and features */
@@ -149,10 +162,10 @@ static char		mge_scratch_buf[20];
  * annunciate float mode until the charger power starts falling from the maximum
  * level indicating the battery is truly at the float voltage or in float mode.
  * The %charge level is based on battery voltage and the charge mode timer
- * (should be 48 hours) and some UPSs add in a value that’s related to charger
+ * (should be 48 hours) and some UPSs add in a value that's related to charger
  * power output.  So you can have UPS that enters float mode with anywhere
  * from 80% or greater battery capacity.
- * float mode is not important from the software’s perspective, it’s there to
+ * float mode is not important from the software's perspective, it's there to
  * help determine if the charger is advancing correctly.
  * So in float mode, the charger is charging the battery, so by definition you
  * can assert the CHRG flag in NUT when in “float” mode or “charge” mode.
@@ -184,8 +197,8 @@ static const char *eaton_abm_enabled_fun(double value)
 }
 
 static info_lkp_t eaton_abm_enabled_info[] = {
-	{ 0, "dummy", eaton_abm_enabled_fun },
-	{ 0, NULL, NULL }
+	{ 0, "dummy", eaton_abm_enabled_fun, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 /* Note 1: This point will need more clarification! */
@@ -202,8 +215,8 @@ static const char *eaton_abm_enabled_legacy_fun(double value)
 }
 
 static info_lkp_t eaton_abm_enabled_legacy_info[] = {
-	{ 0, "dummy", eaton_abm_enabled_legacy_fun },
-	{ 0, NULL, NULL }
+	{ 0, "dummy", eaton_abm_enabled_legacy_fun, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 #endif /* if 0 */
 
@@ -249,8 +262,8 @@ static const char *eaton_abm_status_fun(double value)
 }
 
 static info_lkp_t eaton_abm_status_info[] = {
-	{ 1, "dummy", eaton_abm_status_fun },
-	{ 0, NULL, NULL }
+	{ 1, "dummy", eaton_abm_status_fun, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 /* Used to process ABM flags, for ups.status (CHRG/DISCHRG/RB) */
@@ -283,8 +296,8 @@ static const char *eaton_abm_chrg_dischrg_fun(double value)
 }
 
 static info_lkp_t eaton_abm_chrg_dischrg_info[] = {
-	{ 1, "dummy", eaton_abm_chrg_dischrg_fun },
-	{ 0, NULL, NULL }
+	{ 1, "dummy", eaton_abm_chrg_dischrg_fun, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 /* ABM also implies that standard CHRG/DISCHRG are processed according
@@ -297,7 +310,7 @@ static const char *eaton_abm_check_dischrg_fun(double value)
 {
 	if (advanced_battery_monitoring == ABM_DISABLED)
 	{
-		if (value == 1) {
+		if (d_equal(value, 1)) {
 			snprintf(mge_scratch_buf, sizeof(mge_scratch_buf), "%s", "dischrg");
 		}
 		else {
@@ -315,15 +328,15 @@ static const char *eaton_abm_check_dischrg_fun(double value)
 }
 
 static info_lkp_t eaton_discharging_info[] = {
-	{ 1, "dummy", eaton_abm_check_dischrg_fun },
-	{ 0, NULL, NULL }
+	{ 1, "dummy", eaton_abm_check_dischrg_fun, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 static const char *eaton_abm_check_chrg_fun(double value)
 {
 	if (advanced_battery_monitoring == ABM_DISABLED)
 	{
-		if (value == 1) {
+		if (d_equal(value, 1)) {
 			snprintf(mge_scratch_buf, sizeof(mge_scratch_buf), "%s", "chrg");
 		}
 		else {
@@ -341,8 +354,8 @@ static const char *eaton_abm_check_chrg_fun(double value)
 }
 
 static info_lkp_t eaton_charging_info[] = {
-	{ 1, "dummy", eaton_abm_check_chrg_fun },
-	{ 0, NULL, NULL }
+	{ 1, "dummy", eaton_abm_check_chrg_fun, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 /* The HID path 'UPS.PowerSummary.Time' reports Unix time (ie the number of
@@ -351,8 +364,9 @@ static info_lkp_t eaton_charging_info[] = {
 static const char *mge_date_conversion_fun(double value)
 {
 	time_t	sec = value;
+	struct tm	tmbuf;
 
-	if (strftime(mge_scratch_buf, sizeof(mge_scratch_buf), "%Y/%m/%d", localtime(&sec)) == 10) {
+	if (strftime(mge_scratch_buf, sizeof(mge_scratch_buf), "%Y/%m/%d", localtime_r(&sec, &tmbuf)) == 10) {
 		return mge_scratch_buf;
 	}
 
@@ -363,8 +377,9 @@ static const char *mge_date_conversion_fun(double value)
 static const char *mge_time_conversion_fun(double value)
 {
 	time_t sec = value;
+	struct tm	tmbuf;
 
-	if (strftime(mge_scratch_buf, sizeof(mge_scratch_buf), "%H:%M:%S", localtime(&sec)) == 8) {
+	if (strftime(mge_scratch_buf, sizeof(mge_scratch_buf), "%H:%M:%S", localtime_r(&sec, &tmbuf)) == 8) {
 		return mge_scratch_buf;
 	}
 
@@ -406,19 +421,23 @@ static double mge_time_conversion_nuf(const char *value)
 }
 
 static info_lkp_t mge_date_conversion[] = {
-	{ 0, NULL, mge_date_conversion_fun, mge_date_conversion_nuf }
+	{ 0, NULL, mge_date_conversion_fun, mge_date_conversion_nuf },
+	{ 0, NULL, NULL, NULL }
 };
 
 static info_lkp_t mge_time_conversion[] = {
-	{ 0, NULL, mge_time_conversion_fun, mge_time_conversion_nuf }
+	{ 0, NULL, mge_time_conversion_fun, mge_time_conversion_nuf },
+	{ 0, NULL, NULL, NULL }
 };
 #else
 static info_lkp_t mge_date_conversion[] = {
-	{ 0, NULL, mge_date_conversion_fun, NULL }
+	{ 0, NULL, mge_date_conversion_fun, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 static info_lkp_t mge_time_conversion[] = {
-	{ 0, NULL, mge_time_conversion_fun, NULL }
+	{ 0, NULL, mge_time_conversion_fun, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 #endif /* HAVE_STRPTIME */
 
@@ -448,7 +467,8 @@ static const char *mge_battery_voltage_nominal_fun(double value)
 }
 
 static info_lkp_t mge_battery_voltage_nominal[] = {
-	{ 0, NULL, mge_battery_voltage_nominal_fun }
+	{ 0, NULL, mge_battery_voltage_nominal_fun, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 /* The HID path 'UPS.PowerSummary.Voltage' only reports
@@ -472,7 +492,8 @@ static const char *mge_battery_voltage_fun(double value)
 }
 
 static info_lkp_t mge_battery_voltage[] = {
-	{ 0, NULL, mge_battery_voltage_fun }
+	{ 0, NULL, mge_battery_voltage_fun, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 static const char *mge_powerfactor_conversion_fun(double value)
@@ -482,7 +503,8 @@ static const char *mge_powerfactor_conversion_fun(double value)
 }
 
 static info_lkp_t mge_powerfactor_conversion[] = {
-	{ 0, NULL, mge_powerfactor_conversion_fun }
+	{ 0, NULL, mge_powerfactor_conversion_fun, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 static const char *mge_battery_capacity_fun(double value)
@@ -492,65 +514,66 @@ static const char *mge_battery_capacity_fun(double value)
 }
 
 static info_lkp_t mge_battery_capacity[] = {
-	{ 0, NULL, mge_battery_capacity_fun }
+	{ 0, NULL, mge_battery_capacity_fun, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
-info_lkp_t eaton_enable_disable_info[] = {
-	{ 0, "disabled", NULL },
-	{ 1, "enabled", NULL },
-	{ 0, NULL, NULL }
+static info_lkp_t eaton_enable_disable_info[] = {
+	{ 0, "disabled", NULL, NULL },
+	{ 1, "enabled", NULL, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 static info_lkp_t mge_upstype_conversion[] = {
-	{ 1, "offline / line interactive", NULL },
-	{ 2, "online", NULL },
-	{ 3, "online - unitary/parallel", NULL },
-	{ 4, "online - parallel with hot standy", NULL },
-	{ 5, "online - hot standby redundancy", NULL },
-	{ 0, NULL, NULL }
+	{ 1, "offline / line interactive", NULL, NULL },
+	{ 2, "online", NULL, NULL },
+	{ 3, "online - unitary/parallel", NULL, NULL },
+	{ 4, "online - parallel with hot standy", NULL, NULL },
+	{ 5, "online - hot standby redundancy", NULL, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 static info_lkp_t mge_sensitivity_info[] = {
-	{ 0, "normal", NULL },
-	{ 1, "high", NULL },
-	{ 2, "low", NULL },
-	{ 0, NULL, NULL }
+	{ 0, "normal", NULL, NULL },
+	{ 1, "high", NULL, NULL },
+	{ 2, "low", NULL, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 static info_lkp_t mge_emergency_stop[] = {
-	{ 1, "Emergency stop!", NULL },
-	{ 0, NULL, NULL }
+	{ 1, "Emergency stop!", NULL, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 static info_lkp_t mge_wiring_fault[] = {
-	{ 1, "Wiring fault!", NULL },
-	{ 0, NULL, NULL }
+	{ 1, "Wiring fault!", NULL, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 static info_lkp_t mge_config_failure[] = {
-	{ 1, "Fatal EEPROM fault!", NULL },
-	{ 0, NULL, NULL }
+	{ 1, "Fatal EEPROM fault!", NULL, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 static info_lkp_t mge_inverter_volthi[] = {
-	{ 1, "Inverter AC voltage too high!", NULL },
-	{ 0, NULL, NULL }
+	{ 1, "Inverter AC voltage too high!", NULL, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 static info_lkp_t mge_inverter_voltlo[] = {
-	{ 1, "Inverter AC voltage too low!", NULL },
-	{ 0, NULL, NULL }
+	{ 1, "Inverter AC voltage too low!", NULL, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 static info_lkp_t mge_short_circuit[] = {
-	{ 1, "Output short circuit!", NULL },
-	{ 0, NULL, NULL }
+	{ 1, "Output short circuit!", NULL, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
-info_lkp_t mge_onbatt_info[] = {
-	{ 1, "!online", NULL },
-	{ 0, "online", NULL },
-	{ 0, NULL, NULL }
+static info_lkp_t mge_onbatt_info[] = {
+	{ 1, "!online", NULL, NULL },
+	{ 0, "online", NULL, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 /* allow limiting to ups.model = Protection Station, Ellipse Eco
@@ -565,6 +588,7 @@ static const char *eaton_check_pegasus_fun(double value)
 		/* Only consider non European models */
 		if (country_code != COUNTRY_EUROPE)
 			break;
+		return NULL;
 	default:
 		return NULL;
 	}
@@ -574,10 +598,10 @@ static const char *eaton_check_pegasus_fun(double value)
 }
 
 static info_lkp_t pegasus_threshold_info[] = {
-	{ 10, "10", eaton_check_pegasus_fun },
-	{ 25, "25", eaton_check_pegasus_fun },
-	{ 60, "60", eaton_check_pegasus_fun },
-	{ 0, NULL, NULL }
+	{ 10, "10", eaton_check_pegasus_fun, NULL },
+	{ 25, "25", eaton_check_pegasus_fun, NULL },
+	{ 60, "60", eaton_check_pegasus_fun, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 /* allow limiting standard yes/no info (here, to enable ECO mode) to
@@ -593,11 +617,12 @@ static const char *pegasus_yes_no_info_fun(double value)
 		/* Only consider non European models */
 		if (country_code != COUNTRY_EUROPE)
 			break;
+		return NULL;
 	default:
 		return NULL;
 	}
 
-	return (value == 0) ? "no" : "yes";
+	return (d_equal(value, 0)) ? "no" : "yes";
 }
 
 /* Conversion back of yes/no info */
@@ -611,6 +636,7 @@ static double pegasus_yes_no_info_nuf(const char *value)
 		/* Only consider non European models */
 		if (country_code != COUNTRY_EUROPE)
 			break;
+		return 0;
 	default:
 		return 0;
 	}
@@ -621,10 +647,10 @@ static double pegasus_yes_no_info_nuf(const char *value)
 		return 0;
 }
 
-info_lkp_t pegasus_yes_no_info[] = {
+static info_lkp_t pegasus_yes_no_info[] = {
 	{ 0, "no", pegasus_yes_no_info_fun, pegasus_yes_no_info_nuf },
 	{ 1, "yes", pegasus_yes_no_info_fun, pegasus_yes_no_info_nuf },
-	{ 0, NULL, NULL }
+	{ 0, NULL, NULL, NULL }
 };
 
 /* Determine country using UPS.PowerSummary.Country.
@@ -639,8 +665,42 @@ static const char *eaton_check_country_fun(double value)
 }
 
 static info_lkp_t eaton_check_country_info[] = {
-	{ 0, "dummy", eaton_check_country_fun },
-	{ 0, NULL, NULL }
+	{ 0, "dummy", eaton_check_country_fun, NULL },
+	{ 0, NULL, NULL, NULL }
+};
+
+/* When UPS.PowerConverter.Output.ActivePower is not present,
+ * compute a realpower approximation using available data */
+static const char *eaton_compute_realpower_fun(double value)
+{
+	NUT_UNUSED_VARIABLE(value);
+	const char *str_ups_load = dstate_getinfo("ups.load");
+	const char *str_power_nominal = dstate_getinfo("ups.power.nominal");
+	const char *str_powerfactor = dstate_getinfo("output.powerfactor");
+	float powerfactor = 0.80;
+	int power_nominal = 0;
+	int ups_load = 0;
+	double realpower = 0;
+	if (str_power_nominal && str_ups_load) {
+		/* Extract needed values */
+		ups_load = atoi(str_ups_load);
+		power_nominal = atoi(str_power_nominal);
+		if (str_powerfactor)
+			powerfactor = atoi(str_powerfactor);
+		/* Compute the value */
+		realpower = round(ups_load * 0.01 * power_nominal * powerfactor);
+		snprintf(mge_scratch_buf, sizeof(mge_scratch_buf), "%.0f", realpower);
+		upsdebugx(1, "eaton_compute_realpower_fun(%s)", mge_scratch_buf);
+		return mge_scratch_buf;
+	}
+	/* else can't process */
+	/* Return NULL, not to get the value published! */
+	return NULL;
+}
+
+static info_lkp_t eaton_compute_realpower_info[] = {
+	{ 0, "dummy", eaton_compute_realpower_fun, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 /* Limit nominal output voltage according to HV or LV models */
@@ -687,6 +747,7 @@ static const char *nominal_output_voltage_fun(double value)
 		case 240:
 			if ((mge_type & 0xFF00) >= MGE_DEFAULT)
 				break;
+			return NULL;
 		default:
 			return NULL;
 		}
@@ -709,10 +770,15 @@ static const char *nominal_output_voltage_fun(double value)
 			 * support both HV values */
 			if (country_code == COUNTRY_EUROPE_208)
 				break;
+			/* explicit fallthrough: */
+			goto fallthrough_value;
+
 		case 220:
 		case 230:
 		case 240:
+		fallthrough_value:
 			break;
+
 		default:
 			return NULL;
 		}
@@ -730,19 +796,35 @@ static info_lkp_t nominal_output_voltage_info[] = {
 	/* line-interactive, starting with Evolution, support both HV values */
 	/* HV models */
 	/* 208V */
-	{ 200, "200", nominal_output_voltage_fun },
-	{ 208, "208", nominal_output_voltage_fun },
+	{ 200, "200", nominal_output_voltage_fun, NULL },
+	{ 208, "208", nominal_output_voltage_fun, NULL },
 	/* HV models */
 	/* 230V */
-	{ 220, "220", nominal_output_voltage_fun },
-	{ 230, "230", nominal_output_voltage_fun },
-	{ 240, "240", nominal_output_voltage_fun },
+	{ 220, "220", nominal_output_voltage_fun, NULL },
+	{ 230, "230", nominal_output_voltage_fun, NULL },
+	{ 240, "240", nominal_output_voltage_fun, NULL },
 	/* LV models */
-	{ 100, "100", nominal_output_voltage_fun },
-	{ 110, "110", nominal_output_voltage_fun },
-	{ 120, "120", nominal_output_voltage_fun },
-	{ 127, "127", nominal_output_voltage_fun },
-	{ 0, NULL, NULL }
+	{ 100, "100", nominal_output_voltage_fun, NULL },
+	{ 110, "110", nominal_output_voltage_fun, NULL },
+	{ 120, "120", nominal_output_voltage_fun, NULL },
+	{ 127, "127", nominal_output_voltage_fun, NULL },
+	{ 0, NULL, NULL, NULL }
+};
+
+/* Limit reporting "online / !online" to when "!off" */
+static const char *eaton_converter_online_fun(double value)
+{
+	unsigned ups_status = ups_status_get();
+
+	if (ups_status & STATUS(OFF))
+		return NULL;
+	else
+		return (d_equal(value, 0)) ? "!online" : "online";
+}
+
+static info_lkp_t eaton_converter_online_info[] = {
+	{ 0, "dummy", eaton_converter_online_fun, NULL },
+	{ 0, NULL, NULL, NULL }
 };
 
 /* --------------------------------------------------------------- */
@@ -1006,7 +1088,7 @@ static models_name_t mge_model_names [] =
 	{ "Eaton 5P", "850", EATON_5P, "5P 850" },
 	{ "Eaton 5P", "1150", EATON_5P, "5P 1150" },
 	{ "Eaton 5P", "1550", EATON_5P, "5P 1550" },
-	
+
 	/* Pulsar M models */
 	{ "PULSAR M", "2200", MGE_PULSAR_M_2200, NULL },
 	{ "PULSAR M", "3000", MGE_PULSAR_M_3000, NULL },
@@ -1072,7 +1154,7 @@ static models_name_t mge_model_names [] =
 	{ "GALAXY", "3000_30", MGE_DEFAULT, "Galaxy 3000 30 kVA" },
 
 	/* end of structure. */
-	{ NULL }
+	{ NULL, NULL, 0, NULL }
 };
 
 
@@ -1132,6 +1214,9 @@ static hid_info_t mge_hid2nut[] =
 	{ "ups.timer.reboot", 0, 0, "UPS.PowerSummary.DelayBeforeReboot", NULL, "%.0f", HU_FLAG_QUICK_POLL, NULL},
 	{ "ups.test.result", 0, 0, "UPS.BatterySystem.Battery.Test", NULL, "%s", 0, test_read_info },
 	{ "ups.test.interval", ST_FLAG_RW | ST_FLAG_STRING, 8, "UPS.BatterySystem.Battery.TestPeriod", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
+	/* Duplicate data for some units (such as 3S) that use a different path
+	 * Only the first valid one will be used */
+	{ "ups.beeper.status", 0 ,0, "UPS.BatterySystem.Battery.AudibleAlarmControl", NULL, "%s", HU_FLAG_SEMI_STATIC, beeper_info },
 	{ "ups.beeper.status", 0 ,0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "%s", HU_FLAG_SEMI_STATIC, beeper_info },
 	{ "ups.temperature", 0, 0, "UPS.PowerSummary.Temperature", NULL, "%s", 0, kelvin_celsius_conversion },
 	{ "ups.power", 0, 0, "UPS.PowerConverter.Output.ApparentPower", NULL, "%.0f", 0, NULL },
@@ -1140,6 +1225,9 @@ static hid_info_t mge_hid2nut[] =
 	{ "ups.L3.power", 0, 0, "UPS.PowerConverter.Output.Phase.[3].ApparentPower", NULL, "%.0f", 0, NULL },
 	{ "ups.power.nominal", 0, 0, "UPS.Flow.[4].ConfigApparentPower", NULL, "%.0f", HU_FLAG_STATIC, NULL },
 	{ "ups.realpower", 0, 0, "UPS.PowerConverter.Output.ActivePower", NULL, "%.0f", 0, NULL },
+	/* When not available, process an approximation from other data,
+	 * but map to apparent power to be called */
+	{ "ups.realpower", 0, 0, "UPS.Flow.[4].ConfigApparentPower", NULL, "-1", 0, eaton_compute_realpower_info },
 	{ "ups.L1.realpower", 0, 0, "UPS.PowerConverter.Output.Phase.[1].ActivePower", NULL, "%.0f", 0, NULL },
 	{ "ups.L2.realpower", 0, 0, "UPS.PowerConverter.Output.Phase.[2].ActivePower", NULL, "%.0f", 0, NULL },
 	{ "ups.L3.realpower", 0, 0, "UPS.PowerConverter.Output.Phase.[3].ActivePower", NULL, "%.0f", 0, NULL },
@@ -1160,7 +1248,6 @@ static hid_info_t mge_hid2nut[] =
 	/* Special case: boolean values that are mapped to ups.status and ups.alarm */
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.ACPresent", NULL, NULL, HU_FLAG_QUICK_POLL, online_info },
 	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[3].PresentStatus.Used", NULL, NULL, 0, mge_onbatt_info },
-	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[1].PresentStatus.Used", NULL, NULL, 0, online_info },
 	/* These 2 ones are used when ABM is disabled */
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.Discharging", NULL, NULL, HU_FLAG_QUICK_POLL, eaton_discharging_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.Charging", NULL, NULL, HU_FLAG_QUICK_POLL, eaton_charging_info },
@@ -1182,6 +1269,9 @@ static hid_info_t mge_hid2nut[] =
 	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[1].PresentStatus.VoltageOutOfRange", NULL, NULL, 0, vrange_info },
 	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[1].PresentStatus.FrequencyOutOfRange", NULL, NULL, 0, frange_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.Good", NULL, NULL, 0, off_info },
+	/* NOTE: UPS.PowerConverter.Input.[1].PresentStatus.Used" must only be considered when not "OFF",
+	 * and must hence be after "UPS.PowerSummary.PresentStatus.Good" */
+	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[1].PresentStatus.Used", NULL, NULL, 0, eaton_converter_online_info },
 	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[2].PresentStatus.Used", NULL, NULL, 0, bypass_auto_info }, /* Automatic bypass */
 	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[4].PresentStatus.Used", NULL, NULL, 0, bypass_manual_info }, /* Manual bypass */
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.FanFailure", NULL, NULL, 0, fanfail_info },
@@ -1328,8 +1418,13 @@ static hid_info_t mge_hid2nut[] =
 	{ "shutdown.reboot", 0, 0, "UPS.PowerSummary.DelayBeforeReboot", NULL, "10", HU_TYPE_CMD, NULL},
 	{ "beeper.off", 0, 0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "1", HU_TYPE_CMD, NULL },
 	{ "beeper.on", 0, 0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "2", HU_TYPE_CMD, NULL },
+	/* Duplicate commands for some units (such as 3S) that use a different path
+	 * Only the first valid one will be used */
+	{ "beeper.mute", 0, 0, "UPS.BatterySystem.Battery.AudibleAlarmControl", NULL, "3", HU_TYPE_CMD, NULL },
 	{ "beeper.mute", 0, 0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "3", HU_TYPE_CMD, NULL },
+	{ "beeper.disable", 0, 0, "UPS.BatterySystem.Battery.AudibleAlarmControl", NULL, "1", HU_TYPE_CMD, NULL },
 	{ "beeper.disable", 0, 0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "1", HU_TYPE_CMD, NULL },
+	{ "beeper.enable", 0, 0, "UPS.BatterySystem.Battery.AudibleAlarmControl", NULL, "2", HU_TYPE_CMD, NULL },
 	{ "beeper.enable", 0, 0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "2", HU_TYPE_CMD, NULL },
 
 	/* Command for the outlet collection */
@@ -1339,7 +1434,7 @@ static hid_info_t mge_hid2nut[] =
 	{ "outlet.2.load.on", 0, 0, "UPS.OutletSystem.Outlet.[3].DelayBeforeStartup", NULL, "0", HU_TYPE_CMD, NULL },
 
 	/* end of structure. */
-	{ NULL }
+	{ NULL, 0, 0, NULL, NULL, NULL, 0, NULL }
 };
 
 /*
@@ -1381,7 +1476,31 @@ static char *get_model_name(const char *iProduct, const char *iModel)
 		 * model name by concatenation of iProduct and iModel
 		 */
 		char	buf[SMALLBUF];
-		snprintf(buf, sizeof(buf), "%s %s", iProduct, iModel);
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_TRUNCATION
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_TRUNCATION
+#pragma GCC diagnostic ignored "-Wformat-truncation"
+#endif
+		/* NOTE: We intentionally limit the amount of bytes reported */
+		int len = snprintf(buf, sizeof(buf), "%s %s", iProduct, iModel);
+
+		if (len < 0) {
+			upsdebugx(1, "%s: got an error while extracting iProduct+iModel value", __func__);
+		}
+
+		/* NOTE: SMALLBUF here comes from mge_format_model()
+		 * buffer definitions below
+		 */
+		if ((intmax_t)len > (intmax_t)sizeof(buf)
+		|| (intmax_t)(strnlen(iProduct, SMALLBUF) + strnlen(iModel, SMALLBUF) + 1 + 1)
+		    > (intmax_t)sizeof(buf)
+		) {
+			upsdebugx(1, "%s: extracted iProduct+iModel value was truncated", __func__);
+		}
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_TRUNCATION
+#pragma GCC diagnostic pop
+#endif
 		return strdup(buf);
 	}
 
@@ -1449,6 +1568,21 @@ static int mge_claim(HIDDevice_t *hd) {
 				 * not a UPS, so don't use possibly_supported here
 				 */
 				return 0;
+
+			case PHOENIXTEC:
+				/* The vendorid 0x06da is primarily handled by
+				 * liebert-hid, except for (maybe) AEG PROTECT NAS
+				 * branded devices */
+				if (hd->Vendor && strstr(hd->Vendor, "AEG")) {
+					return 1;
+				}
+				if (hd->Product && strstr(hd->Product, "AEG")) {
+					return 1;
+				}
+
+				/* Let liebert-hid grab this */
+				return 0;
+
 			default: /* Valid for Eaton */
 				/* by default, reject, unless the productid option is given */
 				if (getval("productid")) {
@@ -1459,6 +1593,21 @@ static int mge_claim(HIDDevice_t *hd) {
 		}
 
 	case SUPPORTED:
+
+		switch (hd->VendorID)
+		{
+			case PHOENIXTEC: /* see comments above */
+				if (hd->Vendor && strstr(hd->Vendor, "AEG")) {
+					return 1;
+				}
+				if (hd->Product && strstr(hd->Product, "AEG")) {
+					return 1;
+				}
+
+				/* Let liebert-hid grab this */
+				return 0;
+		}
+
 		return 1;
 
 	case NOT_SUPPORTED:
@@ -1466,7 +1615,8 @@ static int mge_claim(HIDDevice_t *hd) {
 		return 0;
 	}
 #else
-			return 1;
+	NUT_UNUSED_VARIABLE(hd);
+	return 1;
 #endif
 }
 
@@ -1478,4 +1628,5 @@ subdriver_t mge_subdriver = {
 	mge_format_model,
 	mge_format_mfr,
 	mge_format_serial,
+	fix_report_desc,
 };
