@@ -2,6 +2,7 @@
  * nutdrv_atcl_usb.c - driver for generic-brand "ATCL FOR UPS"
  *
  * Copyright (C) 2013-2014 Charles Lepple <clepple+nut@gmail.com>
+ * Copyright (C) 2016      Eaton
  *
  * Loosely based on richcomm_usb.c,
  * Copyright (C) 2007 Peter van Valderen <p.v.valderen@probu.nl>
@@ -27,7 +28,7 @@
 
 /* driver version */
 #define DRIVER_NAME	"'ATCL FOR UPS' USB driver"
-#define DRIVER_VERSION	"1.1"
+#define DRIVER_VERSION	"1.16"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -56,8 +57,8 @@ static usb_device_id_t atcl_usb_id[] = {
 	/* ATCL FOR UPS */
 	{ USB_DEVICE(0x0001, 0x0000),  NULL },
 
-	/* end of list */
-	{-1, -1, NULL}
+	/* Terminating entry */
+	{ 0, 0, NULL }
 };
 
 static usb_dev_handle	*udev = NULL;
@@ -67,10 +68,12 @@ static unsigned int	comm_failures = 0;
 static int device_match_func(USBDevice_t *device, void *privdata)
 {
 	char *requested_vendor;
+	NUT_UNUSED_VARIABLE(privdata);
+
 	switch (is_usb_device_supported(atcl_usb_id, device))
 	{
 	case SUPPORTED:
-		if(!device->Vendor) { 
+		if(!device->Vendor) {
 			upsdebugx(1, "Couldn't retrieve USB string descriptor for vendor. Check permissions?");
 			requested_vendor = getval("vendor");
 			if(requested_vendor) {
@@ -78,7 +81,7 @@ static int device_match_func(USBDevice_t *device, void *privdata)
 					upsdebugx(3, "Matched device with NULL vendor string.");
 					return 1;
 				}
-			}	
+			}
 			upsdebugx(1, "To keep trying (in case your device does not have a vendor string), use vendor=NULL");
 			return 0;
 		}
@@ -94,16 +97,18 @@ static int device_match_func(USBDevice_t *device, void *privdata)
 				upsdebugx(3, "Matched device with vendor='%s'.", requested_vendor);
 				return 1;
 			} else {
-				upsdebugx(2, "idVendor=%04x and idProduct=%04x, but provided vendor '%s' does not match device: '%s'.",
+				upsdebugx(2, "idVendor=%04x and idProduct=%04x, "
+					"but provided vendor '%s' does not match device: '%s'.",
 					device->VendorID, device->ProductID, requested_vendor, device->Vendor);
 				return 0;
 			}
 		}
 
 		/* TODO: automatic way of suggesting other drivers? */
-		upsdebugx(2, "idVendor=%04x and idProduct=%04x, but device vendor string '%s' does not match expected string '%s'. "
-				"Have you tried the nutdrv_qx driver?",
-				device->VendorID, device->ProductID, device->Vendor, USB_VENDOR_STRING);
+		upsdebugx(2, "idVendor=%04x and idProduct=%04x, "
+			"but device vendor string '%s' does not match expected string '%s'. "
+			"Have you tried the nutdrv_qx driver?",
+			device->VendorID, device->ProductID, device->Vendor, USB_VENDOR_STRING);
 		return 0;
 
 	case POSSIBLY_SUPPORTED:
@@ -123,14 +128,15 @@ static int query_ups(char *reply)
 {
 	int	ret;
 
-	ret = usb_interrupt_read(udev, STATUS_ENDPOINT, reply, STATUS_PACKETSIZE, ATCL_USB_TIMEOUT);
+	ret = usb_interrupt_read(udev, STATUS_ENDPOINT, (usb_ctrl_charbuf)reply, STATUS_PACKETSIZE, ATCL_USB_TIMEOUT);
 
 	if (ret <= 0) {
-		upsdebugx(2, "status interrupt read: %s", ret ? usb_strerror() : "timeout");
+		upsdebugx(2, "status interrupt read: %s", ret ? nut_usb_strerror(ret) : "timeout");
 		return ret;
 	}
 
-	upsdebug_hex(3, "read", reply, ret);
+	assert ((uintmax_t)ret < (uintmax_t)SIZE_MAX);
+	upsdebug_hex(3, "read", reply, (size_t)ret);
 	return ret;
 }
 
@@ -163,7 +169,19 @@ static void usb_comm_fail(const char *fmt, ...)
 	}
 
 	va_start(ap, fmt);
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
+#pragma GCC diagnostic ignored "-Wformat-security"
+#endif
 	ret = vsnprintf(why, sizeof(why), fmt, ap);
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic pop
+#endif
 	va_end(ap);
 
 	if ((ret < 1) || (ret >= (int) sizeof(why))) {
@@ -179,7 +197,7 @@ static void usb_comm_good(void)
 		return;
 	}
 
-	upslogx(LOG_NOTICE, "Communications with UPS re-established");	
+	upslogx(LOG_NOTICE, "Communications with UPS re-established");
 	comm_failures = 0;
 }
 
@@ -192,13 +210,16 @@ static void usb_comm_good(void)
  */
 static int driver_callback(usb_dev_handle *handle, USBDevice_t *device)
 {
-	if (usb_set_configuration(handle, 1) < 0) {
-		upslogx(LOG_WARNING, "Can't set USB configuration: %s", usb_strerror());
+	int ret;
+	NUT_UNUSED_VARIABLE(device);
+
+	if ((ret = usb_set_configuration(handle, 1)) < 0) {
+		upslogx(LOG_WARNING, "Can't set USB configuration: %s", nut_usb_strerror(ret));
 		return -1;
 	}
 
-	if (usb_claim_interface(handle, 0) < 0) {
-		upslogx(LOG_WARNING, "Can't claim USB interface: %s", usb_strerror());
+	if ((ret = usb_claim_interface(handle, 0)) < 0) {
+		upslogx(LOG_WARNING, "Can't claim USB interface: %s", nut_usb_strerror(ret));
 		return -1;
 	}
 
@@ -209,31 +230,73 @@ static int driver_callback(usb_dev_handle *handle, USBDevice_t *device)
 
 static int usb_device_close(usb_dev_handle *handle)
 {
+	int ret = 0;
+
 	if (!handle) {
 		return 0;
 	}
 
 	/* usb_release_interface() sometimes blocks and goes
-	into uninterruptible sleep.  So don't do it. */
+	 * into uninterruptible sleep.  So don't do it.
+	 */
 	/* usb_release_interface(handle, 0); */
-	return usb_close(handle);
+
+#if WITH_LIBUSB_1_0
+	libusb_close(handle);
+	libusb_exit(NULL);
+#else
+	ret = usb_close(handle);
+#endif
+
+	return ret;
 }
 
 static int usb_device_open(usb_dev_handle **handlep, USBDevice_t *device, USBDeviceMatcher_t *matcher,
 	int (*callback)(usb_dev_handle *handle, USBDevice_t *device))
 {
-	struct usb_bus	*bus;
+	int ret = 0;
+	uint8_t iManufacturer = 0, iProduct = 0, iSerialNumber = 0;
 
 	/* libusb base init */
+#if WITH_LIBUSB_1_0
+	if (libusb_init(NULL) < 0) {
+		libusb_exit(NULL);
+		fatal_with_errno(EXIT_FAILURE, "Failed to init libusb 1.0");
+	}
+#else  /* => WITH_LIBUSB_0_1 */
 	usb_init();
 	usb_find_busses();
 	usb_find_devices();
+#endif /* WITH_LIBUSB_1_0 */
 
 #ifndef __linux__ /* SUN_LIBUSB (confirmed to work on Solaris and FreeBSD) */
 	/* Causes a double free corruption in linux if device is detached! */
-	usb_device_close(*handlep);
+	/* usb_device_close(*handlep); */
+	if (*handlep)
+		usb_close(*handlep);
 #endif
 
+#if WITH_LIBUSB_1_0
+	libusb_device **devlist;
+	ssize_t devcount = 0;
+	libusb_device_handle *handle;
+	struct libusb_device_descriptor dev_desc;
+	uint8_t bus;
+	int i;
+
+	devcount = libusb_get_device_list(NULL, &devlist);
+	if (devcount <= 0)
+		fatal_with_errno(EXIT_FAILURE, "No USB device found");
+
+	for (i = 0; i < devcount; i++) {
+
+		USBDeviceMatcher_t	*m;
+		libusb_device *dev = devlist[i];
+		libusb_get_device_descriptor(dev, &dev_desc);
+		ret = libusb_open(dev, &handle);
+		*handlep = handle;
+#else  /* => WITH_LIBUSB_0_1 */
+	struct usb_bus	*bus;
 	for (bus = usb_busses; bus; bus = bus->next) {
 
 		struct usb_device	*dev;
@@ -241,18 +304,22 @@ static int usb_device_open(usb_dev_handle **handlep, USBDevice_t *device, USBDev
 
 		for (dev = bus->devices; dev; dev = dev->next) {
 
-			int	i, ret;
+			int	i;
 			USBDeviceMatcher_t	*m;
 
-			upsdebugx(3, "Checking USB device [%04x:%04x] (%s/%s)", dev->descriptor.idVendor,
-				dev->descriptor.idProduct, bus->dirname, dev->filename);
-			
+			upsdebugx(3, "Checking USB device [%04x:%04x] (%s/%s)",
+				dev->descriptor.idVendor,
+				dev->descriptor.idProduct,
+				bus->dirname, dev->filename);
+
 			/* supported vendors are now checked by the supplied matcher */
 
 			/* open the device */
 			*handlep = handle = usb_open(dev);
+#endif /* WITH_LIBUSB_1_0 */
+
 			if (!handle) {
-				upsdebugx(4, "Failed to open USB device, skipping: %s", usb_strerror());
+				upsdebugx(4, "Failed to open USB device, skipping: %s", nut_usb_strerror(ret));
 				continue;
 			}
 
@@ -269,34 +336,70 @@ static int usb_device_open(usb_dev_handle **handlep, USBDevice_t *device, USBDev
 
 			memset(device, 0, sizeof(*device));
 
+#if WITH_LIBUSB_1_0
+			device->VendorID = dev_desc.idVendor;
+			device->ProductID = dev_desc.idProduct;
+			bus = libusb_get_bus_number(dev);
+			device->Bus = (char *)malloc(4);
+			if (device->Bus == NULL) {
+				libusb_free_device_list(devlist, 1);
+				fatal_with_errno(EXIT_FAILURE, "Out of memory");
+			}
+			sprintf(device->Bus, "%03d", bus);
+			iManufacturer = dev_desc.iManufacturer;
+			iProduct = dev_desc.iProduct;
+			iSerialNumber = dev_desc.iSerialNumber;
+#else  /* => WITH_LIBUSB_0_1 */
 			device->VendorID = dev->descriptor.idVendor;
 			device->ProductID = dev->descriptor.idProduct;
-			device->Bus = strdup(bus->dirname);
-			
-			if (dev->descriptor.iManufacturer) {
+			device->Bus = xstrdup(bus->dirname);
+			iManufacturer = dev->descriptor.iManufacturer;
+			iProduct = dev->descriptor.iProduct;
+			iSerialNumber = dev->descriptor.iSerialNumber;
+#endif /* WITH_LIBUSB_1_0 */
+
+			if (iManufacturer) {
 				char	buf[SMALLBUF];
-				ret = usb_get_string_simple(handle, dev->descriptor.iManufacturer,
-					buf, sizeof(buf));
+				ret = usb_get_string_simple(handle, iManufacturer,
+					(usb_ctrl_charbuf)buf, sizeof(buf));
 				if (ret > 0) {
 					device->Vendor = strdup(buf);
+					if (device->Vendor == NULL) {
+#if WITH_LIBUSB_1_0
+						libusb_free_device_list(devlist, 1);
+#endif	/* WITH_LIBUSB_1_0 */
+						fatal_with_errno(EXIT_FAILURE, "Out of memory");
+					}
 				}
 			}
 
-			if (dev->descriptor.iProduct) {
+			if (iProduct) {
 				char	buf[SMALLBUF];
-				ret = usb_get_string_simple(handle, dev->descriptor.iProduct,
-					buf, sizeof(buf));
+				ret = usb_get_string_simple(handle, iProduct,
+					(usb_ctrl_charbuf)buf, sizeof(buf));
 				if (ret > 0) {
 					device->Product = strdup(buf);
+					if (device->Product == NULL) {
+#if WITH_LIBUSB_1_0
+						libusb_free_device_list(devlist, 1);
+#endif	/* WITH_LIBUSB_1_0 */
+						fatal_with_errno(EXIT_FAILURE, "Out of memory");
+					}
 				}
 			}
 
-			if (dev->descriptor.iSerialNumber) {
+			if (iSerialNumber) {
 				char	buf[SMALLBUF];
-				ret = usb_get_string_simple(handle, dev->descriptor.iSerialNumber,
-					buf, sizeof(buf));
+				ret = usb_get_string_simple(handle, iSerialNumber,
+					(usb_ctrl_charbuf)buf, sizeof(buf));
 				if (ret > 0) {
 					device->Serial = strdup(buf);
+					if (device->Serial == NULL) {
+#if WITH_LIBUSB_1_0
+						libusb_free_device_list(devlist, 1);
+#endif	/* WITH_LIBUSB_1_0 */
+						fatal_with_errno(EXIT_FAILURE, "Out of memory");
+					}
 				}
 			}
 
@@ -308,49 +411,106 @@ static int usb_device_open(usb_dev_handle **handlep, USBDevice_t *device, USBDev
 			upsdebugx(4, "- Bus          : %s", device->Bus ? device->Bus : "unknown");
 
 			for (m = matcher; m; m = m->next) {
-				
+
 				switch (m->match_function(device, m->privdata))
 				{
 				case 0:
 					upsdebugx(4, "Device does not match - skipping");
 					goto next_device;
 				case -1:
+#if WITH_LIBUSB_1_0
+					libusb_free_device_list(devlist, 1);
+#endif	/* WITH_LIBUSB_1_0 */
 					fatal_with_errno(EXIT_FAILURE, "matcher");
+#ifndef HAVE___ATTRIBUTE__NORETURN
+# if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_UNREACHABLE_CODE)
+#  pragma GCC diagnostic push
+#  pragma GCC diagnostic ignored "-Wunreachable-code"
+# endif
 					goto next_device;
+# if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_UNREACHABLE_CODE)
+#  pragma GCC diagnostic pop
+# endif
+#endif
 				case -2:
 					upsdebugx(4, "matcher: unspecified error");
 					goto next_device;
 				}
 			}
+#ifdef HAVE_LIBUSB_SET_AUTO_DETACH_KERNEL_DRIVER
+		/* First, try the auto-detach kernel driver method
+		 * This function is not available on FreeBSD 10.1-10.3 */
+		if ((ret = libusb_set_auto_detach_kernel_driver (udev, 1)) < 0)
+			upsdebugx(2, "failed to auto detach kernel driver from USB device: %s",
+				nut_usb_strerror((enum libusb_error)ret));
+		else
+			upsdebugx(2, "auto detached kernel driver from USB device");
+#endif /* HAVE_LIBUSB_SET_AUTO_DETACH_KERNEL_DRIVER */
 
 			for (i = 0; i < 3; i++) {
 
 				ret = callback(handle, device);
 				if (ret >= 0) {
 					upsdebugx(3, "USB device [%04x:%04x] opened", device->VendorID, device->ProductID);
+#if WITH_LIBUSB_1_0
+					libusb_free_device_list(devlist, 1);
+#endif	/* WITH_LIBUSB_1_0 */
 					return ret;
 				}
-#ifdef HAVE_USB_DETACH_KERNEL_DRIVER_NP
+
+#if WITH_LIBUSB_0_1 && (defined HAVE_USB_DETACH_KERNEL_DRIVER_NP)
 				/* this method requires at least libusb 0.1.8:
 				 * it forces device claiming by unbinding
 				 * attached driver... From libhid */
-				if (usb_detach_kernel_driver_np(handle, 0) < 0) {
-					upsdebugx(1, "failed to detach kernel driver from USB device: %s", usb_strerror());
+				if ((ret = usb_detach_kernel_driver_np(handle, 0)) < 0) {
+					upsdebugx(1,
+						"failed to detach kernel driver from USB device: %s",
+						nut_usb_strerror(ret));
 				} else {
 					upsdebugx(4, "detached kernel driver from USB device...");
 				}
-#endif
+#else
+# ifdef HAVE_LIBUSB_DETACH_KERNEL_DRIVER
+				if ((ret = libusb_detach_kernel_driver(udev, 0)) < 0) {
+					upsdebugx(4,
+						"failed to detach kernel driver from USB device: %s",
+						nut_usb_strerror(ret));
+				} else {
+					upsdebugx(4, "detached kernel driver from USB device...");
+				}
+# else
+#  ifdef HAVE_LIBUSB_DETACH_KERNEL_DRIVER_NP
+				if ((ret = libusb_detach_kernel_driver_np(udev, 0)) < 0) {
+					upsdebugx(4,
+						"failed to detach kernel driver from USB device: %s",
+						nut_usb_strerror(ret));
+				} else {
+					upsdebugx(4, "detached kernel driver from USB device...");
+				}
+#  endif /* HAVE_LIBUSB_DETACH_KERNEL_DRIVER_NP */
+# endif /* HAVE_LIBUSB_DETACH_KERNEL_DRIVER */
+#endif /* HAVE_USB_DETACH_KERNEL_DRIVER_NP or HAVE_LIBUSB_DETACH_KERNEL_DRIVER or HAVE_LIBUSB_DETACH_KERNEL_DRIVER_NP */
 			}
 
-			fatalx(EXIT_FAILURE, "USB device [%04x:%04x] matches, but driver callback failed: %s",
-				device->VendorID, device->ProductID, usb_strerror());
+#if WITH_LIBUSB_1_0
+			libusb_free_device_list(devlist, 1);
+#endif	/* WITH_LIBUSB_1_0 */
+			fatalx(EXIT_FAILURE,
+				"USB device [%04x:%04x] matches, but driver callback failed: %s",
+				device->VendorID, device->ProductID,
+				nut_usb_strerror(ret));
 
 		next_device:
 			usb_close(handle);
+#if (!WITH_LIBUSB_1_0)  /* => WITH_LIBUSB_0_1 */
 		}
+#endif /* WITH_LIBUSB_1_0 */
 	}
 
 	*handlep = NULL;
+#if WITH_LIBUSB_1_0
+	libusb_free_device_list(devlist, 1);
+#endif	/* WITH_LIBUSB_1_0 */
 	upsdebugx(3, "No matching USB device found");
 
 	return -1;
@@ -446,8 +606,12 @@ void upsdrv_updateinfo(void)
 		case 2:
 			upsdebugx(2, "reply[0] = 0x%02x -> LB", reply[0]);
 			status_set("LB");
+			goto fallthrough_LB_means_OB;
+			/* Note: the comment below existed for years, so wondering
+			 * if this device CAN set independently LB and OB? */
 			/* fall through */
 		case 1:
+		fallthrough_LB_means_OB:
 			upsdebugx(2, "reply[0] = 0x%02x -> OB", reply[0]);
 			status_set("OB");
 			break;
@@ -466,24 +630,37 @@ void upsdrv_updateinfo(void)
  */
 void upsdrv_shutdown(void)
 {
-	const char	shutdown_packet[SHUTDOWN_PACKETSIZE] = { 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
+	/* Not "const" because this mismatches arg type of usb_interrupt_write() */
+	char	shutdown_packet[SHUTDOWN_PACKETSIZE] = { 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
 	int ret;
 
-	upslogx(LOG_DEBUG, "%s: attempting to call usb_interrupt_write(01 00 00 00 00 00 00 00)", __func__);
+	upslogx(LOG_DEBUG,
+		"%s: attempting to call usb_interrupt_write(01 00 00 00 00 00 00 00)",
+		__func__);
 
-	ret = usb_interrupt_write(udev, SHUTDOWN_ENDPOINT, (char *)shutdown_packet, SHUTDOWN_PACKETSIZE, ATCL_USB_TIMEOUT);
+	ret = usb_interrupt_write(udev,
+		SHUTDOWN_ENDPOINT, (usb_ctrl_charbuf)shutdown_packet,
+		SHUTDOWN_PACKETSIZE, ATCL_USB_TIMEOUT);
 
 	if (ret <= 0) {
-		upslogx(LOG_NOTICE, "%s: first usb_interrupt_write() failed: %s", __func__, ret ? usb_strerror() : "timeout");
+		upslogx(LOG_NOTICE,
+			"%s: first usb_interrupt_write() failed: %s",
+			__func__,
+			ret ? nut_usb_strerror(ret) : "timeout");
 	}
 
 	/* Totally guessing from the .pcap file here. TODO: configurable delay? */
 	usleep(170*1000);
 
-	ret = usb_interrupt_write(udev, SHUTDOWN_ENDPOINT, (char *)shutdown_packet, SHUTDOWN_PACKETSIZE, ATCL_USB_TIMEOUT);
+	ret = usb_interrupt_write(udev,
+		SHUTDOWN_ENDPOINT, (usb_ctrl_charbuf)shutdown_packet,
+		SHUTDOWN_PACKETSIZE, ATCL_USB_TIMEOUT);
 
 	if (ret <= 0) {
-		upslogx(LOG_ERR, "%s: second usb_interrupt_write() failed: %s", __func__, ret ? usb_strerror() : "timeout");
+		upslogx(LOG_ERR,
+			"%s: second usb_interrupt_write() failed: %s",
+			__func__,
+			ret ? nut_usb_strerror(ret) : "timeout");
 	}
 
 }
