@@ -198,16 +198,27 @@ int	blazer_process_setvar(item_t *item, char *value, const size_t valuelen)
 
 	if (!strcasecmp(item->info_type, "ups.delay.start")) {
 
-		int	ondelay = strtol(value, NULL, 10);
+		long	ondelay = strtol(value, NULL, 10);
+
+		if (ondelay < 0) {
+			upslogx(LOG_ERR, "%s: ondelay '%ld' should not be negative",
+				item->info_type, ondelay);
+			return -1;
+		}
 
 		/* Truncate to minute */
 		ondelay -= (ondelay % 60);
-
-		snprintf(value, valuelen, "%d", ondelay);
+		snprintf(value, valuelen, "%ld", ondelay);
 
 	} else if (!strcasecmp(item->info_type, "ups.delay.shutdown")) {
 
-		int	offdelay = strtol(value, NULL, 10);
+		long	offdelay = strtol(value, NULL, 10);
+
+		if (offdelay < 0) {
+			upslogx(LOG_ERR, "%s: offdelay '%ld' should not be negative",
+				item->info_type, offdelay);
+			return -1;
+		}
 
 		/* Truncate to nearest settable value */
 		if (offdelay < 60) {
@@ -216,7 +227,7 @@ int	blazer_process_setvar(item_t *item, char *value, const size_t valuelen)
 			offdelay -= (offdelay % 60);
 		}
 
-		snprintf(value, valuelen, "%d", offdelay);
+		snprintf(value, valuelen, "%ld", offdelay);
 
 	} else {
 
@@ -231,6 +242,16 @@ int	blazer_process_setvar(item_t *item, char *value, const size_t valuelen)
 /* Preprocess instant commands */
 int	blazer_process_command(item_t *item, char *value, const size_t valuelen)
 {
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
+#pragma GCC diagnostic ignored "-Wformat-security"
+#endif
+
 	if (!strcasecmp(item->info_type, "shutdown.return")) {
 
 		/* Sn: Shutdown after n minutes and then turn on when mains is back
@@ -242,25 +263,43 @@ int	blazer_process_command(item_t *item, char *value, const size_t valuelen)
 		 * The fix is to push the return value up by 2, i.e. S01R0003, and it will return online properly.
 		 * (thus the default of ondelay=3 mins) */
 
-		int	offdelay = strtol(dstate_getinfo("ups.delay.shutdown"), NULL, 10),
-			ondelay = strtol(dstate_getinfo("ups.delay.start"), NULL, 10) / 60;
+		long	offdelay = strtol(dstate_getinfo("ups.delay.shutdown"), NULL, 10),
+				ondelay  = strtol(dstate_getinfo("ups.delay.start"), NULL, 10) / 60;
 		char	buf[SMALLBUF] = "";
 
-		if (ondelay == 0) {
+		if (ondelay <= 0) {
+
+			if (offdelay < 0) {
+				upslogx(LOG_ERR, "%s: offdelay '%ld' should not be negative",
+					item->info_type, offdelay);
+				return -1;
+			}
 
 			if (offdelay < 60) {
-				snprintf(buf, sizeof(buf), ".%d", offdelay / 6);
+				snprintf(buf, sizeof(buf), ".%ld", offdelay / 6);
 			} else {
-				snprintf(buf, sizeof(buf), "%02d", offdelay / 60);
+				snprintf(buf, sizeof(buf), "%02ld", offdelay / 60);
 			}
 
 		} else if (offdelay < 60) {
 
-			snprintf(buf, sizeof(buf), ".%dR%04d", offdelay / 6, ondelay);
+			if (offdelay < 0) {
+				upslogx(LOG_ERR, "%s: offdelay '%ld' should not be negative",
+					item->info_type, offdelay);
+				return -1;
+			}
+
+			snprintf(buf, sizeof(buf), ".%ldR%04ld", offdelay / 6, ondelay);
 
 		} else {
 
-			snprintf(buf, sizeof(buf), "%02dR%04d", offdelay / 60, ondelay);
+			if (offdelay < 0) {
+				upslogx(LOG_ERR, "%s: offdelay '%ld' should not be negative",
+					item->info_type, offdelay);
+				return -1;
+			}
+
+			snprintf(buf, sizeof(buf), "%02ldR%04ld", offdelay / 60, ondelay);
 
 		}
 
@@ -272,23 +311,29 @@ int	blazer_process_command(item_t *item, char *value, const size_t valuelen)
 		 * Shutdown after n minutes and stay off
 		 * Accepted values for n: .2 -> .9 , 01 -> 10 */
 
-		int	offdelay = strtol(dstate_getinfo("ups.delay.shutdown"), NULL, 10);
+		long	offdelay = strtol(dstate_getinfo("ups.delay.shutdown"), NULL, 10);
 		char	buf[SMALLBUF] = "";
 
+		if (offdelay < 0) {
+			upslogx(LOG_ERR, "%s: offdelay '%ld' should not be negative",
+				item->info_type, offdelay);
+			return -1;
+		}
+
 		if (offdelay < 60) {
-			snprintf(buf, sizeof(buf), ".%d", offdelay / 6);
+			snprintf(buf, sizeof(buf), ".%ld", offdelay / 6);
 		} else {
-			snprintf(buf, sizeof(buf), "%02d", offdelay / 60);
+			snprintf(buf, sizeof(buf), "%02ld", offdelay / 60);
 		}
 
 		snprintf(value, valuelen, item->command, buf);
 
 	} else if (!strcasecmp(item->info_type, "test.battery.start")) {
 
-		int	delay = strlen(value) > 0 ? strtol(value, NULL, 10) : 600;
+		long	delay = strlen(value) > 0 ? strtol(value, NULL, 10) : 600;
 
 		if ((delay < 60) || (delay > 5940)) {
-			upslogx(LOG_ERR, "%s: battery test time '%d' out of range [60..5940] seconds", item->info_type, delay);
+			upslogx(LOG_ERR, "%s: battery test time '%ld' out of range [60..5940] seconds", item->info_type, delay);
 			return -1;
 		}
 
@@ -303,6 +348,10 @@ int	blazer_process_command(item_t *item, char *value, const size_t valuelen)
 
 	}
 
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
+#pragma GCC diagnostic pop
+#endif
+
 	return 0;
 }
 
