@@ -32,10 +32,10 @@
  * Enhancing the driver for additional capabilities and later units.
  *
  * - 28 November 2003.  Eric Lawson
- * More or less complete re-write for NUT 1.5.9 
- * This was somewhat easier than trying to beat the old driver code 
+ * More or less complete re-write for NUT 1.5.9
+ * This was somewhat easier than trying to beat the old driver code
  * into submission
- * 
+ *
  */
 
 #include "main.h"
@@ -47,7 +47,7 @@ int setcmd(const char* varname, const char* setvalue);
 int instcmd(const char *cmdname, const char *extra);
 
 #define DRIVER_NAME	"Oneac EG/ON/OZ/OB UPS driver"
-#define DRIVER_VERSION	"0.80"
+#define DRIVER_VERSION	"0.81"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -74,22 +74,24 @@ static char UpsFamily [3];
  *  allow for a repeat of the get request.  Also confirm that
  *  the correct number of characters are returned.
  */
- 
-int OneacGetResponse (char* chBuff, const int BuffSize, int ExpectedCount)
+
+static ssize_t OneacGetResponse (char* chBuff, const size_t BuffSize, int ExpectedCount)
 {
 	int Retries = 10;		/* x/2 seconds max with 500000 USEC */
-	int return_val;
+	ssize_t return_val;
 
 	do
 	{
-		return_val = ser_get_line(upsfd, chBuff, BuffSize, ENDCHAR, IGNCHARS, 
-																	SECS, USEC);
+		return_val = ser_get_line(upsfd,
+			chBuff, BuffSize, ENDCHAR, IGNCHARS, SECS, USEC);
 
 		if (return_val == ExpectedCount)
 			break;
 
-		upsdebugx (3,"!OneacGetResponse retry (%d, %d)...", return_val,Retries);
-		
+		upsdebugx (3,
+			"!OneacGetResponse retry (%zd, %d)...",
+			return_val, Retries);
+
 	} while (--Retries > 0);
 
 	upsdebugx (4,"OneacGetResponse buffer: %s",chBuff);
@@ -103,14 +105,14 @@ int OneacGetResponse (char* chBuff, const int BuffSize, int ExpectedCount)
 	{
 		if (Retries < 10)
 			upsdebugx (2,"OneacGetResponse recovered (%d)...", Retries);
-			
+
 		return_val = 0;					/* Good comms */
 	}
-	
-    return return_val;
+
+	return return_val;
 }
 
-void do_battery_test(void)
+static void do_battery_test(void)
 {
 	char buffer[32];
 
@@ -130,7 +132,7 @@ void do_battery_test(void)
 	ser_send(upsfd,"%s%s%s",BAT_TEST_PREFIX,buffer,COMMAND_END);
 }
 
-int SetOutputAllow(const char* lowval, const char* highval)
+static int SetOutputAllow(const char* lowval, const char* highval)
 {
 	char buffer[32];
 
@@ -146,38 +148,38 @@ int SetOutputAllow(const char* lowval, const char* highval)
 		buffer[1] = buffer[0];
 		buffer[0] = '0';
 	}
-	
-	upsdebugx (2,"SetOutputAllow sending %s%.3s,%.3s...", 
+
+	upsdebugx (2,"SetOutputAllow sending %s%.3s,%.3s...",
 											SETX_OUT_ALLOW, buffer, highval);
-					
-	ser_send(upsfd,"%s%.3s,%.3s%s", SETX_OUT_ALLOW, buffer, highval, 
+
+	ser_send(upsfd,"%s%.3s,%.3s%s", SETX_OUT_ALLOW, buffer, highval,
 																COMMAND_END);
 	ser_get_line(upsfd,buffer,sizeof(buffer), ENDCHAR, IGNCHARS,SECS,USEC);
-	
+
 	if(buffer[0] == DONT_UNDERSTAND)
 	{
 		upsdebugx (2,"SetOutputAllow got asterisk back...");
 
 		return 1;					/* Invalid command */
 	}
-	
+
 	return 0;						/* Valid command */
 }
 
-void EliminateLeadingZeroes (const char* buff1, int StringSize, char* buff2, 
-															const int buff2size)
+static void EliminateLeadingZeroes (const char* buff1, int StringSize, char* buff2,
+															const size_t buff2size)
 {
 	int i = 0;
 	int j = 0;
 
 	memset(buff2, '\0', buff2size);			/* Fill with nulls */
-	
+
 	/* Find first non-'0' */
-	while ((buff1[i] == '0') && (i < (StringSize - 1)))
+	while ((i < (StringSize - 1) && (buff1[i] == '0')))
 	{
 		i++;
 	}
-	
+
 	while (i < StringSize)					/* Move rest of string */
 	{
 		buff2[j++] = buff1[i++];
@@ -205,7 +207,7 @@ void upsdrv_initinfo(void)
 	int i,j, k;
 	int VRange=0;
 	int timevalue;
-	int RetValue;
+	ssize_t RetValue;
 	char buffer[256], buffer2[32];
 
 	/* All families should reply to this request so we can confirm that it is
@@ -222,13 +224,13 @@ void upsdrv_initinfo(void)
 	}
 	else
 	{
-		if (strncmp(buffer,FAMILY_ON,FAMILY_SIZE) != 0 && 
+		if (strncmp(buffer,FAMILY_ON,FAMILY_SIZE) != 0 &&
 			strncmp(buffer,FAMILY_OZ,FAMILY_SIZE) != 0 &&
-			strncmp(buffer,FAMILY_OB,FAMILY_SIZE) != 0 && 
+			strncmp(buffer,FAMILY_OB,FAMILY_SIZE) != 0 &&
 			strncmp(buffer,FAMILY_EG,FAMILY_SIZE) != 0)
 		{
 			fatalx(EXIT_FAILURE, "Did not find an ONEAC UPS on %s\n",
-																device_path);	
+																device_path);
 		}
 	}
 
@@ -239,7 +241,7 @@ void upsdrv_initinfo(void)
 	printf("Found %s family of Oneac UPS\n", UpsFamily);
 
 	dstate_setinfo("ups.type", "%s", "Line Interactive");
-	
+
 	dstate_addcmd("test.battery.start.quick");
 	dstate_addcmd("test.battery.stop");
 	dstate_addcmd("test.failure.start");
@@ -257,7 +259,7 @@ void upsdrv_initinfo(void)
 
 	if (strncmp(UpsFamily, FAMILY_EG, FAMILY_SIZE) == 0)
 	{
-		RetValue = OneacGetResponse (buffer, sizeof(buffer), 
+		RetValue = OneacGetResponse (buffer, sizeof(buffer),
 														GETALL_EG_RESP_SIZE);
 	}
 	else
@@ -289,7 +291,7 @@ void upsdrv_initinfo(void)
 
 	dstate_setflags("ups.delay.shutdown", ST_FLAG_STRING | ST_FLAG_RW);
 	dstate_setaux("ups.delay.shutdown", GET_SHUTDOWN_RESP_SIZE);
-	
+
 	/* Setup some ON/OZ/OB only stuff ... i.e. not EG */
 
 	if (strncmp(UpsFamily, FAMILY_EG, FAMILY_SIZE) != 0)
@@ -299,7 +301,7 @@ void upsdrv_initinfo(void)
 		/*nominal input voltage*/
 
 		VRange = buffer[26];			/* Keep for later use also */
-		
+
 		switch (VRange)					/* Will be '1' or '2' */
 		{
 			case V120AC:
@@ -317,9 +319,9 @@ void upsdrv_initinfo(void)
 					"Invalid nom voltage parameter from UPS [%c]", VRange);
 		}
 	}
-		  
+
 	/* Setup some OZ/OB only stuff */
-	
+
 	if ((strncmp (UpsFamily, FAMILY_OZ, FAMILY_SIZE) == 0) ||
 		(strncmp (UpsFamily, FAMILY_OB, FAMILY_SIZE) == 0))
 	{
@@ -328,7 +330,7 @@ void upsdrv_initinfo(void)
 		dstate_addcmd("beeper.enable");
 		dstate_addcmd("beeper.disable");
 		dstate_addcmd("beeper.mute");
-		
+
 		dstate_setaux("ups.delay.shutdown", GETX_SHUTDOWN_RESP_SIZE);
 
 		ser_flush_in(upsfd,"",0);
@@ -355,7 +357,7 @@ void upsdrv_initinfo(void)
 		dstate_setinfo("ups.delay.start", "%s", buffer2);
 		dstate_setflags("ups.delay.start", ST_FLAG_STRING | ST_FLAG_RW);
 		dstate_setaux("ups.delay.start", 4);
-		
+
 		/* Low Batt at time */
 		strncpy(buffer2, buffer+82, 2);
 		buffer2[2]='\0';
@@ -363,24 +365,36 @@ void upsdrv_initinfo(void)
 		dstate_setinfo("battery.runtime.low", "%d",timevalue);
 		dstate_setflags("battery.runtime.low", ST_FLAG_STRING | ST_FLAG_RW);
 		dstate_setaux("battery.runtime.low", 2);
-		
+
 		/*Get the actual model string for ON UPS reported as OZ/OB family*/
 
 		/*UPS Model (full string)*/
 		memset(buffer2, '\0', 32);
-		strncpy(buffer2, buffer+5, 10);
-		for (i = 9; i >= 0 && buffer2[i] == ' '; --i) 
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_TRUNCATION
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_TRUNCATION
+#pragma GCC diagnostic ignored "-Wformat-truncation"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_STRINGOP_TRUNCATION
+#pragma GCC diagnostic ignored "-Wstringop-truncation"
+#endif
+		strncpy(buffer2, buffer + 5, 10);
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_TRUNCATION
+#pragma GCC diagnostic pop
+#endif
+		for (i = 9; i >= 0 && buffer2[i] == ' '; --i)
 		{
 			buffer2[i] = '\0';
 		}
 
 		dstate_setinfo("device.model", "%s", buffer2);
-		
+
 		/* Serial number */
 		dstate_setinfo("device.serial", "%.4s-%.4s", buffer+18, buffer+22);
-		printf("Found %.10s UPS with serial number %.4s-%.4s\n", 
+		printf("Found %.10s UPS with serial number %.4s-%.4s\n",
 												buffer2, buffer+18, buffer+22);
-		
+
 		/* Manufacture Date */
 		dstate_setinfo("ups.mfr.date", "%.6s (yymmdd)", buffer+38);
 
@@ -388,7 +402,7 @@ void upsdrv_initinfo(void)
 		dstate_setinfo("battery.date", "%.6s (yymmdd)", buffer+44);
 		dstate_setflags("battery.date", ST_FLAG_STRING | ST_FLAG_RW);
 		dstate_setaux("battery.date", 6);
-		
+
 		/* Real power nominal */
 		EliminateLeadingZeroes (buffer+55, 5, buffer2, sizeof(buffer2));
 		dstate_setinfo("ups.realpower.nominal", "%s", buffer2);
@@ -402,27 +416,39 @@ void upsdrv_initinfo(void)
 		if ((strncmp (UpsFamily, FAMILY_OB, FAMILY_SIZE) == 0) ||
 			(strcmp (dstate_getinfo("ups.firmware"), MIN_ALLOW_FW) >= 0 ))
 		{
-			upsdebugx (2,"Can get output window min/max! (%s)", 
+			upsdebugx (2,"Can get output window min/max! (%s)",
 												dstate_getinfo("ups.firmware"));
 
 			ser_send(upsfd,"%s%s",GETX_ALLOW_RANGE,COMMAND_END);
 			if(OneacGetResponse (buffer, sizeof(buffer), GETX_RANGE_RESP_SIZE))
 			{
-				fatalx(EXIT_FAILURE, 
+				fatalx(EXIT_FAILURE,
 						"Serial timeout(4) with ONEAC UPS on %s\n",device_path);
 			}
 
 			strncpy(buffer2, buffer, 3);
 			buffer2[3]='\0';
 			i = atoi(buffer2);		/* Minimum voltage */
-			
-			strncpy(buffer2, buffer+4, 3);
+
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_TRUNCATION
+#pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_TRUNCATION
+#pragma GCC diagnostic ignored "-Wformat-truncation"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_STRINGOP_TRUNCATION
+#pragma GCC diagnostic ignored "-Wstringop-truncation"
+#endif
+			strncpy(buffer2, buffer + 4, 3);
+#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_TRUNCATION
+#pragma GCC diagnostic pop
+#endif
 			j = atoi(buffer2);		/* Maximum voltage */
 
 			strncpy(buffer2, buffer+8, 2);
 			buffer2[2]='\0';
 			k = atoi(buffer2);		/* Spread between */
-			
+
 			dstate_setinfo("input.transfer.low.min", "%3d", i);
 			dstate_setinfo("input.transfer.low.max", "%3d", j-k);
 			dstate_setinfo("input.transfer.high.min", "%3d", i+k);
@@ -432,7 +458,7 @@ void upsdrv_initinfo(void)
 		else
 		{
 			/* Use default values from firmware */
-			upsdebugx (2,"Using trip defaults (%s)...", 
+			upsdebugx (2,"Using trip defaults (%s)...",
 												dstate_getinfo("ups.firmware"));
 
 			switch (VRange)				/* Held from initial use */
@@ -465,7 +491,7 @@ void upsdrv_updateinfo(void)
 	char buffer[256];	/* Main response buffer */
 	char buffer2[32];	/* Conversion buffer */
 	char s;
-	int RetValue;
+	ssize_t RetValue;
 	int timevalue;
 
 	/* Start with EG/ON information */
@@ -485,7 +511,7 @@ void upsdrv_updateinfo(void)
 	{
 		ser_comm_fail("Oneac UPS Comm failure continues on port %s",
 																device_path);
-	}	
+	}
 	else if (RetValue != 0)
 	{
 		if (--CommTry == 0)
@@ -521,10 +547,10 @@ void upsdrv_updateinfo(void)
 			{
 				status_set("OL");
 			}
-			
+
 			if (s & 0x02)			/* Low Battery */
 				status_set("LB");
-			
+
 			if (s & 0x04)			/* General fault */
 			{
 				dstate_setinfo("ups.test.result","UPS Internal Failure");
@@ -557,7 +583,7 @@ void upsdrv_updateinfo(void)
 					"High Input Voltage");
 				break;
 			case NO_VALUE_YET :
-				dstate_setinfo("input.transfer.reason", 
+				dstate_setinfo("input.transfer.reason",
 					"No transfer yet.");
 				break;
 			default :
@@ -578,7 +604,7 @@ void upsdrv_updateinfo(void)
 			/*battery charge*/
 			if(buffer[10] == YES)
 				dstate_setinfo("battery.charge", "0%.2s",buffer+33);
-			else 
+			else
 				dstate_setinfo("battery.charge", "100");
 
 			EliminateLeadingZeroes (buffer+35, 3, buffer2, sizeof(buffer2));
@@ -601,10 +627,10 @@ void upsdrv_updateinfo(void)
 			{
 				/* A shutdown is underway! */
 				status_set("FSD");
-				
+
 				if(buffer[15] != HIGH_COUNT)
 				{
-					EliminateLeadingZeroes (buffer+15, 3, buffer2, 
+					EliminateLeadingZeroes (buffer+15, 3, buffer2,
 															sizeof(buffer2));
 					dstate_setinfo("ups.timer.shutdown", "%s", buffer2);
 				}
@@ -614,18 +640,18 @@ void upsdrv_updateinfo(void)
 				}
 			}
 
-			if (buffer[47] == YES) 
+			if (buffer[47] == YES)
 				status_set("BOOST");
 		}
 
 		/* Now update info for only the OZ/OB families of UPS */
 
 		if ((strncmp(UpsFamily, FAMILY_OZ, FAMILY_SIZE) == 0) ||
-			(strncmp(UpsFamily, FAMILY_OB, FAMILY_SIZE) == 0)) 
+			(strncmp(UpsFamily, FAMILY_OB, FAMILY_SIZE) == 0))
 		{
 			ser_flush_in(upsfd,"",0);  /*just in case*/
 			ser_send (upsfd,"%c%s",GETX_ALL_1,COMMAND_END);
-			RetValue = OneacGetResponse (buffer, sizeof(buffer), 
+			RetValue = OneacGetResponse (buffer, sizeof(buffer),
 														GETX_ALL1_RESP_SIZE);
 
 			if(RetValue)
@@ -667,13 +693,13 @@ void upsdrv_updateinfo(void)
 
 				/* Battery replace date */
 				ser_send (upsfd,"%c%s",GETX_BATT_REPLACED,COMMAND_END);
-				if(!OneacGetResponse (buffer, sizeof(buffer), 
+				if(!OneacGetResponse (buffer, sizeof(buffer),
 														GETX_DATE_RESP_SIZE))
 					dstate_setinfo("battery.date", "%.6s (yymmdd)", buffer);
 
 				/* Low and high output trip points */
 				ser_send (upsfd,"%c%s",GETX_LOW_OUT_ALLOW,COMMAND_END);
-				if(!OneacGetResponse (buffer, sizeof(buffer), 
+				if(!OneacGetResponse (buffer, sizeof(buffer),
 														GETX_ALLOW_RESP_SIZE))
 				{
 					EliminateLeadingZeroes (buffer, 3, buffer2,sizeof(buffer2));
@@ -681,16 +707,16 @@ void upsdrv_updateinfo(void)
 				}
 
 				ser_send (upsfd,"%c%s",GETX_HI_OUT_ALLOW,COMMAND_END);
-				if(!OneacGetResponse (buffer, sizeof(buffer), 
+				if(!OneacGetResponse (buffer, sizeof(buffer),
 														GETX_ALLOW_RESP_SIZE))
 					dstate_setinfo("input.transfer.high", "%s", buffer);
 
 				/* Restart delay */
 				ser_send (upsfd,"%c%s",GETX_RESTART_DLY,COMMAND_END);
-				if(!OneacGetResponse (buffer, sizeof(buffer), 
+				if(!OneacGetResponse (buffer, sizeof(buffer),
 														GETX_RSTRT_RESP_SIZE))
 				{
-					EliminateLeadingZeroes (buffer, 4, buffer2, 
+					EliminateLeadingZeroes (buffer, 4, buffer2,
 															sizeof(buffer2));
 					dstate_setinfo("ups.delay.start", "%s", buffer2);
 				}
@@ -699,7 +725,7 @@ void upsdrv_updateinfo(void)
 				ser_send (upsfd,"%s%s",GETX_BUZZER_WHAT,COMMAND_END);
 				if(!OneacGetResponse (buffer, sizeof(buffer), 1))
 				{
-					switch (buffer[0]) 
+					switch (buffer[0])
 					{
 						case BUZZER_ENABLED :
 							dstate_setinfo("ups.beeper.status",	"enabled");
@@ -737,15 +763,15 @@ void upsdrv_updateinfo(void)
 
 				/* Shutdown timer */
 				ser_send (upsfd,"%c%s",GETX_SHUTDOWN,COMMAND_END);
-				if(!OneacGetResponse (buffer, sizeof(buffer), 
+				if(!OneacGetResponse (buffer, sizeof(buffer),
 													GETX_SHUTDOWN_RESP_SIZE))
 				{
-					/* ON would have handled NO_VALUE_YET and setting FSD 
+					/* ON would have handled NO_VALUE_YET and setting FSD
 					 *  above so only deal with counter value here.
 					 */
 					if (buffer[0] != NO_VALUE_YET)
 					{
-						EliminateLeadingZeroes (buffer, 5, buffer2, 
+						EliminateLeadingZeroes (buffer, 5, buffer2,
 															sizeof(buffer2));
 						dstate_setinfo("ups.timer.shutdown", "%s", buffer2);
 					}
@@ -753,7 +779,7 @@ void upsdrv_updateinfo(void)
 
 				/* Restart timer */
 				ser_send (upsfd,"%s%s",GETX_RESTART_COUNT,COMMAND_END);
-				if(!OneacGetResponse (buffer, sizeof(buffer), 
+				if(!OneacGetResponse (buffer, sizeof(buffer),
 														GETX_RSTRT_RESP_SIZE))
 				{
 					if (atoi(buffer) == 0)
@@ -762,7 +788,7 @@ void upsdrv_updateinfo(void)
 					}
 					else
 					{
-						EliminateLeadingZeroes (buffer, 4, buffer2, 
+						EliminateLeadingZeroes (buffer, 4, buffer2,
 															sizeof(buffer2));
 						dstate_setinfo("ups.timer.start", "%s", buffer2);
 					}
@@ -839,7 +865,7 @@ int instcmd(const char *cmdname, const char *extra)
 
 		return STAT_INSTCMD_HANDLED;
 	}
-	
+
 	if(!strcasecmp(cmdname, "shutdown.reboot")) {
 		ser_send(upsfd, "%s", SHUTDOWN);
 		return STAT_INSTCMD_HANDLED;
@@ -860,7 +886,7 @@ int instcmd(const char *cmdname, const char *extra)
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	if (!strcasecmp(cmdname, "test.battery.stop")) 
+	if (!strcasecmp(cmdname, "test.battery.stop"))
 	{
 		if ((strncmp (UpsFamily, FAMILY_EG, FAMILY_SIZE) == 0) ||
 			(strncmp (UpsFamily, FAMILY_ON, FAMILY_SIZE) == 0))
@@ -870,7 +896,7 @@ int instcmd(const char *cmdname, const char *extra)
 		else
 		{
 			ser_send(upsfd,"%c%s",TEST_ABORT,COMMAND_END);
-		}			
+		}
 		return STAT_INSTCMD_HANDLED;
 	}
 
@@ -992,7 +1018,7 @@ int setcmd(const char* varname, const char* setvalue)
 		if (atoi(setvalue) <= 99)
 		{
 			ser_send(upsfd,"%s%s%s",SETX_LOWBATT_AT, setvalue, COMMAND_END);
-	
+
 			dstate_setinfo("battery.runtime.low", "%s", setvalue);
 			return STAT_SET_HANDLED;
 		}
@@ -1016,7 +1042,7 @@ int setcmd(const char* varname, const char* setvalue)
 			dstate_setinfo("ups.start.auto", "no");
 			return STAT_SET_HANDLED;
 		}
-		
+
 		return STAT_SET_UNKNOWN;
 	}
 
