@@ -25,7 +25,7 @@
 #include "powercom-hid.h"
 #include "usb-common.h"
 
-#define POWERCOM_HID_VERSION	"PowerCOM HID 0.5"
+#define POWERCOM_HID_VERSION	"PowerCOM HID 0.6"
 /* FIXME: experimental flag to be put in upsdrv_info */
 
 /* PowerCOM */
@@ -48,7 +48,7 @@ static usb_device_id_t powercom_usb_device_table[] = {
 	{ USB_DEVICE(POWERCOM_VENDORID, 0x0001), NULL },
 
 	/* Terminating entry */
-	{ -1, -1, NULL }
+	{ 0, 0, NULL }
 };
 
 static char powercom_scratch_buf[32];
@@ -67,9 +67,18 @@ static double powercom_startup_nuf(const char *value)
 {
 	const char	*s = dstate_getinfo("ups.delay.start");
 	uint16_t	val, command;
+	int iv;
 
-	val = atoi(value ? value : s) / 60;
-	command = ((val << 8) + (val >> 8));
+	iv = atoi(value ? value : s) / 60;
+	if (iv < 0 || (intmax_t)iv > (intmax_t)UINT16_MAX) {
+		upsdebugx(0, "%s: value = %d is not in uint16_t range", __func__, iv);
+		return 0;
+	}
+
+	/* COMMENTME: What are we doing here, a byte-swap in the word? */
+	val = (uint16_t)iv;
+	command =  (uint16_t)(val << 8);
+	command += (uint16_t)(val >> 8);
 	upsdebugx(3, "%s: value = %s, command = %04X", __func__, value, command);
 
 	return command;
@@ -93,10 +102,17 @@ static double powercom_shutdown_nuf(const char *value)
 {
 	const char	*s = dstate_getinfo("ups.delay.shutdown");
 	uint16_t	val, command;
+	int iv;
 
-	val = atoi(value ? value : s);
+	iv = atoi(value ? value : s);
+	if (iv < 0 || (intmax_t)iv > (intmax_t)UINT16_MAX) {
+		upsdebugx(0, "%s: value = %d is not in uint16_t range", __func__, iv);
+		return 0;
+	}
+
+	val = (uint16_t)iv;
 	val = val ? val : 1;    /* 0 sets the maximum delay */
-	command = ((val % 60) << 8) + (val / 60);
+	command = ((uint16_t)((val % 60) << 8)) + (uint16_t)(val / 60);
 	command |= 0x4000;	/* AC RESTART NORMAL ENABLE */
 	upsdebugx(3, "%s: value = %s, command = %04X", __func__, value, command);
 
@@ -111,10 +127,17 @@ static double powercom_stayoff_nuf(const char *value)
 {
 	const char	*s = dstate_getinfo("ups.delay.shutdown");
 	uint16_t	val, command;
+	int iv;
 
-	val = atoi(value ? value : s);
+	iv = atoi(value ? value : s);
+	if (iv < 0 || (intmax_t)iv > (intmax_t)UINT16_MAX) {
+		upsdebugx(0, "%s: value = %d is not in uint16_t range", __func__, iv);
+		return 0;
+	}
+
+	val = (uint16_t)iv;
 	val = val ? val : 1;    /* 0 sets the maximum delay */
-	command = ((val % 60) << 8) + (val / 60);
+	command = ((uint16_t)((val % 60) << 8)) + (uint16_t)(val / 60);
 	command |= 0x8000;	/* AC RESTART NORMAL DISABLE */
 	upsdebugx(3, "%s: value = %s, command = %04X", __func__, value, command);
 
@@ -126,9 +149,9 @@ static info_lkp_t powercom_stayoff_info[] = {
 };
 
 static info_lkp_t powercom_beeper_info[] = {
-	{ 1, "enabled", NULL },
-	{ 2, "disabled", NULL },	/* muted? */
-	{ 0, NULL, NULL }
+	{ 1, "enabled", NULL, NULL },
+	{ 2, "disabled", NULL, NULL },	/* muted? */
+	{ 0, NULL, NULL, NULL }
 };
 
 static const char *powercom_voltage_conversion_fun(double value)
@@ -139,7 +162,7 @@ static const char *powercom_voltage_conversion_fun(double value)
 }
 
 static info_lkp_t powercom_voltage_conversion[] = {
-	{ 0, NULL, powercom_voltage_conversion_fun }
+	{ 0, NULL, powercom_voltage_conversion_fun, NULL }
 };
 
 static const char *powercom_upsfail_conversion_fun(double value)
@@ -152,7 +175,7 @@ static const char *powercom_upsfail_conversion_fun(double value)
 }
 
 static info_lkp_t powercom_upsfail_conversion[] = {
-	{ 0, NULL, powercom_upsfail_conversion_fun }
+	{ 0, NULL, powercom_upsfail_conversion_fun, NULL }
 };
 
 static const char *powercom_replacebatt_conversion_fun(double value)
@@ -165,7 +188,7 @@ static const char *powercom_replacebatt_conversion_fun(double value)
 }
 
 static info_lkp_t powercom_replacebatt_conversion[] = {
-	{ 0, NULL, powercom_replacebatt_conversion_fun }
+	{ 0, NULL, powercom_replacebatt_conversion_fun, NULL }
 };
 
 static const char *powercom_test_conversion_fun(double value)
@@ -178,7 +201,7 @@ static const char *powercom_test_conversion_fun(double value)
 }
 
 static info_lkp_t powercom_test_conversion[] = {
-	{ 0, NULL, powercom_test_conversion_fun }
+	{ 0, NULL, powercom_test_conversion_fun, NULL }
 };
 
 static const char *powercom_shutdownimm_conversion_fun(double value)
@@ -191,7 +214,7 @@ static const char *powercom_shutdownimm_conversion_fun(double value)
 }
 
 static info_lkp_t powercom_shutdownimm_conversion[] = {
-	{ 0, NULL, powercom_shutdownimm_conversion_fun }
+	{ 0, NULL, powercom_shutdownimm_conversion_fun, NULL }
 };
 
 static const char *powercom_online_conversion_fun(double value)
@@ -204,7 +227,7 @@ static const char *powercom_online_conversion_fun(double value)
 }
 
 static info_lkp_t powercom_online_conversion[] = {
-	{ 0, NULL, powercom_online_conversion_fun }
+	{ 0, NULL, powercom_online_conversion_fun, NULL }
 };
 
 static const char *powercom_lowbatt_conversion_fun(double value)
@@ -217,7 +240,7 @@ static const char *powercom_lowbatt_conversion_fun(double value)
 }
 
 static info_lkp_t powercom_lowbatt_conversion[] = {
-	{ 0, NULL, powercom_lowbatt_conversion_fun }
+	{ 0, NULL, powercom_lowbatt_conversion_fun, NULL }
 };
 
 static const char *powercom_trim_conversion_fun(double value)
@@ -230,7 +253,7 @@ static const char *powercom_trim_conversion_fun(double value)
 }
 
 static info_lkp_t powercom_trim_conversion[] = {
-	{ 0, NULL, powercom_trim_conversion_fun }
+	{ 0, NULL, powercom_trim_conversion_fun, NULL }
 };
 
 static const char *powercom_boost_conversion_fun(double value)
@@ -243,7 +266,7 @@ static const char *powercom_boost_conversion_fun(double value)
 }
 
 static info_lkp_t powercom_boost_conversion[] = {
-	{ 0, NULL, powercom_boost_conversion_fun }
+	{ 0, NULL, powercom_boost_conversion_fun, NULL }
 };
 
 static const char *powercom_overload_conversion_fun(double value)
@@ -256,7 +279,7 @@ static const char *powercom_overload_conversion_fun(double value)
 }
 
 static info_lkp_t powercom_overload_conversion[] = {
-	{ 0, NULL, powercom_overload_conversion_fun }
+	{ 0, NULL, powercom_overload_conversion_fun, NULL }
 };
 
 /* --------------------------------------------------------------- */
@@ -422,6 +445,7 @@ static hid_info_t powercom_hid2nut[] = {
 /*	{ "UPS.DesignCapacity", 0, 0, "PowercomUPS.PowercomDesignCapacity", NULL, "%.0f", 0, NULL }, is always 255 */
 	{ "ups.mfr.date", 0, 0, "PowercomUPS.PowercomManufacturerDate", NULL, "%s", 0, date_conversion },
 	{ "battery.temperature", 0, 0, "PowercomUPS.PowercomBatterySystem.PowercomTemperature", NULL, "%.0f", 0, NULL },
+	{ "battery.temperature", 0, 0, "UPS.Battery.Temperature", NULL, "%.1f", 0, NULL },	
 	{ "battery.charge", 0, 0, "PowercomUPS.PowercomBatterySystem.PowercomVoltage", NULL, "%.0f", 0, NULL },
 /*	{ "UPS.BatterySystem.SpecificationInfo", 0, 0, "PowercomUPS.PowercomBatterySystem.PowercomSpecificationInfo", NULL, "%.0f", 0, NULL }, */
 	{ "input.frequency", 0, 0, "PowercomUPS.PowercomPowerConverter.PowercomInput.PowercomFrequency", NULL, "%.0f", 0, NULL },
@@ -526,4 +550,5 @@ subdriver_t powercom_subdriver = {
 	powercom_format_model,
 	powercom_format_mfr,
 	powercom_format_serial,
+	fix_report_desc,
 };
