@@ -13,21 +13,21 @@
  * (C) 2013 Florian Bruhin <nut@the-compiler.org>
  * (C) 2002 Simon Rozman <simon@rozman.net>
  * (C) 1999  Peter Bieringer <pb@bieringer.de>
- *                              
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *                            
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *    
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- * 
+ *
  * rev 0.7: Alexey Sidorov <alexsid@altlinux.org>
  * - add Powercom's Black Knight Pro model support ( BNT-400/500/600/800/801/1000/1200/1500/2000AP 220-240V )
  *
@@ -74,7 +74,11 @@
  * - Fixed the processing of input/output voltages for KIN models
  *   (https://github.com/networkupstools/nut/issues/187)
  *
- */ 
+ * rev 0.18: Rouben Tchakhmakhtchian
+ * - Added nobt flag to config that skips UPS battery check on startup/init
+ *   (https://github.com/networkupstools/nut/issues/546)
+ *
+ */
 
 #include "main.h"
 #include "serial.h"
@@ -82,7 +86,7 @@
 #include "math.h"
 
 #define DRIVER_NAME		"PowerCom protocol UPS driver"
-#define DRIVER_VERSION	"0.17"
+#define DRIVER_VERSION	"0.19"
 
 /* driver description structure */
 upsdrv_info_t	upsdrv_info = {
@@ -92,7 +96,8 @@ upsdrv_info_t	upsdrv_info = {
 	"Peter Bieringer <pb@bieringer.de>\n" \
 	"Alexey Sidorov <alexsid@altlinux.org>\n" \
 	"Florian Bruhin <nut@the-compiler.org>\n" \
-	"Arnaud Quette <ArnaudQuette@Eaton.com>",
+	"Arnaud Quette <ArnaudQuette@Eaton.com>\n" \
+	"Rouben Tchakhmakhtchian <rouben@rouben.net>",
 	DRV_STABLE,
 	{ NULL }
 };
@@ -140,94 +145,94 @@ static void no_flow_control (void);
  *	  120VoltageFactor, 120VoltageConstant },
  */
 static struct type types[] = {
-        {
-                "Trust",
-                11,
-				{  "dtr0rts1", dtr0rts1 },
-                { { 5U, 0U }, { 7U, 0U }, { 8U, 0U } },
-                { { 0U, 10U }, 'n' },
-                {  0.00020997, 0.00020928 },
-                {  6.1343, -0.3808,  4.3110,  0.1811 },
-                {  5.0000,  0.3268,  -825.00,  4.5639, -835.82 },
-                {  1.9216, -0.0977,  0.9545,  0.0000 },
-        },
-        {
-                "Egys",
-                16,
-				{  "no_flow_control", no_flow_control },
-                { { 5U, 0x80U }, { 7U, 0U }, { 8U, 0U } },
-                { { 0U, 10U }, 'n' },
-                {  0.00020997, 0.00020928 },
-                {  6.1343, -0.3808,  1.3333,  0.6667 },
-                {  5.0000,  0.3268,  -825.00,  2.2105, -355.37 },
-                {  1.9216, -0.0977,  0.9545,  0.0000 },
-        },
-        {
-                "KP625AP",
-                16,
-				{  "dtr0rts1", dtr0rts1 },
-                { { 5U, 0x80U }, { 7U, 0U }, { 8U, 0U } },
-                { { 0U, 10U }, 'n' },
-                {  0.00020997, 0.00020928 },
-                {  6.1343, -0.3808,  4.3110,  0.1811 },
-                {  5.0000,  0.3268,  -825.00,  4.5639, -835.82 },
-                {  1.9216, -0.0977,  0.9545,  0.0000 },
-        },
-        {
-                "IMP",
-                16,
-				{  "no_flow_control", no_flow_control },
-                { { 5U, 0xFFU }, { 7U, 0U }, { 8U, 0U } },
-                { { 1U, 30U }, 'y' },
-                {  0.00020997, 0.00020928 },
-                {  6.1343, -0.3808,  4.3110,  0.1811 },
-                {  5.0000,  0.3268,  -825.00,  4.5639, -835.82 },
-                {  1.9216, -0.0977,  0.9545,  0.0000 },
-        },
-        {
-                "KIN",
-                16,
-				{  "no_flow_control", no_flow_control },
-                { { 11U, 0x4bU }, { 8U, 0U }, { 8U, 0U } },
-                { { 1U, 30U }, 'y' },
-                {  0.00020997, 0.0 },
-                {  6.1343, -0.3808,  1.075,  0.1811 },
-                {  5.0000,  0.3268,  -825.00,  0.46511, 0 },
-                {  1.9216, -0.0977,  0.82857,  0.0000 },
-        },
-        {
-                "BNT",
-                16,
-				{  "no_flow_control", no_flow_control },
-                { { 11U, 0x42U }, { 8U, 0U }, { 8U, 0U } },
-                { { 1U, 30U }, 'y' },
-                {  0.00020803, 0.0 },
-                {  1.4474,     0.0,   0.8594,  0.0 },
-                {  5.0000,  0.3268,  -825.00,  0.46511, 0 },
-                {  1.9216, -0.0977,  0.82857,  0.0000 },
-        },
-        {
-                "BNT-other",
-                16,
-				{  "no_flow_control", no_flow_control },
-                { { 8U, 0U }, { 8U, 0U }, { 8U, 0U } },
-                { { 1U, 30U }, 'y' },
-                {  0.00027778, 0.0000 },
-                {  1.0000,  0.0000,  1.0000,  0.0000 },
-                {  1.0000,  0.0000,  0.0000,  1.0000,  0.0000 },
-                {  2.0000,  0.0000,  2.0000,  0.0000 },
-        },
-        {
-                "OPTI",
-                16,
-                {  "no_flow_control", no_flow_control },
-                { { 5U, 0xFFU }, { 7U, 0U }, { 8U, 0U } },
-                { { 1U, 30U }, 'y' },
-                {  0.0000, 0.0000 },
-                {  1.0000,  0.0000,  1.0000,  0.0000 },
-                {  1.0000,  0.0000,  0.0000,  1.0000,  0.0000 },
-                {  2.0000,  0.0000,  2.0000,  0.0000 },
-        },
+	{
+		"Trust",
+		11,
+		{  "dtr0rts1", dtr0rts1 },
+		{ { 5U, 0U }, { 7U, 0U }, { 8U, 0U } },
+		{ { 0U, 10U }, 'n' },
+		{  0.00020997, 0.00020928 },
+		{  6.1343, -0.3808,  4.3110,  0.1811 },
+		{  5.0000,  0.3268,  -825.00,  4.5639, -835.82 },
+		{  1.9216, -0.0977,  0.9545,  0.0000 },
+	},
+	{
+		"Egys",
+		16,
+		{  "no_flow_control", no_flow_control },
+		{ { 5U, 0x80U }, { 7U, 0U }, { 8U, 0U } },
+		{ { 0U, 10U }, 'n' },
+		{  0.00020997, 0.00020928 },
+		{  6.1343, -0.3808,  1.3333,  0.6667 },
+		{  5.0000,  0.3268,  -825.00,  2.2105, -355.37 },
+		{  1.9216, -0.0977,  0.9545,  0.0000 },
+	},
+	{
+		"KP625AP",
+		16,
+		{  "dtr0rts1", dtr0rts1 },
+		{ { 5U, 0x80U }, { 7U, 0U }, { 8U, 0U } },
+		{ { 0U, 10U }, 'n' },
+		{  0.00020997, 0.00020928 },
+		{  6.1343, -0.3808,  4.3110,  0.1811 },
+		{  5.0000,  0.3268,  -825.00,  4.5639, -835.82 },
+		{  1.9216, -0.0977,  0.9545,  0.0000 },
+	},
+	{
+		"IMP",
+		16,
+		{  "no_flow_control", no_flow_control },
+		{ { 5U, 0xFFU }, { 7U, 0U }, { 8U, 0U } },
+		{ { 1U, 30U }, 'y' },
+		{  0.00020997, 0.00020928 },
+		{  6.1343, -0.3808,  4.3110,  0.1811 },
+		{  5.0000,  0.3268,  -825.00,  4.5639, -835.82 },
+		{  1.9216, -0.0977,  0.9545,  0.0000 },
+	},
+	{
+		"KIN",
+		16,
+		{  "no_flow_control", no_flow_control },
+		{ { 11U, 0x4bU }, { 8U, 0U }, { 8U, 0U } },
+		{ { 1U, 30U }, 'y' },
+		{  0.00020997, 0.0 },
+		{  6.1343, -0.3808,  1.075,  0.1811 },
+		{  5.0000,  0.3268,  -825.00,  0.46511, 0 },
+		{  1.9216, -0.0977,  0.82857,  0.0000 },
+	},
+	{
+		"BNT",
+		16,
+		{  "no_flow_control", no_flow_control },
+		{ { 11U, 0x42U }, { 8U, 0U }, { 8U, 0U } },
+		{ { 1U, 30U }, 'y' },
+		{  0.00020803, 0.0 },
+		{  1.4474,     0.0,   0.8594,  0.0 },
+		{  5.0000,  0.3268,  -825.00,  0.46511, 0 },
+		{  1.9216, -0.0977,  0.82857,  0.0000 },
+	},
+	{
+		"BNT-other",
+		16,
+		{  "no_flow_control", no_flow_control },
+		{ { 8U, 0U }, { 8U, 0U }, { 8U, 0U } },
+		{ { 1U, 30U }, 'y' },
+		{  0.00027778, 0.0000 },
+		{  1.0000,  0.0000,  1.0000,  0.0000 },
+		{  1.0000,  0.0000,  0.0000,  1.0000,  0.0000 },
+		{  2.0000,  0.0000,  2.0000,  0.0000 },
+	},
+	{
+		"OPTI",
+		16,
+		{  "no_flow_control", no_flow_control },
+		{ { 5U, 0xFFU }, { 7U, 0U }, { 8U, 0U } },
+		{ { 1U, 30U }, 'y' },
+		{  0.0000, 0.0000 },
+		{  1.0000,  0.0000,  1.0000,  0.0000 },
+		{  1.0000,  0.0000,  0.0000,  1.0000,  0.0000 },
+		{  2.0000,  0.0000,  2.0000,  0.0000 },
+	},
 };
 
 /* values for sending to UPS */
@@ -235,7 +240,7 @@ enum commands {
 	SEND_DATA    = '\x01',
 	BATTERY_TEST = '\x03',
 	WAKEUP_TIME  = '\x04',
-	RESTART	     = '\xb9',
+	RESTART      = '\xb9',
 	SHUTDOWN     = '\xba',
 	COUNTER      = '\xbc'
 };
@@ -274,56 +279,69 @@ enum status {
 	OFF           = 128U
 };
 
-unsigned int voltages[]={100,110,115,120,0,0,0,200,220,230,240,0,0,0,0,0};
-unsigned int BNTmodels[]={0,400,500,600,800,801,1000,1200,1500,2000,0,0,0,0,0,0};
-unsigned int KINmodels[]={0,425,500,525,625,800,1000,1200,1500,1600,2200,2200,2500,3000,5000,0};
-unsigned int IMPmodels[]={0,425,525,625,825,1025,1200,1500,2000,0,0,0,0,0,0,0};
-unsigned int OPTImodels[]={0,0,0,575,0,0,0,0,0,0,0,0,0,0,0,0};
+static unsigned int voltages[]	= {100,110,115,120,0,0,0,200,220,230,240,0,0,0,0,0};
+static unsigned int BNTmodels[]	= {0,400,500,600,800,801,1000,1200,1500,2000,0,0,0,0,0,0};
+static unsigned int KINmodels[]	= {0,425,500,525,625,800,1000,1200,1500,1600,2200,2200,2500,3000,5000,0};
+static unsigned int IMPmodels[]	= {0,425,525,625,825,1025,1200,1500,2000,0,0,0,0,0,0,0};
+static unsigned int OPTImodels[]	= {0,0,0,575,0,0,0,0,0,0,0,0,0,0,0,0};
 
 /*
  * local used functions
  */
 
+static void shutdown_halt(void)
+	__attribute__((noreturn));
+
 static void shutdown_halt(void)
 {
-	ser_send_char (upsfd, SHUTDOWN);
-	if (types[type].shutdown_arguments.minutesShouldBeUsed != 'n') 
+	ser_send_char (upsfd, (unsigned char)SHUTDOWN);
+	if (types[type].shutdown_arguments.minutesShouldBeUsed != 'n')
 		ser_send_char (upsfd, types[type].shutdown_arguments.delay[0]);
 	ser_send_char (upsfd, types[type].shutdown_arguments.delay[1]);
 	upslogx(LOG_INFO, "Shutdown (stayoff) initiated.");
 	exit (0);
 }
 
+static void shutdown_ret(void)
+	__attribute__((noreturn));
+
 static void shutdown_ret(void)
 {
-	ser_send_char (upsfd, RESTART);
-	ser_send_char (upsfd, COUNTER);
-	if (types[type].shutdown_arguments.minutesShouldBeUsed != 'n') 
+	ser_send_char (upsfd, (unsigned char)RESTART);
+	ser_send_char (upsfd, (unsigned char)COUNTER);
+	if (types[type].shutdown_arguments.minutesShouldBeUsed != 'n')
 		ser_send_char (upsfd, types[type].shutdown_arguments.delay[0]);
 	ser_send_char (upsfd, types[type].shutdown_arguments.delay[1]);
 	upslogx(LOG_INFO, "Shutdown (return) initiated.");
-			
+
 	exit (0);
 }
 
 /* registered instant commands */
 static int instcmd (const char *cmdname, const char *extra)
 {
-	if (!strcasecmp(cmdname, "test.battery.start")) { 
+	if (!strcasecmp(cmdname, "test.battery.start")) {
 	    ser_send_char (upsfd, BATTERY_TEST);
 	    return STAT_INSTCMD_HANDLED;
-    }
-	if (!strcasecmp(cmdname, "shutdown.return")) { 
+	}
+	if (!strcasecmp(cmdname, "shutdown.return")) {
+		/* NOTE: In this context, "return" is UPS behavior after the
+		 * wall-power gets restored. The routine exits the driver anyway.
+		 */
 		shutdown_ret();
+#ifndef HAVE___ATTRIBUTE__NORETURN
 		return STAT_INSTCMD_HANDLED;
+#endif
 	}
 	if (!strcasecmp(cmdname, "shutdown.stayoff")) {
-		shutdown_halt(); 
+		shutdown_halt();
+#ifndef HAVE___ATTRIBUTE__NORETURN
 		return STAT_INSTCMD_HANDLED;
+#endif
 	}
 
-    upslogx(LOG_NOTICE, "instcmd: unknown command [%s]", cmdname);
-    return STAT_INSTCMD_UNKNOWN;
+	upslogx(LOG_NOTICE, "instcmd: unknown command [%s] [%s]", cmdname, extra);
+	return STAT_INSTCMD_UNKNOWN;
 }
 
 /* set DTR and RTS lines on a serial port to supply a passive
@@ -331,8 +349,8 @@ static int instcmd (const char *cmdname, const char *extra)
  */
 static void dtr0rts1 (void)
 {
-	ser_set_dtr(upsfd, 0); 
-	ser_set_rts(upsfd, 1); 
+	ser_set_dtr(upsfd, 0);
+	ser_set_rts(upsfd, 1);
 	upsdebugx(2, "DTR => 0, RTS => 1");
 }
 
@@ -340,13 +358,13 @@ static void dtr0rts1 (void)
 static void no_flow_control (void)
 {
 	struct termios tio;
-	
+
 	tcgetattr (upsfd, &tio);
-	
-	tio.c_iflag &= ~ (IXON | IXOFF);
+
+	tio.c_iflag &= ~ ((tcflag_t)IXON | (tcflag_t)IXOFF);
 	tio.c_cc[VSTART] = _POSIX_VDISABLE;
 	tio.c_cc[VSTOP] = _POSIX_VDISABLE;
-				
+
 	upsdebugx(2, "Flow control disable");
 
 	/* disable any flow control */
@@ -356,15 +374,15 @@ static void no_flow_control (void)
 /* sane check for returned buffer */
 static int validate_raw_data (void)
 {
-	int i = 0, 
-    num_of_tests = 
+	int i = 0,
+	num_of_tests =
 		sizeof types[0].validation / sizeof types[0].validation[0];
-	
+
 	for (i = 0;
-		 i < num_of_tests  && 
+		 i < num_of_tests  &&
 		   raw_data[
 		        types[type].validation[i].index_of_byte] ==
-          		       types[type].validation[i].required_value;
+		        types[type].validation[i].required_value;
 		 i++)  ;
 	return (i < num_of_tests) ? 1 : 0;
 }
@@ -372,8 +390,9 @@ static int validate_raw_data (void)
 /* get info from ups */
 static int ups_getinfo(void)
 {
-	int	i, c;
-	
+	size_t	i;
+	ssize_t c;
+
 	/* send trigger char to UPS */
 	if (ser_send_char (upsfd, SEND_DATA) != 1) {
 		upslogx(LOG_NOTICE, "writing error");
@@ -382,33 +401,34 @@ static int ups_getinfo(void)
 	} else {
 		upsdebugx(5, "Num of bytes requested for reading from UPS: %d", types[type].num_of_bytes_from_ups);
 
+		/* Note: num_of_bytes_from_ups is (unsigned char) so comparable
+		 * to ssize_t without more range checks */
 		c = ser_get_buf_len(upsfd, raw_data,
 			types[type].num_of_bytes_from_ups, 3, 0);
-	
-		if (c != types[type].num_of_bytes_from_ups) {
-			upslogx(LOG_NOTICE, "data receiving error (%d instead of %d bytes)", c, types[type].num_of_bytes_from_ups);
+
+		if (c != (ssize_t)types[type].num_of_bytes_from_ups) {
+			upslogx(LOG_NOTICE, "data receiving error (%zd instead of %d bytes)", c, types[type].num_of_bytes_from_ups);
 			dstate_datastale();
 			return 0;
 		} else
-			upsdebugx(5, "Num of bytes received from UPS: %d", c);
-
-	};
+			upsdebugx(5, "Num of bytes received from UPS: %zd", c);
+	}
 
 	/* optional dump of raw data */
 	if (nut_debug_level > 4) {
 		/* FIXME: use upsdebug_hex() ? */
 		printf("Raw data from UPS:\n");
 		for (i = 0; i < types[type].num_of_bytes_from_ups; i++) {
-	 		printf("%2d 0x%02x (%c)\n", i, raw_data[i], raw_data[i]>=0x20 ? raw_data[i] : ' ');
-		};
-	};
+			printf("%2zu 0x%02x (%c)\n", i, raw_data[i], raw_data[i]>=0x20 ? raw_data[i] : ' ');
+		}
+	}
 
 	/* validate raw data for correctness */
 	if (validate_raw_data() != 0) {
 		upslogx(LOG_NOTICE, "data receiving error (validation check)");
 		dstate_datastale();
 		return 0;
-	};
+	}
 	return 1;
 }
 
@@ -416,7 +436,7 @@ static float input_voltage(void)
 {
 	unsigned int model;
 	float tmp=0.0;
-			
+
 	if ( !strcmp(types[type].name, "BNT") && raw_data[MODELNUMBER]%16 > 7 ) {
 		tmp=2.2*raw_data[INPUT_VOLTAGE]-24;
 	} else if ( !strcmp(types[type].name, "KIN")) {
@@ -443,7 +463,7 @@ static float input_voltage(void)
 	} else if ( !strcmp(types[type].name, "IMP") || !strcmp(types[type].name, "OPTI")) {
 		tmp=raw_data[INPUT_VOLTAGE]*2.0;
 	} else {
-	    tmp=linevoltage >= 220 ?
+		tmp=linevoltage >= 220 ?
 			types[type].voltage[0] * raw_data[INPUT_VOLTAGE] + types[type].voltage[1] :
 			types[type].voltage[2] * raw_data[INPUT_VOLTAGE] + types[type].voltage[3];
 	}
@@ -489,16 +509,16 @@ static float output_voltage(void)
 		model=KINmodels[raw_data[MODELNUMBER]/16];
 		if (statINV == 0) {
 			if (statAVR == 0) {
-				// FIXME: miss test "if (iUPS == 1) {"
+				/* FIXME: miss test "if (iUPS == 1) {" */
 				if (linevoltage >= 200) {
-					if (linevoltage <= 625)
+					if (model <= 625)
 						tmp = 1.79*raw_data[OUTPUT_VOLTAGE] + 3.35;
 					else if (model<2000)
 						tmp = 1.61*raw_data[OUTPUT_VOLTAGE];
 					else
 						tmp = 1.625*raw_data[OUTPUT_VOLTAGE];
 				} else {
-					if (linevoltage <= 625)
+					if (model <= 625)
 						tmp = 0.89 * raw_data[OUTPUT_VOLTAGE] + 6.18;
 					else if (model<2000)
 						tmp = 1.61 * raw_data[OUTPUT_VOLTAGE] / 2.0;
@@ -507,7 +527,7 @@ static float output_voltage(void)
 				}
 			}
 			else if (statAVR == 1) {
-				// FIXME: miss test "if ((iUPS == 1) || (iUPS == 13)) {"
+				/* FIXME: miss test "if ((iUPS == 1) || (iUPS == 13)) {" */
 				if (linevoltage >= 200) {
 					if (model <= 525)
 						tmp = 2.07 * raw_data[OUTPUT_VOLTAGE];
@@ -526,7 +546,7 @@ static float output_voltage(void)
 						tmp = 1.875 * raw_data[OUTPUT_VOLTAGE] / 2.0;
 				}
 			} else {
-				// FIXME: miss test "if ((iUPS == 1) || (iUPS == 13)) {"
+				/* FIXME: miss test "if ((iUPS == 1) || (iUPS == 13)) {" */
 				if (linevoltage >= 200) {
 					if (model == 625)
 						tmp = 1.571 * raw_data[OUTPUT_VOLTAGE];
@@ -544,7 +564,7 @@ static float output_voltage(void)
 				}
 			}
 		} else {
-			// FIXME: miss test "if ((iUPS == 1) && (T != 0))"
+			/* FIXME: miss test "if ((iUPS == 1) && (T != 0))" */
 			if (linevoltage < 200) {
 				rdatax = datax1[raw_data[MODELNUMBER]/16];
 				rdatay = datay1[raw_data[MODELNUMBER]/16];
@@ -562,7 +582,9 @@ static float output_voltage(void)
 				tmp=(raw_data[BATTERY_CHARGE]*rdatax)*(raw_data[BATTERY_CHARGE]*rdatax)*
 					(t-raw_data[OUTPUT_VOLTAGE])/t;
 				if (tmp>0)
-					tmp=sqrt(tmp)*rdatay*boostdata-raw_data[UPS_LOAD]*rdataz*boostdata;
+					/* Casts below try to avoid potential multiplication overflow */
+					tmp=(float)( (double)sqrt(tmp)*rdatay*boostdata -
+						(double)raw_data[UPS_LOAD]*rdataz*boostdata );
 			} else {
 				tmp=(raw_data[BATTERY_CHARGE]*rdatax-raw_data[UPS_LOAD]*rdataz)*
 					(raw_data[BATTERY_CHARGE]*rdatax-raw_data[UPS_LOAD]*rdataz)*
@@ -570,7 +592,7 @@ static float output_voltage(void)
 				if (tmp>0)
 					tmp=sqrt(tmp)*rdatay;
 			}
-			// FIXME: may miss a last processing with ErrorVal = 5 | 10
+			/* FIXME: may miss a last processing with ErrorVal = 5 | 10 */
 		}
 	} else if ( !strcmp(types[type].name, "IMP") || !strcmp(types[type].name, "OPTI")) {
 		tmp=raw_data[OUTPUT_VOLTAGE]*2.0;
@@ -591,10 +613,10 @@ static float input_freq(void)
 		return 4807.0/raw_data[INPUT_FREQUENCY];
 	else if ( !strcmp(types[type].name, "IMP") || !strcmp(types[type].name, "OPTI"))
 		return raw_data[INPUT_FREQUENCY];
-	return raw_data[INPUT_FREQUENCY] ? 
+	return raw_data[INPUT_FREQUENCY] ?
 		1.0 / (types[type].freq[0] *
-                raw_data[INPUT_FREQUENCY] +
-                        types[type].freq[1]) : 0;
+				raw_data[INPUT_FREQUENCY] +
+						types[type].freq[1]) : 0;
 }
 
 static float output_freq(void)
@@ -603,10 +625,10 @@ static float output_freq(void)
 		return 4807.0/raw_data[OUTPUT_FREQUENCY];
 	else if ( !strcmp(types[type].name, "IMP") || !strcmp(types[type].name, "OPTI"))
 		return raw_data[OUTPUT_FREQUENCY];
-	return raw_data[OUTPUT_FREQUENCY] ? 
+	return raw_data[OUTPUT_FREQUENCY] ?
 		1.0 / (types[type].freq[0] *
-                raw_data[OUTPUT_FREQUENCY] +
-                        types[type].freq[1]) : 0;
+				raw_data[OUTPUT_FREQUENCY] +
+						types[type].freq[1]) : 0;
 }
 
 static float load_level(void)
@@ -629,7 +651,7 @@ static float load_level(void)
 	int load801i[]={1,1,1,1,1,1,1,1,44,42,40};
 	int load1000i[]={1,1,1,1,1,1,1,1,56,54,52};
 	int load1200i[]={1,1,1,1,1,1,1,1,76,74,72};
-	
+
 	if ( !strcmp(types[type].name, "BNT") && raw_data[MODELNUMBER]%16 > 7 ) {
 		statINV=raw_data[STATUS_A] & ONLINE;
 		voltage=raw_data[MODELNUMBER]%16;
@@ -663,18 +685,18 @@ static float load_level(void)
 			if (model==525) return raw_data[UPS_LOAD]*110.0/load525[voltage];
 			if (model==625) return raw_data[UPS_LOAD]*110.0/load625[voltage];
 			if (model<2000) return raw_data[UPS_LOAD]*1.13;
-			if (model>=2000) return raw_data[UPS_LOAD]*110.0/load2k[voltage];
+			return raw_data[UPS_LOAD]*110.0/load2k[voltage];
 		} else {
 			if (model==425) return raw_data[UPS_LOAD]*110.0/load425i[voltage];
 			if (model==525) return raw_data[UPS_LOAD]*110.0/load525i[voltage];
 			if (model==625) return raw_data[UPS_LOAD]*110.0/load625i[voltage];
 			if (model<2000) return raw_data[UPS_LOAD]*1.66;
-			if (model>=2000) return raw_data[UPS_LOAD]*110.0/load2ki[voltage];
+			return raw_data[UPS_LOAD]*110.0/load2ki[voltage];
 		}
 	} else if ( !strcmp(types[type].name, "IMP") || !strcmp(types[type].name, "OPTI")) {
 		return raw_data[UPS_LOAD];
 	}
-	return raw_data[STATUS_A] & MAINS_FAILURE ?
+	return (raw_data[STATUS_A] & MAINS_FAILURE) ?
 		types[type].loadpct[0] * raw_data[UPS_LOAD] +
 									types[type].loadpct[1] :
 		types[type].loadpct[2] * raw_data[UPS_LOAD] +
@@ -683,9 +705,10 @@ static float load_level(void)
 
 static float batt_level(void)
 {
-	int bat0,bat29,bat100,model;
+	int bat0,bat29,bat100;
+	unsigned int model;
 	float battval;
-	
+
 	if ( !strcmp(types[type].name, "BNT") ) {
 		bat0=157;
 		bat29=165;
@@ -693,9 +716,9 @@ static float batt_level(void)
 		battval=(raw_data[UPS_LOAD])/4+raw_data[BATTERY_CHARGE];
 		if (battval<=bat0)
 			return 0.0;
-		if (battval>bat0 && battval<=bat29)
+		if (battval<=bat29)
 			return (battval-bat0)*30.0/(bat29-bat0);
-		if (battval>bat29 && battval<=bat100)
+		if (battval<=bat100)
 			return 30.0+(battval-bat29)*70.0/(bat100-bat29);
 		return 100.0;
 	}
@@ -729,7 +752,7 @@ static float batt_level(void)
 	}
 	if ( !strcmp(types[type].name, "IMP") || !strcmp(types[type].name, "OPTI"))
 		return raw_data[BATTERY_CHARGE];
-	return raw_data[STATUS_A] & ONLINE ? /* Are we on battery power? */
+	return (raw_data[STATUS_A] & ONLINE) ? /* Are we on battery power? */
 		/* Yes */
 		types[type].battpct[0] * raw_data[BATTERY_CHARGE] +
 			types[type].battpct[1] * load_level() + types[type].battpct[2] :
@@ -746,11 +769,11 @@ static float batt_level(void)
 void upsdrv_updateinfo(void)
 {
 	char	val[32];
-	
+
 	if (!ups_getinfo()){
 		return;
 	}
-	
+
 	/* input.frequency */
 	upsdebugx(3, "input.frequency   (raw data): [raw: %u]",
 	                            raw_data[INPUT_FREQUENCY]);
@@ -763,7 +786,7 @@ void upsdrv_updateinfo(void)
 	dstate_setinfo("output.frequency", "%02.2f", output_freq());
 	upsdebugx(2, "output.frequency: %s", dstate_getinfo("output.frequency"));
 
-	/* ups.load */	
+	/* ups.load */
 	upsdebugx(3, "ups.load  (raw data): [raw: %u]",
 	                            raw_data[UPS_LOAD]);
 	dstate_setinfo("ups.load", "%03.1f", load_level());
@@ -775,23 +798,23 @@ void upsdrv_updateinfo(void)
 	dstate_setinfo("battery.charge", "%03.1f", batt_level());
 	upsdebugx(2, "battery.charge: %s", dstate_getinfo("battery.charge"));
 
-	/* input.voltage */	
+	/* input.voltage */
 	upsdebugx(3, "input.voltage (raw data): [raw: %u]",
 	                            raw_data[INPUT_VOLTAGE]);
 	dstate_setinfo("input.voltage", "%03.1f",input_voltage());
 	upsdebugx(2, "input.voltage: %s", dstate_getinfo("input.voltage"));
-	
-	/* output.voltage */	
+
+	/* output.voltage */
 	upsdebugx(3, "output.voltage (raw data): [raw: %u]",
 	                            raw_data[OUTPUT_VOLTAGE]);
 	dstate_setinfo("output.voltage", "%03.1f",output_voltage());
 	upsdebugx(2, "output.voltage: %s", dstate_getinfo("output.voltage"));
 
 	status_init();
-	
+
 	*val = 0;
 	if (!(raw_data[STATUS_A] & MAINS_FAILURE)) {
-		!(raw_data[STATUS_A] & OFF) ? 
+		!(raw_data[STATUS_A] & OFF) ?
 			status_set("OL") : status_set("OFF");
 	} else {
 		status_set("OB");
@@ -800,7 +823,7 @@ void upsdrv_updateinfo(void)
 	if (raw_data[STATUS_A] & LOW_BAT)  status_set("LB");
 
 	if (raw_data[STATUS_A] & AVR_ON) {
-		input_voltage() < linevoltage ? 
+		input_voltage() < linevoltage ?
 			status_set("BOOST") : status_set("TRIM");
 	}
 
@@ -817,6 +840,9 @@ void upsdrv_updateinfo(void)
 }
 
 /* shutdown UPS */
+void upsdrv_shutdown(void)
+	__attribute__((noreturn));
+
 void upsdrv_shutdown(void)
 {
 	/* power down the attached load immediately */
@@ -827,71 +853,72 @@ void upsdrv_shutdown(void)
 /* initialize UPS */
 void upsdrv_initups(void)
 {
-	int tmp,model = 0;
+	int tmp;
+	unsigned int model = 0;
 	unsigned int i;
 	static char buf[20];
 
 	/* check manufacturer name from arguments */
-	if (getval("manufacturer") != NULL) 
+	if (testvar("manufacturer"))
 		manufacturer = getval("manufacturer");
-	
+
 	/* check model name from arguments */
-	if (getval("modelname") != NULL) 
+	if (testvar("modelname"))
 		modelname = getval("modelname");
-	
+
 	/* check serial number from arguments */
-	if (getval("serialnumber") != NULL) 
+	if (testvar("serialnumber"))
 		serialnumber = getval("serialnumber");
-	
+
 	/* get and check type */
-	if (getval("type") != NULL) {
-		for (i = 0; 
+	if (testvar("type")) {
+		for (i = 0;
 			 i < NUM_OF_SUBTYPES  &&  strcmp(types[i].name, getval("type"));
 			 i++) ;
 		if (i >= NUM_OF_SUBTYPES) {
 			printf("Given UPS type '%s' isn't valid!\n", getval("type"));
 			exit (1);
 		}
-		type = i;	
-	};
-	
+		type = i;
+	}
+
 	/* check line voltage from arguments */
-	if (getval("linevoltage") != NULL) {
+	if (testvar("linevoltage")) {
 		tmp = atoi(getval("linevoltage"));
 		if (! ( (tmp >= 200 && tmp <= 240) || (tmp >= 100 && tmp <= 120) ) ) {
 			printf("Given line voltage '%d' is out of range (100-120 or 200-240 V)\n", tmp);
 			exit (1);
-		};
+		}
 		linevoltage = (unsigned int) tmp;
-	};
+	}
 
-	if (getval("numOfBytesFromUPS") != NULL) {
+	if (testvar("numOfBytesFromUPS")) {
 		tmp = atoi(getval("numOfBytesFromUPS"));
 		if (! (tmp > 0 && tmp <= MAX_NUM_OF_BYTES_FROM_UPS) ) {
 			printf("Given numOfBytesFromUPS '%d' is out of range (1 to %d)\n",
-	               tmp, MAX_NUM_OF_BYTES_FROM_UPS);
+			       tmp, MAX_NUM_OF_BYTES_FROM_UPS);
 			exit (1);
-		};
+		}
 		types[type].num_of_bytes_from_ups = (unsigned char) tmp;
 	}
 
-	if (getval("methodOfFlowControl") != NULL) {
-		for (i = 0; 
-			 i < NUM_OF_SUBTYPES  &&  
+	if (testvar("methodOfFlowControl")) {
+		for (i = 0;
+			 i < NUM_OF_SUBTYPES  &&
 					strcmp(types[i].flowControl.name,
 							getval("methodOfFlowControl"));
 			 i++) ;
 		if (i >= NUM_OF_SUBTYPES) {
-			printf("Given methodOfFlowControl '%s' isn't valid!\n", 
+			printf("Given methodOfFlowControl '%s' isn't valid!\n",
 					getval("methodOfFlowControl"));
 			exit (1);
-		};
-		types[type].flowControl = types[i].flowControl;	
+		}
+		types[type].flowControl = types[i].flowControl;
 	}
 
-	if (getval("validationSequence")  &&
-            sscanf(getval("validationSequence"),
-					"{{%u,%x},{%u,%x},{%u,%x}}",
+	if (testvar("validationSequence")  &&
+	    sscanf(getval("validationSequence"),
+			        "{{%u,%x},{%u,%x},{%u,%x}}",
 			                &types[type].validation[0].index_of_byte,
 			                &types[type].validation[0].required_value,
 			                &types[type].validation[1].index_of_byte,
@@ -900,58 +927,60 @@ void upsdrv_initups(void)
 			                &types[type].validation[2].required_value
 			      ) < 6
 	   ) {
-		printf("Given validationSequence '%s' isn't valid!\n", 
+		printf("Given validationSequence '%s' isn't valid!\n",
 								         getval("validationSequence"));
 		exit (1);
 	}
 
-	if (getval("shutdownArguments")  &&
-	    sscanf(getval("shutdownArguments"), "{{%u,%u},%c}",
+	/* NOTE: %hhu is not supported before C99; that would need reading
+	 * arguments into an uint as %u, checking range and casting */
+	if (testvar("shutdownArguments")  &&
+	    sscanf(getval("shutdownArguments"), "{{%hhu,%hhu},%c}",
 	                &types[type].shutdown_arguments.delay[0],
 	                &types[type].shutdown_arguments.delay[1],
-	                &types[type].shutdown_arguments.minutesShouldBeUsed 
+	                &types[type].shutdown_arguments.minutesShouldBeUsed
 	          ) < 3
 	   ) {
-	    printf("Given shutdownArguments '%s' isn't valid!\n", 
+	    printf("Given shutdownArguments '%s' isn't valid!\n",
 								         getval("shutdownArguments"));
 		exit (1);
-	} 
+	}
 
-	if (getval("frequency")  &&
-            sscanf(getval("frequency"), "{%f,%f}",
-			                &types[type].freq[0], &types[type].freq[1]
+	if (testvar("frequency")  &&
+	        sscanf(getval("frequency"), "{%f,%f}",
+	                &types[type].freq[0], &types[type].freq[1]
 	              ) < 2
 	   ) {
-		printf("Given frequency '%s' isn't valid!\n", 
+		printf("Given frequency '%s' isn't valid!\n",
 										getval("frequency"));
 		exit (1);
 	}
 
-	if (getval("loadPercentage")  && 
-            sscanf(getval("loadPercentage"), "{%f,%f,%f,%f}",
+	if (testvar("loadPercentage")  &&
+	        sscanf(getval("loadPercentage"), "{%f,%f,%f,%f}",
 	            &types[type].loadpct[0], &types[type].loadpct[1],
 	            &types[type].loadpct[2], &types[type].loadpct[3]
 	              ) < 4
 	   ) {
-		printf("Given loadPercentage '%s' isn't valid!\n", 
+		printf("Given loadPercentage '%s' isn't valid!\n",
 								         getval("loadPercentage"));
 		exit (1);
 	}
 
-	if (getval("batteryPercentage")  && 
-            sscanf(getval("batteryPercentage"), "{%f,%f,%f,%f,%f}",
+	if (testvar("batteryPercentage")  &&
+	        sscanf(getval("batteryPercentage"), "{%f,%f,%f,%f,%f}",
 	                &types[type].battpct[0], &types[type].battpct[1],
 	                &types[type].battpct[2], &types[type].battpct[3],
 	                &types[type].battpct[4]
 	              ) < 5
 	   ) {
-		printf("Given batteryPercentage '%s' isn't valid!\n", 
+		printf("Given batteryPercentage '%s' isn't valid!\n",
 								         getval("batteryPercentage"));
 		exit (1);
 	}
 
-	if (getval("voltage")  &&
-            sscanf(getval("voltage"), "{%f,%f,%f,%f}",
+	if (testvar("voltage")  &&
+	        sscanf(getval("voltage"), "{%f,%f,%f,%f}",
 	            &types[type].voltage[0], &types[type].voltage[1],
 	            &types[type].voltage[2], &types[type].voltage[3]
 				  ) < 4
@@ -963,7 +992,7 @@ void upsdrv_initups(void)
 	/* open serial port */
 	upsfd = ser_open(device_path);
 	ser_set_speed(upsfd, device_path, B1200);
-	
+
 	/* setup flow control */
 	types[type].flowControl.setup_flow_control();
 
@@ -998,55 +1027,61 @@ void upsdrv_initups(void)
 		}
 		linevoltage=voltages[raw_data[MODELNUMBER]%16];
 		if (!strcmp(types[type].name, "OPTI")) {
-			snprintf(buf,sizeof(buf),"%s-%d",types[type].name,model);
+			snprintf(buf,sizeof(buf),"%s-%u",types[type].name, model);
 		} else {
-			snprintf(buf,sizeof(buf),"%s-%dAP",types[type].name,model);
+			snprintf(buf,sizeof(buf),"%s-%uAP",types[type].name, model);
 		}
 		if (!strcmp(modelname, "Unknown"))
 			modelname=buf;
 		upsdebugx(1,"Detected: %s , %dV",buf,linevoltage);
-		if (ser_send_char (upsfd, BATTERY_TEST) != 1) {
-			upslogx(LOG_NOTICE, "writing error");
-			dstate_datastale();
-			return;
+		if (testvar("nobt") || dstate_getinfo("driver.flag.nobt")) {
+			upslogx(LOG_NOTICE, "nobt flag set, skipping battery test as requested");
+		}
+		else {
+			upslogx(LOG_NOTICE, "nobt flag not set, performing battery test as requested");
+			if (ser_send_char (upsfd, BATTERY_TEST) != 1) {
+				upslogx(LOG_NOTICE, "Write error: failed to send battery test command to UPS!");
+				dstate_datastale();
+				return;
+			}
 		}
 	}
-	
+
 	upsdebugx(1, "Values of arguments:");
 	upsdebugx(1, " manufacturer            : '%s'", manufacturer);
 	upsdebugx(1, " model name              : '%s'", modelname);
 	upsdebugx(1, " serial number           : '%s'", serialnumber);
 	upsdebugx(1, " line voltage            : '%u'", linevoltage);
 	upsdebugx(1, " type                    : '%s'", types[type].name);
-	upsdebugx(1, " number of bytes from UPS: '%u'", 
-	                        types[type].num_of_bytes_from_ups);
-	upsdebugx(1, " method of flow control  : '%s'", 
-    	                    types[type].flowControl.name);
+	upsdebugx(1, " number of bytes from UPS: '%u'",
+	            types[type].num_of_bytes_from_ups);
+	upsdebugx(1, " method of flow control  : '%s'",
+	            types[type].flowControl.name);
 	upsdebugx(1, " validation sequence: '{{%u,%#x},{%u,%#x},{%u,%#x}}'",
-			            types[type].validation[0].index_of_byte,
-			            types[type].validation[0].required_value,
-			            types[type].validation[1].index_of_byte,
-			            types[type].validation[1].required_value,
-			            types[type].validation[2].index_of_byte,
-			            types[type].validation[2].required_value);
+	            types[type].validation[0].index_of_byte,
+	            types[type].validation[0].required_value,
+	            types[type].validation[1].index_of_byte,
+	            types[type].validation[1].required_value,
+	            types[type].validation[2].index_of_byte,
+	            types[type].validation[2].required_value);
 	upsdebugx(1, " shutdown arguments: '{{%u,%u},%c}'",
-	                types[type].shutdown_arguments.delay[0],
-	                types[type].shutdown_arguments.delay[1],
-	                types[type].shutdown_arguments.minutesShouldBeUsed); 
+	            types[type].shutdown_arguments.delay[0],
+	            types[type].shutdown_arguments.delay[1],
+	            types[type].shutdown_arguments.minutesShouldBeUsed);
 	if ( strcmp(types[type].name, "KIN") && strcmp(types[type].name, "BNT") && strcmp(types[type].name, "IMP")) {
 		upsdebugx(1, " frequency calculation coefficients: '{%f,%f}'",
-								types[type].freq[0], types[type].freq[1]);
+		        types[type].freq[0], types[type].freq[1]);
 		upsdebugx(1, " load percentage calculation coefficients: "
-					"'{%f,%f,%f,%f}'",
-						types[type].loadpct[0], types[type].loadpct[1],
-						types[type].loadpct[2], types[type].loadpct[3]);
-		upsdebugx(1, " battery percentage calculation coefficients: " 
-					"'{%f,%f,%f,%f,%f}'",
-						types[type].battpct[0], types[type].battpct[1],
-						types[type].battpct[2], types[type].battpct[3],
-						types[type].battpct[4]);
+		        "'{%f,%f,%f,%f}'",
+		        types[type].loadpct[0], types[type].loadpct[1],
+		        types[type].loadpct[2], types[type].loadpct[3]);
+		upsdebugx(1, " battery percentage calculation coefficients: "
+		        "'{%f,%f,%f,%f,%f}'",
+		        types[type].battpct[0], types[type].battpct[1],
+		        types[type].battpct[2], types[type].battpct[3],
+		        types[type].battpct[4]);
 		upsdebugx(1, " voltage calculation coefficients: '{%f,%f}'",
-							types[type].voltage[2], types[type].voltage[3]);
+		        types[type].voltage[2], types[type].voltage[3]);
 	}
 
 }
@@ -1054,8 +1089,8 @@ void upsdrv_initups(void)
 /* display help */
 void upsdrv_help(void)
 {
-	//               1         2         3         4         5         6         7         8
-	//      12345678901234567890123456789012345678901234567890123456789012345678901234567890 MAX
+	/*               1         2         3         4         5         6         7         8 */
+	/*      12345678901234567890123456789012345678901234567890123456789012345678901234567890 MAX */
 	printf("\n");
 	printf("Specify UPS information in the ups.conf file.\n");
 	printf(" type:          Type of UPS: 'Trust','Egys','KP625AP','IMP','KIN','BNT',\n");
@@ -1093,7 +1128,8 @@ void upsdrv_help(void)
 	printf(" voltage:       Voltage conversion values for 240 and 120 voltage:\n");
 	printf("                 {240A,240B,120A,120B}\n");
 	printf("                 used in function: A*x+B\n");
-	printf("                If the raw value x IS HALF the Voltage, then A=2, B=0\n\n");
+	printf("                If the raw value x IS HALF the Voltage, then A=2, B=0\n");
+	printf(" nobt:          Flag to skip battery check on init/startup.\n\n");
 
 	printf("Example for BNT1500AP in ups.conf:\n");
 	printf("[BNT1500AP]\n");
@@ -1113,6 +1149,7 @@ void upsdrv_help(void)
 	printf("#   loadPercentage = {1.0000,0.0,1.0000,0.0}\n");
 	printf("#   batteryPercentage = {1.0000,0.0000,0.0000,1.0000,0.0000}\n");
 	printf("#   voltage = {2.0000,0.0000,2.0000,0.0000}\n");
+	printf("    nobt\n");
 	return;
 }
 
@@ -1127,17 +1164,17 @@ void upsdrv_initinfo(void)
 	dstate_setinfo ("input.voltage.nominal", "%u", linevoltage);
 
 	/* now add the instant commands */
-    dstate_addcmd ("test.battery.start");
-   	dstate_addcmd ("shutdown.return");
-   	dstate_addcmd ("shutdown.stayoff");
+	dstate_addcmd ("test.battery.start");
+	dstate_addcmd ("shutdown.return");
+	dstate_addcmd ("shutdown.stayoff");
 	upsh.instcmd = instcmd;
 }
 
 /* define possible arguments */
 void upsdrv_makevartable(void)
 {
-		//        1         2         3         4         5         6         7         8
-		//2345678901234567890123456789012345678901234567890123456789012345678901234567890 MAX
+		/*        1         2         3         4         5         6         7         8 */
+		/*2345678901234567890123456789012345678901234567890123456789012345678901234567890 MAX */
 	addvar(VAR_VALUE, "type",
 		"Type of UPS: 'Trust','Egys','KP625AP','IMP','KIN','BNT','BNT-other','OPTI'\n"
 		" (default: 'Trust')");
@@ -1166,6 +1203,8 @@ void upsdrv_makevartable(void)
 			"Battery conversion values: OffFactor, LoadFactor, OffConst, OnFactor, OnConst");
 		addvar(VAR_VALUE, "voltage",
 			"Voltage conversion values: 240VFactor, 240VConst, 120VFactor, 120VConst");
+		addvar(VAR_FLAG, "nobt",
+			"Disable battery test at driver init/startup");
 	}
 }
 
