@@ -21,15 +21,18 @@
  *
  */
 
+#ifndef NUT_POWERCOM_H_SEEN
+#define NUT_POWERCOM_H_SEEN 1
+
 /* C-libary includes */
 #include <sys/file.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
-#include "serial.h"
-#include <limits.h>
 
 /* nut includes */
+#include "serial.h"
+#include "nut_stdint.h"
 #include "timehead.h"
 
 
@@ -37,51 +40,51 @@
 struct type {
 	const char    *name;
 	unsigned char num_of_bytes_from_ups;
-	
+
 	struct method_of_flow_control {
-	    const char *name;
-	    void (*setup_flow_control)(void);
+		const char *name;
+		void (*setup_flow_control)(void);
 	} flowControl;
-	
+
 	struct validation_byte {
-	    unsigned int index_of_byte, required_value;
-	    /* An example might explain the intention better then prose.
-	     * Suppose we want to validate the data with:
-	     *     powercom_raw_data[5] == 0x80
-	     * then we will set index_of_byte to 5U and required_value to
-	     * 0x80U: { 5U, 0x80U }.
-	     */
+		unsigned int index_of_byte, required_value;
+		/* An example might explain the intention better then prose.
+		 * Suppose we want to validate the data with:
+		 *     powercom_raw_data[5] == 0x80
+		 * then we will set index_of_byte to 5U and required_value to
+		 * 0x80U: { 5U, 0x80U }.
+		 */
 	} validation[3];
-	/* The validation array is of length 3 because 3 is longest 
+	/* The validation array is of length 3 because 3 is longest
 	 * validation sequence for any type.
 	 */
-	
-	/* Some UPSs must have a minutes and a seconds arguments for 
+
+	/* Some UPSs must have a minutes and a seconds arguments for
 	 * the COUNTER commands while others are known to work with the
 	 * seconds argument alone.
 	 */
 	struct delay_for_power_kill {
-	    unsigned int  delay[2];   /* { minutes, seconds } */
-	    unsigned char minutesShouldBeUsed;
-	    /* 'n' in case the minutes value, which is delay[0], should
-		 * be skipped and not sent to the UPS. 
+		unsigned char delay[2];   /* { minutes, seconds } */
+		unsigned char minutesShouldBeUsed;
+		/* 'n' in case the minutes value, which is delay[0], should
+		 * be skipped and not sent to the UPS.
 		 */
 	} shutdown_arguments;
-	
+
 	/* parameters to calculate input and output freq., one pair used for
 	 * both input and output functions:
 	 *  The pair [0],[1] defines parameters for 1/(A*x+B) to calculate freq.
 	 *  from raw data 'x'.
 	 */
 	float         freq[2];
-	
+
 	/* parameters to calculate load %, two pairs for each type:
 	 *  First pair [0],[1] defines the parameters for A*x+B to calculate load
 	 *  from raw data when offline and the second pair [2],[3] is used when
 	 *  online
 	 */
 	float         loadpct[4];
-	
+
 	/* parameters to calculate battery %, five parameters for each type:
 	 *  First three params [0],[1],[2] defines the parameters for A*x+B*y+C to calculate
 	 *  battery % (x is raw data, y is load %) when offline.
@@ -97,3 +100,5 @@ struct type {
 	 */
 	float         voltage[4];
 };
+
+#endif	/* NUT_POWERCOM_H_SEEN */
