@@ -20,10 +20,10 @@
 
 #include "main.h"
 
-#include <libpowerman.h>
+#include <libpowerman.h>	/* pm_err_t and other beasts */
 
 #define DRIVER_NAME	"Powerman PDU client driver"
-#define DRIVER_VERSION	"0.11"
+#define DRIVER_VERSION	"0.12"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -35,11 +35,11 @@ upsdrv_info_t upsdrv_info = {
 };
 
 /* Powerman functions and variables */
-static pm_err_t query_one(pm_handle_t pm, char *s, int mode);
-static pm_err_t query_all(pm_handle_t pm, int mode);
+static pm_err_t query_one(pm_handle_t arg_pm, char *s, int mode);
+static pm_err_t query_all(pm_handle_t arg_pm, int mode);
 
-pm_handle_t pm;
-char ebuf[64];
+static pm_handle_t pm;
+static char ebuf[64];
 
 /* modes to snmp_ups_walk. */
 #define WALKMODE_INIT	0
@@ -49,7 +49,7 @@ static int reconnect_ups(void);
 
 static int instcmd(const char *cmdname, const char *extra)
 {
-	pm_err_t rv = -1;
+	pm_err_t rv = PM_EBADARG;
 	char *cmdsuffix = NULL;
 	char *cmdindex = NULL;
 	char outletname[SMALLBUF];
@@ -90,7 +90,7 @@ static int instcmd(const char *cmdname, const char *extra)
 		return (rv==PM_ESUCCESS)?STAT_INSTCMD_HANDLED:STAT_SET_INVALID;
 	}
 
-	upslogx(LOG_NOTICE, "instcmd: unknown command [%s]", cmdname);
+	upslogx(LOG_NOTICE, "instcmd: unknown command [%s] [%s]", cmdname, extra);
 	return STAT_INSTCMD_UNKNOWN;
 }
 
@@ -133,6 +133,9 @@ void upsdrv_initinfo(void)
 	/* FIXME: no need for setvar (ex for outlet.n.delay.*)!? */
 }
 
+void upsdrv_shutdown(void)
+	__attribute__((noreturn));
+
 void upsdrv_shutdown(void)
 {
 	/* FIXME: shutdown all outlets? */
@@ -208,7 +211,7 @@ static int reconnect_ups(void)
  * powerman support functions
  ****************************/
 
-static pm_err_t query_one(pm_handle_t pm, char *s, int outletnum)
+static pm_err_t query_one(pm_handle_t arg_pm, char *s, int outletnum)
 {
 	pm_err_t rv;
 	pm_node_state_t ns;
@@ -216,7 +219,7 @@ static pm_err_t query_one(pm_handle_t pm, char *s, int outletnum)
 
 	upsdebugx(1, "entering query_one (%s)", s);
 
-	rv = pm_node_status(pm, s, &ns);
+	rv = pm_node_status(arg_pm, s, &ns);
 	if (rv == PM_ESUCCESS) {
 
 		upsdebugx(3, "updating status");
@@ -229,7 +232,7 @@ static pm_err_t query_one(pm_handle_t pm, char *s, int outletnum)
 	return rv;
 }
 
-static pm_err_t query_all(pm_handle_t pm, int mode)
+static pm_err_t query_all(pm_handle_t arg_pm, int mode)
 {
 	pm_err_t rv;
 	pm_node_iterator_t itr;
@@ -239,7 +242,7 @@ static pm_err_t query_all(pm_handle_t pm, int mode)
 
 	upsdebugx(1, "entering query_all ()");
 
-	rv = pm_node_iterator_create(pm, &itr);
+	rv = pm_node_iterator_create(arg_pm, &itr);
 	if (rv != PM_ESUCCESS)
 		return rv;
 
@@ -247,7 +250,7 @@ static pm_err_t query_all(pm_handle_t pm, int mode)
 
 		/* in WALKMODE_UPDATE, we always call this one for the
 		 * status update... */
-		if ((rv = query_one(pm, s, outletnum)) != PM_ESUCCESS)
+		if ((rv = query_one(arg_pm, s, outletnum)) != PM_ESUCCESS)
 			break;
 		else  {
 			/* set the initial generic properties (ie except status)
