@@ -11,6 +11,10 @@
 #  include <termios.h>
 #endif /* HAVE_SYS_TERMIOS_H */
 
+#include <unistd.h>             /* for usleep() and useconds_t, latter also might be via <sys/types.h> */
+#include <sys/types.h>
+#include <sys/select.h>         /* for suseconds_t */
+
 /* limit the amount of spew that goes in the syslog when we lose the UPS */
 #define SER_ERR_LIMIT 10	/* start limiting after 10 in a row  */
 #define SER_ERR_RATE 100	/* then only print every 100th error */
@@ -34,41 +38,41 @@ int ser_flush_io(int fd);
 
 int ser_close(int fd, const char *port);
 
-int ser_send_char(int fd, unsigned char ch);
+ssize_t ser_send_char(int fd, unsigned char ch);
 
 /* send the results of the format string with d_usec delay after each char */
-int ser_send_pace(int fd, unsigned long d_usec, const char *fmt, ...)
+ssize_t ser_send_pace(int fd, useconds_t d_usec, const char *fmt, ...)
 	__attribute__ ((__format__ (__printf__, 3, 4)));
 
 /* send the results of the format string with no delay */
-int ser_send(int fd, const char *fmt, ...)
+ssize_t ser_send(int fd, const char *fmt, ...)
 	__attribute__ ((__format__ (__printf__, 2, 3)));
 
 /* send buflen bytes from buf with no delay */
-int ser_send_buf(int fd, const void *buf, size_t buflen);
+ssize_t ser_send_buf(int fd, const void *buf, size_t buflen);
 
 /* send buflen bytes from buf with d_usec delay after each char */
-int ser_send_buf_pace(int fd, unsigned long d_usec, const void *buf, 
+ssize_t ser_send_buf_pace(int fd, useconds_t d_usec, const void *buf,
 	size_t buflen);
 
-int ser_get_char(int fd, void *ch, long d_sec, long d_usec);
+ssize_t ser_get_char(int fd, void *ch, time_t d_sec, useconds_t d_usec);
 
-int ser_get_buf(int fd, void *buf, size_t buflen, long d_sec, long d_usec);
+ssize_t ser_get_buf(int fd, void *buf, size_t buflen, time_t d_sec, useconds_t d_usec);
 
 /* keep reading until buflen bytes are received or a timeout occurs */
-int ser_get_buf_len(int fd, void *buf, size_t buflen, long d_sec, long d_usec);
+ssize_t ser_get_buf_len(int fd, void *buf, size_t buflen, time_t d_sec, useconds_t d_usec);
 
 /* reads a line up to <endchar>, discarding anything else that may follow,
    with callouts to the handler if anything matches the alertset */
-int ser_get_line_alert(int fd, void *buf, size_t buflen, char endchar,
+ssize_t ser_get_line_alert(int fd, void *buf, size_t buflen, char endchar,
 	const char *ignset, const char *alertset, void handler (char ch), 
-	long d_sec, long d_usec);
+	time_t d_sec, useconds_t d_usec);
 
 /* as above, only with no alertset handling (just a wrapper) */
-int ser_get_line(int fd, void *buf, size_t buflen, char endchar,
-	const char *ignset, long d_sec, long d_usec);
+ssize_t ser_get_line(int fd, void *buf, size_t buflen, char endchar,
+	const char *ignset, time_t d_sec, useconds_t d_usec);
 
-int ser_flush_in(int fd, const char *ignset, int verbose);
+ssize_t ser_flush_in(int fd, const char *ignset, int verbose);
 
 /* unified failure reporting: call these often */
 void ser_comm_fail(const char *fmt, ...)
