@@ -1,18 +1,20 @@
 /*
 	anything commented is optional
 	anything else is mandatory
-	 
+
 	for more information, refer to:
 	* docs/developers.txt
 	* docs/new-drivers.txt
 	* docs/new-names.txt
-	
+
 	and possibly also to:
 	* docs/hid-subdrivers.txt for USB/HID devices
 	* or docs/snmp-subdrivers.txt for SNMP devices
 */
 
+#include "config.h"
 #include "main.h"
+#include "attribute.h"
 
 /* #include "serial.h" */
 
@@ -20,7 +22,7 @@
 /* #define IGNCHARS	""	*/
 
 #define DRIVER_NAME	"Skeleton UPS driver"
-#define DRIVER_VERSION	"0.02"
+#define DRIVER_VERSION	"0.03"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -55,7 +57,7 @@ void upsdrv_updateinfo(void)
 	/* ser_send(upsfd, "foo%d", 1234); */
 	/* ser_send_buf(upsfd, bincmd, 12); */
 
-	/* 
+	/*
 	 * ret = ser_get_line(upsfd, temp, sizeof(temp), ENDCHAR, IGNCHARS);
 	 *
 	 * if (ret < STATUS_LEN) {
@@ -92,6 +94,9 @@ void upsdrv_updateinfo(void)
 	 */
 }
 
+void upsdrv_shutdown(void)
+	__attribute__((noreturn));
+
 void upsdrv_shutdown(void)
 {
 	/* tell the UPS to shut down, then return - DO NOT SLEEP HERE */
