@@ -29,7 +29,7 @@
 #include "tripplite-hid.h"
 #include "usb-common.h"
 
-#define TRIPPLITE_HID_VERSION "TrippLite HID 0.82"
+#define TRIPPLITE_HID_VERSION "TrippLite HID 0.84"
 /* FIXME: experimental flag to be put in upsdrv_info */
 
 
@@ -47,17 +47,20 @@ static double   io_current_scale = 1.0;
 /* Specific handlers for USB device matching */
 static void *battery_scale_1dot0(USBDevice_t *device)
 {
+	NUT_UNUSED_VARIABLE(device);
 	/* FIXME: we could remove this one since it's the default! */
 	battery_scale = 1.0;
 	return NULL;
 }
 static void *battery_scale_0dot1(USBDevice_t *device)
 {
+	NUT_UNUSED_VARIABLE(device);
 	battery_scale = 0.1;
 	return NULL;
 }
 static void *smart1500lcdt_scale(USBDevice_t *device)
 {
+	NUT_UNUSED_VARIABLE(device);
 	battery_scale = 100000.0;
 	io_voltage_scale = 100000.0;
 	io_frequency_scale = 0.01;
@@ -71,6 +74,9 @@ static void *smart1500lcdt_scale(USBDevice_t *device)
 /* Hewlett Packard */
 #define HP_VENDORID 0x03f0
 
+/* Delta/Minuteman */
+#define DELTA_VENDORID 0x05dd
+
 /* USB IDs device table */
 static usb_device_id_t tripplite_usb_device_table[] = {
 	/* e.g. TrippLite AVR550U */
@@ -81,6 +87,8 @@ static usb_device_id_t tripplite_usb_device_table[] = {
 	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x1008), battery_scale_0dot1 },
 	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x1009), battery_scale_0dot1 },
 	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x1010), battery_scale_0dot1 },
+	/* e.g. TrippLite SU3000LCD2UHV */
+	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x1330), battery_scale_1dot0 },
 	/* e.g. TrippLite OMNI1000LCD */
 	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x2005), battery_scale_0dot1 },
 	/* e.g. TrippLite OMNI900LCD */
@@ -109,6 +117,8 @@ static usb_device_id_t tripplite_usb_device_table[] = {
 	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x3015), battery_scale_1dot0 },
 	/* e.g. TrippLite Smart1500LCD (newer unit) */
 	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x3016), smart1500lcdt_scale },
+	/* e.g. TrippLite AVR750U (newer unit) */
+	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x3024), smart1500lcdt_scale },
 	/* e.g. TrippLite SmartOnline SU1500RTXL2UA (older unit?) */
 	{ USB_DEVICE(TRIPPLITE_VENDORID, 0x4001), battery_scale_1dot0 },
 	/* e.g. TrippLite SmartOnline SU6000RT4U? */
@@ -142,8 +152,11 @@ static usb_device_id_t tripplite_usb_device_table[] = {
 	/* HP R/T 2200 INTL (like SMART2200RMXL2U) */
 	{ USB_DEVICE(HP_VENDORID, 0x1f0a), battery_scale_1dot0 },
 
+	/* Delta/Minuteman Enterprise Plus E1500RM2U */
+	{ USB_DEVICE(DELTA_VENDORID, 0xa011), battery_scale_1dot0 },
+
 	/* Terminating entry */
-	{ -1, -1, NULL }
+	{ 0, 0, NULL }
 };
 
 /* returns statically allocated string - must not use it again before
@@ -169,7 +182,7 @@ static const char *tripplite_chemistry_fun(double value)
 }
 
 static info_lkp_t tripplite_chemistry[] = {
-	{ 0, NULL, tripplite_chemistry_fun }
+	{ 0, NULL, tripplite_chemistry_fun, NULL }
 };
 
 /* returns statically allocated string - must not use it again before
@@ -184,7 +197,7 @@ static const char *tripplite_battvolt_fun(double value)
 }
 
 static info_lkp_t tripplite_battvolt[] = {
-	{ 0, NULL, tripplite_battvolt_fun }
+	{ 0, NULL, tripplite_battvolt_fun, NULL }
 };
 
 static const char *tripplite_iovolt_fun(double value)
@@ -197,7 +210,7 @@ static const char *tripplite_iovolt_fun(double value)
 }
 
 static info_lkp_t tripplite_iovolt[] = {
-	{ 0, NULL, tripplite_iovolt_fun }
+	{ 0, NULL, tripplite_iovolt_fun, NULL }
 };
 
 static const char *tripplite_iofreq_fun(double value)
@@ -210,7 +223,7 @@ static const char *tripplite_iofreq_fun(double value)
 }
 
 static info_lkp_t tripplite_iofreq[] = {
-	{ 0, NULL, tripplite_iofreq_fun }
+	{ 0, NULL, tripplite_iofreq_fun, NULL }
 };
 
 static const char *tripplite_ioamp_fun(double value)
@@ -223,7 +236,7 @@ static const char *tripplite_ioamp_fun(double value)
 }
 
 static info_lkp_t tripplite_ioamp[] = {
-	{ 0, NULL, tripplite_ioamp_fun }
+	{ 0, NULL, tripplite_ioamp_fun, NULL }
 };
 
 /* --------------------------------------------------------------- */
@@ -549,4 +562,5 @@ subdriver_t tripplite_subdriver = {
 	tripplite_format_model,
 	tripplite_format_mfr,
 	tripplite_format_serial,
+	fix_report_desc,
 };
