@@ -25,8 +25,10 @@ int is_usb_device_supported(usb_device_id_t *usb_device_id_list, USBDevice_t *de
 	int retval = NOT_SUPPORTED;
 	usb_device_id_t *usbdev;
 
-	for (usbdev = usb_device_id_list; usbdev->vendorID != -1; usbdev++) {
-
+	for (usbdev = usb_device_id_list;
+	     (usbdev->vendorID != 0 || usbdev->productID != 0 || usbdev->fun != NULL);
+	     usbdev++
+	) {
 		if (usbdev->vendorID != device->VendorID) {
 			continue;
 		}
@@ -78,28 +80,49 @@ static int strcmp_null(char *s1, char *s2)
 static int match_function_exact(USBDevice_t *hd, void *privdata)
 {
 	USBDevice_t	*data = (USBDevice_t *)privdata;
-	
+
+	upsdebugx(3, "%s: matching a device...", __func__);
+
 	if (hd->VendorID != data->VendorID) {
+		upsdebugx(2, "%s: failed match of %s: %4x != %4x",
+		    __func__, "VendorID", hd->VendorID, data->VendorID);
 		return 0;
 	}
 
 	if (hd->ProductID != data->ProductID) {
+		upsdebugx(2, "%s: failed match of %s: %4x != %4x",
+		    __func__, "ProductID", hd->ProductID, data->ProductID);
 		return 0;
 	}
 
 	if (strcmp_null(hd->Vendor, data->Vendor) != 0) {
+		upsdebugx(2, "%s: failed match of %s: %s != %s",
+		    __func__, "Vendor", hd->Vendor, data->Vendor);
 		return 0;
 	}
 
 	if (strcmp_null(hd->Product, data->Product) != 0) {
+		upsdebugx(2, "%s: failed match of %s: %s != %s",
+		    __func__, "Product", hd->Product, data->Product);
 		return 0;
 	}
 
 	if (strcmp_null(hd->Serial, data->Serial) != 0) {
+		upsdebugx(2, "%s: failed match of %s: %s != %s",
+		    __func__, "Serial", hd->Serial, data->Serial);
 		return 0;
 	}
 #ifdef DEBUG_EXACT_MATCH_BUS
 	if (strcmp_null(hd->Bus, data->Bus) != 0) {
+		upsdebugx(2, "%s: failed match of %s: %s != %s",
+		    __func__, "Bus", hd->Bus, data->Bus);
+		return 0;
+	}
+#endif
+#ifdef DEBUG_EXACT_MATCH_DEVICE
+	if (strcmp_null(hd->Device, data->Device) != 0) {
+		upsdebugx(2, "%s: failed match of %s: %s != %s",
+		    __func__, "Device", hd->Device, data->Device);
 		return 0;
 	}
 #endif
@@ -137,6 +160,9 @@ int USBNewExactMatcher(USBDeviceMatcher_t **matcher, USBDevice_t *hd)
 	data->Serial = hd->Serial ? strdup(hd->Serial) : NULL;
 #ifdef DEBUG_EXACT_MATCH_BUS
 	data->Bus = hd->Bus ? strdup(hd->Bus) : NULL;
+#endif
+#ifdef DEBUG_EXACT_MATCH_DEVICE
+	data->Device = hd->Device ? strdup(hd->Device) : NULL;
 #endif
 	*matcher = m;
 
@@ -159,6 +185,9 @@ void USBFreeExactMatcher(USBDeviceMatcher_t *matcher)
 	free(data->Serial);
 #ifdef DEBUG_EXACT_MATCH_BUS
 	free(data->Bus);
+#endif
+#ifdef DEBUG_EXACT_MATCH_DEVICE
+	free(data->Device);
 #endif
 	free(data);
 	free(matcher);
@@ -271,7 +300,7 @@ static int match_regex_hex(regex_t *preg, int n)
 
 /* private data type: hold a set of compiled regular expressions. */
 typedef struct regex_matcher_data_s {
-	regex_t	*regex[6];
+	regex_t	*regex[7];
 } regex_matcher_data_t;
 
 /* private callback function for regex matches */
@@ -279,48 +308,97 @@ static int match_function_regex(USBDevice_t *hd, void *privdata)
 {
 	regex_matcher_data_t	*data = (regex_matcher_data_t *)privdata;
 	int r;
-	
+
+	upsdebugx(3, "%s: matching a device...", __func__);
+
 	r = match_regex_hex(data->regex[0], hd->VendorID);
 	if (r != 1) {
+/*
+		upsdebugx(2, "%s: failed match of %s: %4x !~ %s",
+		    __func__, "VendorID", hd->VendorID, data->regex[0]);
+*/
+		upsdebugx(2, "%s: failed match of %s: %4x",
+		    __func__, "VendorID", hd->VendorID);
 		return r;
 	}
 
 	r = match_regex_hex(data->regex[1], hd->ProductID);
 	if (r != 1) {
+/*
+		upsdebugx(2, "%s: failed match of %s: %4x !~ %s",
+		    __func__, "ProductID", hd->ProductID, data->regex[1]);
+*/
+		upsdebugx(2, "%s: failed match of %s: %4x",
+		    __func__, "ProductID", hd->ProductID);
 		return r;
 	}
 
 	r = match_regex(data->regex[2], hd->Vendor);
 	if (r != 1) {
+/*
+		upsdebugx(2, "%s: failed match of %s: %s !~ %s",
+		    __func__, "Vendor", hd->Vendor, data->regex[2]);
+*/
+		upsdebugx(2, "%s: failed match of %s: %s",
+		    __func__, "Vendor", hd->Vendor);
 		return r;
 	}
 
 	r = match_regex(data->regex[3], hd->Product);
 	if (r != 1) {
+/*
+		upsdebugx(2, "%s: failed match of %s: %s !~ %s",
+		    __func__, "Product", hd->Product, data->regex[3]);
+*/
+		upsdebugx(2, "%s: failed match of %s: %s",
+		    __func__, "Product", hd->Product);
 		return r;
 	}
 
 	r = match_regex(data->regex[4], hd->Serial);
 	if (r != 1) {
+/*
+		upsdebugx(2, "%s: failed match of %s: %s !~ %s",
+		    __func__, "Serial", hd->Serial, data->regex[4]);
+*/
+		upsdebugx(2, "%s: failed match of %s: %s",
+		    __func__, "Serial", hd->Serial);
 		return r;
 	}
 
 	r = match_regex(data->regex[5], hd->Bus);
 	if (r != 1) {
+/*
+		upsdebugx(2, "%s: failed match of %s: %s !~ %s",
+		    __func__, "Bus", hd->Bus, data->regex[5]);
+*/
+		upsdebugx(2, "%s: failed match of %s: %s",
+		    __func__, "Bus", hd->Bus);
+		return r;
+	}
+
+	r = match_regex(data->regex[6], hd->Device);
+	if (r != 1) {
+/*
+		upsdebugx(2, "%s: failed match of %s: %s !~ %s",
+		    __func__, "Device", hd->Device, data->regex[6]);
+*/
+		upsdebugx(2, "%s: failed match of %s: %s",
+		    __func__, "Device", hd->Device);
 		return r;
 	}
 	return 1;
 }
 
 /* constructor: create a regular expression matcher. This matcher is
- * based on six regular expression strings in regex_array[0..5],
+ * based on seven regular expression strings in regex_array[0..6],
  * corresponding to: vendorid, productid, vendor, product, serial,
- * bus. Any of these strings can be NULL, which matches
+ * bus, device. Any of these strings can be NULL, which matches
  * everything. Cflags are as in regcomp(3). Typical values for cflags
  * are REG_ICASE (case insensitive matching) and REG_EXTENDED (use
  * extended regular expressions).  On success, return 0 and store the
  * matcher in *matcher. On error, return -1 with errno set, or return
- * i=1--6 to indicate that the regular expression regex_array[i-1] was
+ * i=1--7 to indicate that the regular expression regex_array[i-1] was
  * ill-formed (an error message can then be retrieved with
  * regerror(3)).
  */
@@ -345,7 +423,7 @@ int USBNewRegexMatcher(USBDeviceMatcher_t **matcher, char **regex, int cflags)
 	m->privdata = (void *)data;
 	m->next = NULL;
 
-	for (i=0; i<6; i++) {
+	for (i=0; i<7; i++) {
 		r = compile_regex(&data->regex[i], regex[i], cflags);
 		if (r == -2) {
 			r = i+1;
@@ -365,14 +443,14 @@ void USBFreeRegexMatcher(USBDeviceMatcher_t *matcher)
 {
 	int	i;
 	regex_matcher_data_t	*data;
-	
+
 	if (!matcher) {
 		return;
 	}
 
 	data = (regex_matcher_data_t *)matcher->privdata;
 
-	for (i = 0; i < 6; i++) {
+	for (i = 0; i < 7; i++) {
 		if (!data->regex[i]) {
 			continue;
 		}
@@ -384,3 +462,38 @@ void USBFreeRegexMatcher(USBDeviceMatcher_t *matcher)
 	free(data);
 	free(matcher);
 }
+
+void warn_if_bad_usb_port_filename(const char *fn) {
+	/* USB drivers ignore the 'port' setting - log a notice
+	 * if it is not "auto". Note: per se, ignoring the port
+	 * (or internally the device_path variable from main.c)
+	 * is currently not a bug and is actually documented at
+	 * docs/config-notes.txt; however it is not something
+	 * evident to users during troubleshooting a device.
+	 * Documentation and common practice recommend port=auto
+	 * so here we warn during driver start if it has some
+	 * other value and users might think it is honoured.
+	 */
+
+	if (!fn) {
+		upslogx(LOG_WARNING,
+			"WARNING: %s(): port argument was not "
+			"specified to the driver",
+			__func__);
+		return;
+	}
+
+	if (!strcmp(fn, "auto"))
+		return;
+
+	upslogx(LOG_WARNING,
+		"WARNING: %s(): port argument specified to\n"
+		"  the driver is \"%s\" but USB drivers do "
+		"not use it and rely on\n"
+		"  libusb walking all devices and matching "
+		"their identification metadata.\n"
+		"  NUT documentation recommends port=\"auto\" "
+		"for USB devices to avoid confusion.",
+		__func__, fn);
+	return;
+}
