@@ -19,10 +19,18 @@
  */
 
 #ifndef NUT_STDINT_H_SEEN
-#define NUT_STDINT_H_SEEN
+#define NUT_STDINT_H_SEEN 1
 
 #include "config.h"
 
+#ifndef __STDC_LIMIT_MACROS
+#define __STDC_LIMIT_MACROS 1
+#endif
+
+#ifndef __STDC_CONSTANT_MACROS
+#define __STDC_CONSTANT_MACROS 1
+#endif
+
 #if defined HAVE_INTTYPES_H
 #  include <inttypes.h>
 #endif
@@ -31,4 +39,33 @@
 #  include <stdint.h>
 #endif
 
+#if defined HAVE_LIMITS_H
+#  include <limits.h>
+#endif
+
+#ifndef SIZE_MAX
+#define SIZE_MAX ((size_t)(-1LL))
+#endif
+
+#ifndef SSIZE_MAX
+#define SSIZE_MAX ((ssize_t)(-1LL))
+#endif
+
+/* Printing format for size_t and ssize_t */
+#ifndef PRIsize
+# if defined(__MINGW32__)
+#  define PRIsize "u"
+# else
+#  define PRIsize "zu"
+# endif
+#endif
+
+#ifndef PRIssize
+# if defined(__MINGW32__)
+#  define PRIssize "d"
+# else
+#  define PRIssize "zd"
+# endif
+#endif /* format for size_t and ssize_t */
+
 #endif	/* NUT_STDINT_H_SEEN */
