@@ -1,5 +1,28 @@
-#ifndef PROTO_H
-#define PROTO_H
+/* proto.h - fill in the gaps about prototypes and definitions for portability
+
+   Copyright (C) 2001  Russell Kroll <rkroll@exploits.org>
+	2005	Arnaud Quette <arnaud.quette@free.fr>
+	2006	Peter Selinger <selinger@users.sourceforge.net>
+	2013	Emilien Kia <kiae.dev@gmail.com>
+	2020	Jim Klimov <jimklimov@gmail.com>
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+*/
+
+#ifndef NUT_PROTO_H_SEEN
+#define NUT_PROTO_H_SEEN 1
 
 #include "attribute.h"
 
@@ -91,4 +114,4 @@ int putenv(char *);
 /* *INDENT-ON* */
 #endif
 
-#endif /* PROTO_H */
+#endif /* NUT_PROTO_H_SEEN */
