@@ -3,6 +3,7 @@
    Copyright (C)
 	2001	Russell Kroll <rkroll@exploits.org>
 	2008	Arjen de Korte <adkorte-guest@alioth.debian.org>
+	2020	Jim Klimov <jimklimov@gmail.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,6 +20,9 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
+#ifndef NUT_CONF_H_SEEN
+#define NUT_CONF_H_SEEN 1
+
 #ifdef __cplusplus
 /* *INDENT-OFF* */
 extern "C" {
@@ -47,6 +51,7 @@ void delete_acls(void);
 void delete_access(void);
 
 extern	int	num_ups;
+extern	int	nut_debug_level_global;
 
 #ifdef __cplusplus
 /* *INDENT-OFF* */
@@ -54,3 +59,4 @@ extern	int	num_ups;
 /* *INDENT-ON* */
 #endif
 
+#endif /* NUT_CONF_H_SEEN */
