@@ -21,7 +21,7 @@
 #include <fcntl.h>
 #include <stdio.h>
 #include <unistd.h>
-#include <sys/un.h> 
+#include <sys/un.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -31,11 +31,11 @@
 
 	PCONF_CTX_t	sock_ctx;
 
-static void sock_arg(int numarg, char **arg)
+static void sock_arg(size_t numarg, char **arg)
 {
-	int	i;
+	size_t	i;
 
-	printf("numarg=%d : ", numarg);
+	printf("numarg=%zu : ", numarg);
 
 	for (i = 0; i < numarg; i++)
 		printf("[%s] ", arg[i]);
@@ -48,6 +48,8 @@ static int socket_connect(const char *sockfn)
 	int	ret, fd;
 	struct	sockaddr_un sa;
 
+	check_unix_socket_filename(sockfn);
+
 	memset(&sa, '\0', sizeof(sa));
 	sa.sun_family = AF_UNIX;
 	snprintf(sa.sun_path, sizeof(sa.sun_path), "%s", sockfn);
@@ -98,7 +100,7 @@ static void read_sock(int fd)
 	}
 
 	if (ret < 0) {
-		perror("read sockfd"); 
+		perror("read sockfd");
 		exit(EXIT_FAILURE);
 	}
 
