@@ -28,28 +28,34 @@
 use File::Find;
 use strict;
 
+
+my $TOP_SRCDIR = "..";
+if (defined $ENV{'TOP_SRCDIR'}) {
+    $TOP_SRCDIR = $ENV{'TOP_SRCDIR'};
+}
+
+my $TOP_BUILDDIR = "..";
+if (defined $ENV{'TOP_BUILDDIR'}) {
+    $TOP_BUILDDIR = $ENV{'TOP_BUILDDIR'};
+}
+
 # path to scan for USB_DEVICE pattern
-my $scanPath="../drivers";
+my $scanPath="$TOP_SRCDIR/drivers";
 
 # Hotplug output file
-my $outputHotplug="../scripts/hotplug/libhid.usermap";
+my $outputHotplug="$TOP_BUILDDIR/scripts/hotplug/libhid.usermap";
 
 # udev output file
-my $outputUdev="../scripts/udev/nut-usbups.rules.in";
+my $outputUdev="$TOP_BUILDDIR/scripts/udev/nut-usbups.rules.in";
 
 # BSD devd output file
-my $output_devd="../scripts/devd/nut-usb.conf.in";
+my $output_devd="$TOP_BUILDDIR/scripts/devd/nut-usb.conf.in";
 
 # UPower output file
-my $outputUPower="../scripts/upower/95-upower-hid.rules";
-
-# tmp output, to allow generating the ENV{UPOWER_VENDOR} header list
-my $tmpOutputUPower;
-# mfr header flag
-my $upowerMfrHeaderDone = 0;
+my $outputUPower="$TOP_BUILDDIR/scripts/upower/95-upower-hid.hwdb";
 
 # NUT device scanner - C header
-my $outputDevScanner = "./nut-scanner/nutscan-usb.h";
+my $outputDevScanner = "$TOP_BUILDDIR/tools/nut-scanner/nutscan-usb.h";
 
 my $GPL_header = "\
  *  Copyright (C) 2011 - Arnaud Quette <arnaud.quette\@free.fr>\
@@ -68,7 +74,7 @@ my $GPL_header = "\
  *  along with this program; if not, write to the Free Software\
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA";
 
-# array of products indexed by vendorID 
+# array of products indexed by vendorID
 my %vendor;
 
 # contain for each vendor, its name (and...)
@@ -76,7 +82,7 @@ my %vendorName;
 
 ################# MAIN #################
 
-find(\&find_usbdevs,$scanPath);
+find({wanted=>\&find_usbdevs, preprocess=>sub{sort @_}}, $scanPath);
 &gen_usb_files;
 
 ################# SUB METHOD #################
@@ -97,10 +103,10 @@ sub gen_usb_files
 	# Udev file header
 	open my $outUdev, ">$outputUdev" || die "error $outputUdev : $!";
 	print $outUdev '# This file is generated and installed by the Network UPS Tools package.'."\n\n";
-	print $outUdev 'ACTION!="add|change", GOTO="nut-usbups_rules_end"'."\n";
+	print $outUdev 'ACTION=="remove", GOTO="nut-usbups_rules_end"'."\n";
 	print $outUdev 'SUBSYSTEM=="usb_device", GOTO="nut-usbups_rules_real"'."\n";
 	print $outUdev 'SUBSYSTEM=="usb", GOTO="nut-usbups_rules_real"'."\n";
-	print $outUdev 'SUBSYSTEM!="usb", GOTO="nut-usbups_rules_end"'."\n\n";
+	print $outUdev 'GOTO="nut-usbups_rules_end"'."\n\n";
 	print $outUdev 'LABEL="nut-usbups_rules_real"'."\n";
 
 	open my $out_devd, ">$output_devd" || die "error $output_devd : $!";
@@ -111,20 +117,34 @@ sub gen_usb_files
 	open my $outputUPower, ">$outputUPower" || die "error $outputUPower : $!";
 	print $outputUPower '##############################################################################################################'."\n";
 	print $outputUPower '# Uninterruptible Power Supplies with USB HID interfaces'."\n#\n";
-	print $outputUPower '# This file was automatically generated by NUT:'."\n#".' https://github.com/networkupstools/nut/'."\n#\n";
-	print $outputUPower '# To keep up to date, monitor upstream NUT'."\n#".' https://github.com/networkupstools/nut/commits/master/scripts/upower/95-upower-hid.rules'."\n";
-	print $outputUPower "# or checkout the NUT repository and call 'tools/nut-usbinfo.pl'\n\n";
-	print $outputUPower '# newer hiddev are part of the usbmisc class'."\n".'SUBSYSTEM=="usbmisc", GOTO="up_hid_chkdev"'."\n";
-	print $outputUPower '# only support USB, else ignore'."\n".'SUBSYSTEM!="usb", GOTO="up_hid_end"'."\n\n";
-	print $outputUPower '# if usbraw device, ignore'."\n".'LABEL="up_hid_chkdev"'."\n".'KERNEL!="hiddev*", GOTO="up_hid_end"'."\n\n";
-	print $outputUPower '# if an interface, ignore'."\n".'ENV{DEVTYPE}=="usb_interface", GOTO="up_hid_end"'."\n\n";
+	print $outputUPower '# This file was automatically generated by NUT:'."\n";
+	print $outputUPower '# https://github.com/networkupstools/nut/'."\n#\n";
+	print $outputUPower '# To keep up to date, monitor upstream NUT'."\n";
+	print $outputUPower '# https://github.com/networkupstools/nut/commits/master/scripts/upower/95-upower-hid.hwdb'."\n";
+	print $outputUPower "# or checkout the NUT repository and call 'tools/nut-usbinfo.pl'\n";
 
 	# Device scanner header
 	open my $outputDevScanner, ">$outputDevScanner" || die "error $outputDevScanner : $!";
 	print $outputDevScanner '/* nutscan-usb'.$GPL_header."\n */\n\n";
 	print $outputDevScanner "#ifndef DEVSCAN_USB_H\n#define DEVSCAN_USB_H\n\n";
-	print $outputDevScanner "#include <usb.h>\n";
-	print $outputDevScanner "#include \"nut_stdint.h\"\t/* for uint16_t */\n\n";
+	print $outputDevScanner "#include \"nut_stdint.h\"\t/* for uint16_t etc. */\n\n";
+	print $outputDevScanner "#include <limits.h>\t/* for PATH_MAX in usb.h etc. */\n\n";
+	print $outputDevScanner "#include <sys/param.h>\t/* for MAXPATHLEN etc. */\n\n";
+	print $outputDevScanner "/* libusb header file */\n";
+	print $outputDevScanner "#if (!WITH_LIBUSB_1_0) && (!WITH_LIBUSB_0_1)\n";
+	print $outputDevScanner "#error \"configure script error: Neither WITH_LIBUSB_1_0 nor WITH_LIBUSB_0_1 is set\"\n";
+	print $outputDevScanner "#endif\n\n";
+	print $outputDevScanner "#if (WITH_LIBUSB_1_0) && (WITH_LIBUSB_0_1)\n";
+	print $outputDevScanner "#error \"configure script error: Both WITH_LIBUSB_1_0 and WITH_LIBUSB_0_1 are set\"\n";
+	print $outputDevScanner "#endif\n\n";
+	print $outputDevScanner "#if WITH_LIBUSB_1_0\n";
+	print $outputDevScanner " #include <libusb.h>\n";
+	print $outputDevScanner "#endif\n";
+	print $outputDevScanner "#if WITH_LIBUSB_0_1\n";
+	print $outputDevScanner " #include <usb.h>\n";
+	print $outputDevScanner " /* simple remap to avoid bloating structures */\n";
+	print $outputDevScanner " typedef usb_dev_handle libusb_device_handle;\n";
+	print $outputDevScanner "#endif\n";
 	# vid, pid, driver
 	print $outputDevScanner "typedef struct {\n\tuint16_t\tvendorID;\n\tuint16_t\tproductID;\n\tchar*\tdriver_name;\n} usb_device_id_t;\n\n";
 	print $outputDevScanner "/* USB IDs device table */\nstatic usb_device_id_t usb_device_table[] = {\n\n";
@@ -149,7 +169,7 @@ sub gen_usb_files
 
 
 		# UPower vendor header flag
-		$upowerMfrHeaderDone = 0;
+		my $upowerVendorHasDevices = 0;
 
 		foreach my $productId (sort { lc $a cmp lc $b } keys %{$vendor{$vendorId}})
 		{
@@ -163,7 +183,7 @@ sub gen_usb_files
 			print $outUdev "ATTR{idVendor}==\"".removeHexPrefix($vendorId);
 			print $outUdev "\", ATTR{idProduct}==\"".removeHexPrefix($productId)."\",";
 			print $outUdev ' MODE="664", GROUP="@RUN_AS_GROUP@"'."\n";
-			
+
 			# devd device entry
 			print $out_devd "# ".$vendor{$vendorId}{$productId}{"comment"}.' - '.$vendor{$vendorId}{$productId}{"driver"}."\n";
 			print $out_devd "notify 100 {\n\tmatch \"system\"\t\t\"USB\";\n";
@@ -178,35 +198,32 @@ sub gen_usb_files
 			# UPower device entry (only for USB/HID devices!)
 			if ($vendor{$vendorId}{$productId}{"driver"} eq "usbhid-ups")
 			{
-				if ($upowerMfrHeaderDone == 0)
-				{
-					# UPower vendor header
+				if (!$upowerVendorHasDevices) {
 					if ($vendorName{$vendorId}) {
-						$tmpOutputUPower = $tmpOutputUPower."\n# ".$vendorName{$vendorId}."\n";
+						print $outputUPower "\n# ".$vendorName{$vendorId}."\n";
 					}
-					print $outputUPower "ATTRS{idVendor}==\"".removeHexPrefix($vendorId)."\", ENV{UPOWER_VENDOR}=\"".$vendorName{$vendorId}."\"\n";
-					$upowerMfrHeaderDone = 1;
+					$upowerVendorHasDevices = 1;
 				}
-				$tmpOutputUPower = $tmpOutputUPower."ATTRS{idVendor}==\"".removeHexPrefix($vendorId);
-				$tmpOutputUPower = $tmpOutputUPower."\", ATTRS{idProduct}==\"".removeHexPrefix($productId)."\",";
-				$tmpOutputUPower = $tmpOutputUPower.' ENV{UPOWER_BATTERY_TYPE}="ups"'."\n";
+				print $outputUPower "usb:v".uc(removeHexPrefix($vendorId))."p".uc(removeHexPrefix($productId))."*\n";
 			}
 
 			# Device scanner entry
 			print $outputDevScanner "\t{ ".$vendorId.', '.$productId.", \"".$vendor{$vendorId}{$productId}{"driver"}."\" },\n";
 		}
+
+		if ($upowerVendorHasDevices) {
+			print $outputUPower " UPOWER_BATTERY_TYPE=ups\n";
+			if ($vendorName{$vendorId}) {
+				print $outputUPower " UPOWER_VENDOR=".$vendorName{$vendorId}."\n";
+			}
+		}
+
 	}
 	# Udev footer
 	print $outUdev "\n".'LABEL="nut-usbups_rules_end"'."\n";
 
-	# UPower...
-	# ...flush device table
-	print $outputUPower $tmpOutputUPower;
-	# ...and print footer
-	print $outputUPower "\n".'LABEL="up_hid_end"'."\n";
-
 	# Device scanner footer
-	print $outputDevScanner "\t/* Terminating entry */\n\t{ -1, -1, NULL }\n};\n#endif /* DEVSCAN_USB_H */\n\n";
+	print $outputDevScanner "\n\t/* Terminating entry */\n\t{ 0, 0, NULL }\n};\n#endif /* DEVSCAN_USB_H */\n\n";
 }
 
 sub find_usbdevs
@@ -216,7 +233,7 @@ sub find_usbdevs
 
 	my $nameFile=$_;
 	my $lastComment="";
-	
+
 	open my $file,$nameFile or die "error open file $nameFile";
 	while(my $line=<$file>)
 	{
@@ -226,7 +243,7 @@ sub find_usbdevs
 			$lastComment=$1;
 		}
 
-		if($line =~/^\s*\{\s*USB_DEVICE\((.+)\,(.+)\)\s*/) # for example : { USB_DEVICE(MGE_VENDORID, 0x0001)... }
+		if($line =~/^\s*\{\s*USB_DEVICE\(([^)]+)\,([^)]+)\)\s*/) # for example : { USB_DEVICE(MGE_VENDORID, 0x0001)... }
 		{
 			my $VendorID=trim($1);
 			my $ProductID=trim($2);
@@ -243,7 +260,7 @@ sub find_usbdevs
 				{
 					# catch Vendor Name
 					if($data =~/\s*\/\*(.+)\*\/\s*$/)
-					{			
+					{
 						$VendorName=$1;
 					}
 					# catch VendorID
@@ -280,31 +297,19 @@ sub find_usbdevs
 			if($nameFile=~/(.+)-hid\.c/) {
 				$driver="usbhid-ups";
 			}
-			# FIXME: make a generic matching rule *.c => *
-			elsif ($nameFile eq "bcmxcp_usb.c") {
-				$driver="bcmxcp_usb";
-			}
-			elsif ($nameFile eq "tripplite_usb.c") {
-				$driver="tripplite_usb";
-			}
-			elsif ($nameFile eq "blazer_usb.c") {
-				$driver="blazer_usb";
-			}
-			elsif ($nameFile eq "richcomm_usb.c") {
-				$driver="richcomm_usb";
-			}
-			elsif ($nameFile eq "nutdrv_atcl_usb.c") {
-				$driver="nutdrv_atcl_usb";
-			}
-			elsif ($nameFile eq "riello_usb.c") {
-				$driver="riello_usb";
-			}
-			elsif ($nameFile eq "nutdrv_qx.c") {
-				$driver="nutdrv_qx";
+			# generic matching rule *.c => *
+			elsif ($nameFile =~ /(.+)\.c$/) {
+				$driver=$1;
 			}
 			else {
 				die "Unknown driver type: $nameFile";
 			}
+			if ($vendor{$VendorID}{$ProductID}{"driver"} && $ENV{"DEBUG"}) {
+				print STDERR "nut-usbinfo.pl: VendorID=$VendorID ProductID=$ProductID " .
+					"was already related to driver '" .
+					$vendor{$VendorID}{$ProductID}{"driver"} .
+					"' and changing to '$driver'\n";
+			}
 			$vendor{$VendorID}{$ProductID}{"driver"}=$driver;
 		}
 	}
